//*****************************************************************************
// File: error.h
// Description: Nirva local error classes and error codes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _ERROR_H
#define _ERROR_H

#include "../nvs/task.h"
#include <string>

// Local service error codes

#define NV_OK										0		// No error

// REQUEST CLASS errors
#define NV_LOCAL_UNKNOWN_RESULT						101		// The command has been sent to the server but not executed
#define NV_LOCAL_BAD_PARAMETER						102		// Bad function parameter
#define NV_LOCAL_CANNOT_GET_SERVER_CONNECTION		103		// Cannot get connection
#define NV_LOCAL_NETWORK_ERROR						104		// Network error
#define NV_LOCAL_BAD_SERVER							105		// The server is not a Nirva server
#define NV_LOCAL_SYNC_ERROR							106		// Synchronisation error between command and return queues
#define NV_LOCAL_SERVER_COMMAND_ERROR				107		// Error in one of the commands sent to the server (more info in error for each command)
#define NV_LOCAL_COMMAND_NOT_SENT					108		// This command has not been sent to the server
#define NV_LOCAL_COMMAND_BUFFER_NOT_EMPTY			109		// The command buffer contains some un-sent commands
#define NV_LOCAL_UNKNOWN_COMMAND					110		// Unknown command
#define NV_LOCAL_UNKNOWN_CLASS						111		// Unknown class
#define NV_LOCAL_INVALID_COMMAND_INDEX				112		// Invalid command index
#define NV_LOCAL_HTTP_PROXY_AUTHENTICATION_ERROR	113		// Proxy http authentication error
#define NV_LOCAL_HTTP_AUTHENTICATION_ERROR			114		// Host http authentication error

// OBJECT CLASS errors
#define NV_LOCAL_OBJECT_NO_TYPE						101		// No object type
#define NV_LOCAL_OBJECT_UNKNOWN_TYPE				102		// Unknown type in the GetObject system command
#define NV_LOCAL_OBJECT_CANNOT_CREATE				103		// Cannot create local object
#define NV_LOCAL_OBJECT_NO_FILENAME					104		// No file name defined for a local file object
#define NV_LOCAL_OBJECT_CANNOT_POPULATE				105		// Cannot populate the local object
#define NV_LOCAL_OBJECT_CANNOT_GET					106		// Cannot get the local object
#define NV_LOCAL_OBJECT_CANNOT_GET_DATA				107		// Cannot get the local object data
#define NV_LOCAL_OBJECT_EVER_EXIST					108		// The local object ever exist
#define NV_LOCAL_INVALID_OBJECT_INDEX				109		// Invalid object index
#define NV_LOCAL_INVALID_OBJECT						110		// The object doesn't exist
#define NV_LOCAL_SAME_OBJECTS						111		// The objects are the same
#define NV_LOCAL_OBJECTS_NOT_COMPATIBLE				112		// The objects are not compatible
#define NV_LOCAL_OBJECT_CANNOT_COPY					113		// Cannot copy objects
#define NV_LOCAL_INVALID_STRINGLIST_INDEX			114		// Invalid string list index
#define NV_LOCAL_INVALID_INDSTRINGLIST_KEY			115		// Invalid indexed string list key
#define NV_LOCAL_TABLE_SEARCH_ERROR					116		// Error in search table
#define NV_LOCAL_TABLE_CANNOT_SAVE					117		// Error in save table
#define NV_LOCAL_TABLE_CANNOT_LOAD					118		// Error in load table
#define NV_LOCAL_TABLE_CANNOT_IMPORT				119		// Error in import table
#define NV_LOCAL_TABLE_CANNOT_EXPORT				120		// Error in export table
#define NV_LOCAL_TABLE_INVALID_CELL					121		// Invalid cell
#define NV_LOCAL_TABLE_CANNOT_SET_CELL_LINE			122		// Error in set cell line
#define NV_LOCAL_TABLE_CANNOT_INSERT_CELL_LINE		123		// Error in insert cell line
#define NV_LOCAL_TABLE_REMOVE_ROWS_ERROR			124		// Error in remove rows
#define NV_LOCAL_TABLE_CLEAR_ROWS_ERROR				125		// Error in clear rows
#define NV_LOCAL_TABLE_BAD_COLUMN_NAME				126		// Bad column name
#define NV_LOCAL_TABLE_MODIFY_COLUMNS_ERROR			127		// Error in modify columns
#define NV_LOCAL_FILE_CANNOT_CREATE					128		// Cannot create file for file object
#define NV_LOCAL_FILE_CANNOT_GET_SIZE				129		// Cannot get file size
#define NV_LOCAL_FILE_CANNOT_CLEAR					130		// Cannot clear file for file object
#define NV_LOCAL_FILE_CANNOT_APPEND					131		// Cannot append file to file object
#define NV_LOCAL_FILE_NO_FILE						132		// No file name in file object
#define NV_LOCAL_BINARY_CANNOT_APPEND				133		// Cannot append binary data
#define NV_LOCAL_BINARY_CANNOT_SAVE					134		// Cannot save binary data to file
#define NV_LOCAL_BINARY_CANNOT_LOAD					135		// Cannot load binary data from file
#define NV_LOCAL_TABLE_CANNOT_SELECT_FROM			136		// Error in table select from
#define NV_LOCAL_TABLE_CANNOT_SET_PRIMARY			137		// Cannot set the primary key
#define NV_LOCAL_TABLE_CANNOT_SET_ROW_DATA			138		// Cannot set row data
#define NV_LOCAL_OBJECT_BAD_TYPE					139		// Bad object type
#define NV_LOCAL_TABLE_CANNOT_JOIN					140		// Cannot join tables
#define NV_LOCAL_TABLE_CANNOT_ADD					141		// Cannot add tables
#define NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX			142		// Cannot get row index from primary value
#define NV_LOCAL_TABLE_CANNOT_ADD_RECORDS			143		// Cannot add records to the table
#define NV_LOCAL_FILE_CANNOT_COMPRESS				144		// Cannot compress local file
#define NV_LOCAL_FILE_CANNOT_DECOMPRESS				145		// Cannot decompress local file

/////////////////////////////////////////////////////////////////////////////
// NvError class
class NvError
{
public:
	int m_iErrorCode;
	std::string m_sErrorService;
	std::string m_sErrorClass;
	std::string m_sErrorInfo;
	std::string m_sErrorDesc;

	NvError();
	virtual ~NvError();

	void SetError(std::string ErrorService = "LOCAL", std::string ErrorClass = "", int ErrorCode = NV_OK, std::string ErrorInfo = "", std::string ErrorDesc = "");
	void SetLocalError(std::string ErrorClass = "", int ErrorCode = NV_OK, std::string ErrorInfo = "", const char *Language = NULL);
};




#endif //_ERROR_H
