//*****************************************************************************
// File: NvcxCtl.cpp
// Description: Nirva ActiveX connector
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#include "stdafx.h"
#include "nvcx.h"
#include "NvcxCtl.h"
#include "NvcxPpg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define RESULT_BUFFER_LENGTH	1024

IMPLEMENT_DYNCREATE(CNvcxCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CNvcxCtrl, COleControl)
	//{{AFX_MSG_MAP(CNvcxCtrl)
	// NOTE - ClassWizard will add and remove message map entries
	//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG_MAP
	ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CNvcxCtrl, COleControl)
	//{{AFX_DISPATCH_MAP(CNvcxCtrl)
	DISP_PROPERTY_NOTIFY(CNvcxCtrl, "NvResult", m_nvResult, OnNvResultChanged, VT_BSTR)
	DISP_FUNCTION(CNvcxCtrl, "OpenRequest", OpenRequest, VT_I2, VTS_BSTR)
	DISP_FUNCTION(CNvcxCtrl, "CloseRequest", CloseRequest, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CNvcxCtrl, "Command", Command, VT_I2, VTS_BSTR)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CNvcxCtrl, COleControl)
	//{{AFX_EVENT_MAP(CNvcxCtrl)
	// NOTE - ClassWizard will add and remove event map entries
	//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CNvcxCtrl, 1)
	PROPPAGEID(CNvcxPropPage::guid)
END_PROPPAGEIDS(CNvcxCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CNvcxCtrl, "NVCX.NvcxCtrl.1",
	0x8296d45d, 0xfe97, 0x4cff, 0x92, 0xea, 0x6d, 0xab, 0xd0, 0x80, 0x4e, 0x64)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CNvcxCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DNvcx =
		{ 0xfedabf23, 0x2786, 0x4fcf, { 0xb2, 0xf, 0x2b, 0x26, 0xdd, 0x16, 0x45, 0xd2 } };
const IID BASED_CODE IID_DNvcxEvents =
		{ 0xecd2c7f, 0xf49b, 0x4cd0, { 0xb2, 0x1f, 0xbc, 0x5c, 0x94, 0x44, 0x2f, 0x7f } };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwNvcxOleMisc =
	OLEMISC_INVISIBLEATRUNTIME |
	OLEMISC_SETCLIENTSITEFIRST |
	OLEMISC_INSIDEOUT |
	OLEMISC_CANTLINKINSIDE |
	OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CNvcxCtrl, IDS_NVCX, _dwNvcxOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::CNvcxCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CNvcxCtrl

BOOL CNvcxCtrl::CNvcxCtrlFactory::UpdateRegistry(BOOL bRegister)
{
	// TODO: Verify that your control follows apartment-model threading rules.
	// Refer to MFC TechNote 64 for more information.
	// If your control does not conform to the apartment-model rules, then
	// you must modify the code below, changing the 6th parameter from
	// afxRegApartmentThreading to 0.

	if (bRegister)
		return AfxOleRegisterControlClass(
			AfxGetInstanceHandle(),
			m_clsid,
			m_lpszProgID,
			IDS_NVCX,
			IDB_NVCX,
			afxRegApartmentThreading,
			_dwNvcxOleMisc,
			_tlid,
			_wVerMajor,
			_wVerMinor);
	else
		return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::CNvcxCtrl - Constructor

CNvcxCtrl::CNvcxCtrl()
{
	InitializeIIDs(&IID_DNvcx, &IID_DNvcxEvents);

	// TODO: Initialize your control's instance data here.
	ResultBufferSize = 0;
	SetBufferSize(0);
	m_nvResult = "";
	// Open the request with the default string
	hRequest = NvOpenRequest("");
}


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::~CNvcxCtrl - Destructor

CNvcxCtrl::~CNvcxCtrl()
{
	// Close the request if not yet made
	CloseRequest();
	if(ResultBufferSize)
		delete ResultBuffer;
}


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::OnDraw - Drawing function

void CNvcxCtrl::OnDraw(
			CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
	// TODO: Replace the following code with your own drawing code.
	pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
	pdc->Ellipse(rcBounds);
}


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::DoPropExchange - Persistence support

void CNvcxCtrl::DoPropExchange(CPropExchange* pPX)
{
	ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
	COleControl::DoPropExchange(pPX);

	// TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::GetControlFlags -
// Flags to customize MFC's implementation of ActiveX controls.
//
// For information on using these flags, please see MFC technical note
// #nnn, "Optimizing an ActiveX Control".
DWORD CNvcxCtrl::GetControlFlags()
{
	DWORD dwFlags = COleControl::GetControlFlags();


	// The control can activate without creating a window.
	// TODO: when writing the control's message handlers, avoid using
	//		the m_hWnd member variable without first checking that its
	//		value is non-NULL.
	dwFlags |= windowlessActivate;
	return dwFlags;
}


/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl::OnResetState - Reset control to default state

void CNvcxCtrl::OnResetState()
{
	COleControl::OnResetState();  // Resets defaults found in DoPropExchange

	// TODO: Reset any other control state here.
}

int CNvcxCtrl::SetBufferSize(int DataSize)
{
	if(DataSize < ResultBufferSize)
		return 1;

	if(ResultBufferSize == 0)
	{
		ResultBuffer = (char *)malloc(RESULT_BUFFER_LENGTH);
		if(ResultBuffer == NULL)
			return 0;
		ResultBufferSize = RESULT_BUFFER_LENGTH;
	}

	if(DataSize < ResultBufferSize)
		return 1;

	char *NewResultBuffer = (char *)realloc(ResultBuffer, DataSize+1);
	if(NewResultBuffer == NULL)
		return 0;

	ResultBuffer = NewResultBuffer;
	ResultBufferSize = DataSize+1;

	return 2;
}



/////////////////////////////////////////////////////////////////////////////
// CNvcxCtrl message handlers

void CNvcxCtrl::OnNvResultChanged() 
{
	// TODO: Add notification handler code

	SetModifiedFlag();
}

short CNvcxCtrl::OpenRequest(LPCTSTR ConnectString) 
{
	// First close the request
	CloseRequest();
	hRequest = NvOpenRequest(ConnectString);
	if(hRequest == NULL)
		return 0;
	return 1;
}

void CNvcxCtrl::CloseRequest() 
{
	// Close the Nirva request
	if(hRequest != NULL)
		NvCloseRequest(hRequest);

	hRequest = NULL;
}

short CNvcxCtrl::Command(LPCTSTR CommandString) 
{
	m_nvResult = "";

	if(hRequest == NULL)
		return 0;

	strcpy(ResultBuffer, "");
	int DataSize;
	if(!NvCommand(hRequest, CommandString, ResultBuffer, ResultBufferSize, &DataSize))
		return 0;
	int ResultSetBufferSize = SetBufferSize(DataSize);
	if(ResultSetBufferSize == 0)
		return 0;	// Error
	if(ResultSetBufferSize == 2)
	{
		// We got a reallocation so we must get the last output buffer
		NvCommand(hRequest, "NV_CMD=|LOCAL:GET_LAST_OUTPUT_BUFFER|", ResultBuffer, ResultBufferSize, &DataSize);
//		if(!NvCommand(hRequest, CommandString, ResultBuffer, ResultBufferSize, &DataSize))
//			return 0;
	}

	m_nvResult = ResultBuffer;

	return 1;
}
