// nvsext.h : header file
// Nirva external service header

#ifndef _NVSEXT_H
#define _NVSEXT_H



// Classes declared in this file
class NvServiceCommand;
class NvRendererInt;

#ifndef WIN32
#include <pthread.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#ifdef WIN32
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#include <ctype.h>
#endif


// Command source
#define NV_SOURCE_CLIENT				0		// nvc source
#define NV_SOURCE_BROWSER				1		// web browser source
#define NV_SOURCE_PROCEDURE				2		// procedure source
#define NV_SOURCE_SERVICE				3		// service source
#define NV_SOURCE_SERVER				4		// server source (used in init and close of services)




/////////////////////////////////////////////////////////////////////////////
// NvServiceCommand class
// Interface to services
class NvServiceCommand
{
public:
	virtual void SetError(const char *ErrorClass = "", int ErrorCode = 0, const char *ErrorInfo = "") = 0 ;
	virtual bool Command(const char *CommandString, char *Buffer = NULL, int BufferSize = 0, int *DataSize = NULL) = 0 ;
	virtual void GetSessionId(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetClass(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetCommand(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetInContainer(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetOutContainer(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetLanguage(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetMachineName(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetMachineUser(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual int GetNumParameters() = 0 ;
	virtual int GetSource() = 0 ;
	virtual bool ParameterExist(const char *ParameterName) = 0 ;
	virtual void GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void AddHttpHeader(const char *HeaderName, const char *HeaderValue) = 0 ;
	virtual void SetServiceSession(void *ServiceSession) = 0 ;
	virtual void* GetServiceSession() = 0 ;
	virtual bool IsCommand(const char *Class = NULL, const char *Command = NULL, int Source = -1) = 0 ;
};


// Renderer interface
class NvRendererInt
{
public:
	virtual const char* GetInputData() = 0 ;
	virtual unsigned long GetInputDataSize() = 0 ;
	virtual const char* GetInputEncoding() = 0 ;
	virtual const char* GetPageName() = 0 ;
	virtual char* AllocOutputBuffer(unsigned long Size) = 0 ;
	virtual char* GetOutputBuffer() = 0 ;
	virtual unsigned long GetOutputBufferSize() = 0 ;
	virtual const char* GetSessionId() = 0 ;
	virtual const char* GetAppName() = 0 ;
	virtual void SetReason(const char* Reason) = 0 ;
	virtual void SetContentType(const char* ContentType) = 0 ;
	virtual void SetHttpRet(const char* HttpRet) = 0 ;
	virtual void* GetExtraData(const char* DataName) = 0 ;
	virtual void FreeInputData() = 0 ;
};

#endif //_NVSEXT_H
