// ***************************************************************************************
// File: license.cpp
// Description: license management
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************



#include "nvs.h"
#ifdef AIX
#include <sys/utsname.h>
#endif
#ifdef LINUX
#ifndef MAC
#include <sys/ioctl.h>
#include <linux/if.h>
#endif
#endif

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;





/////////////////////////////////////////////////////////////////////////////
// NvLcLib

NvLcLib::NvLcLib()
{
	hLibLc = NULL;
	LcLibInit = NULL;
	LcLibFree = NULL;
	LcOpen = NULL;
	LcClose = NULL;
	LcSetLicenseFile = NULL;
	LcSetKey = NULL;
	LcCheckKey = NULL;
	LcRemoveKey = NULL;
	LcImport = NULL;
	LcExport = NULL;
	LcGetNumKey = NULL;
	LcGetKeyByIndex = NULL;
	LcGetKeyByName = NULL;
	LcLoad = NULL;
	LcSave = NULL;
	LcGetMachineKey = NULL;
	LcSetMachineKey = NULL;
	m_bAvailable = false;
}

NvLcLib::~NvLcLib()
{
	// Unload the license interface library if loaded
	FreeLcLib();
}


bool NvLcLib::LoadLcLib(const char *NirvaDir, bool UseLocking)
{
	if(UseLocking)
		NV_LOCKLOCK(&ProtectLc);
	if(hLibLc != NULL)
	{
		if(UseLocking)
			NV_UNLOCKLOCK(&ProtectLc);
		return true;
	}

	// Load the library here
	string LcLibPath = GetMain()->GetNirvaDir();
	AddDirSuffix(LcLibPath);
	LcLibPath += "Bin";
	AddDirSuffix(LcLibPath);
	LcLibPath += "nvlc";
#ifdef WIN32
#ifdef NV_DEBUG
	LcLibPath = GetMain()->GetNirvaDir();
	AddDirSuffix(LcLibPath);
	LcLibPath += "Bin";
	AddDirSuffix(LcLibPath);
	LcLibPath += "nvlcd";
#endif
	LcLibPath += ".dll";
#endif
#ifdef LINUX
	LcLibPath += ".so";
#endif
#ifdef HPUX
	LcLibPath += ".sl";
#endif
#ifdef HPUXI
	LcLibPath += ".so";
#endif
#ifdef SOLARIS
	LcLibPath += ".so";
#endif
#ifdef AIX
	LcLibPath += ".a";
#endif

	hLibLc = MyLoadLibrary(LcLibPath.c_str());
	if(hLibLc == NULL)
	{
		if(UseLocking)
			NV_UNLOCKLOCK(&ProtectLc);
		return false;
	}

	LcLibInit = (INITLCLIB)MyGetLibFunction(hLibLc, "InitLcLib");
	LcLibFree = (EXITLCLIB)MyGetLibFunction(hLibLc, "ExitLcLib");
	LcOpen = (LC_OPEN)MyGetLibFunction(hLibLc, "NvLcOpen");
	LcClose = (LC_CLOSE)MyGetLibFunction(hLibLc, "NvLcClose");
	LcSetLicenseFile = (LC_SETLICENSEFILE)MyGetLibFunction(hLibLc, "NvLcSetLicenseFile");
	LcSetKey = (LC_SETKEY)MyGetLibFunction(hLibLc, "NvLcSetKey");
	LcCheckKey = (LC_CHECKKEY)MyGetLibFunction(hLibLc, "NvLcCheckKey");
	LcRemoveKey = (LC_REMOVEKEY)MyGetLibFunction(hLibLc, "NvLcRemoveKey");
	LcImport = (LC_IMPORT)MyGetLibFunction(hLibLc, "NvLcImport");
	LcExport = (LC_EXPORT)MyGetLibFunction(hLibLc, "NvLcExport");
	LcGetNumKey = (LC_GETNUMKEY)MyGetLibFunction(hLibLc, "NvLcGetNumKey");
	LcGetKeyByIndex = (LC_GETKEYBYINDEX)MyGetLibFunction(hLibLc, "NvLcGetKeyByIndex");
	LcGetKeyByName = (LC_GETKEYBYNAME)MyGetLibFunction(hLibLc, "NvLcGetKeyByName");
	LcLoad = (LC_LOAD)MyGetLibFunction(hLibLc, "NvLcLoad");
	LcSave = (LC_SAVE)MyGetLibFunction(hLibLc, "NvLcSave");
	LcGetMachineKey = (LC_GETMACHINEKEY)MyGetLibFunction(hLibLc, "NvLcGetMachineKey");
	LcSetMachineKey = (LC_SETMACHINEKEY)MyGetLibFunction(hLibLc, "NvLcSetMachineKey");
	if((LcLibInit == NULL) || (LcLibFree == NULL) ||
		(LcOpen == NULL) || (LcClose == NULL) ||
		(LcSetLicenseFile == NULL) || (LcSetKey == NULL) ||
		(LcCheckKey == NULL) || (LcRemoveKey == NULL) ||
		(LcImport == NULL) || (LcExport == NULL) ||
		(LcGetNumKey == NULL) || (LcGetKeyByIndex == NULL) ||
		(LcGetKeyByName == NULL) || (LcLoad == NULL) ||
		(LcSave == NULL) || (LcGetMachineKey == NULL) || (LcSetMachineKey == NULL))
	{
		MyFreeLibrary(hLibLc);
		LcLibInit = NULL;
		LcLibFree = NULL;
		LcOpen = NULL;
		LcClose = NULL;
		LcSetLicenseFile = NULL;
		LcSetKey = NULL;
		LcCheckKey = NULL;
		LcRemoveKey = NULL;
		LcImport = NULL;
		LcExport = NULL;
		LcGetNumKey = NULL;
		LcGetKeyByIndex = NULL;
		LcGetKeyByName = NULL;
		LcLoad = NULL;
		LcSave = NULL;
		LcGetMachineKey = NULL;
		LcSetMachineKey = NULL;
		hLibLc = NULL;
		if(UseLocking)
			NV_UNLOCKLOCK(&ProtectLc);
		return false;
	}

	if((LcLibInit)(NirvaDir) == 0)
	{
		MyFreeLibrary(hLibLc);
		LcLibInit = NULL;
		LcLibFree = NULL;
		LcOpen = NULL;
		LcClose = NULL;
		LcSetLicenseFile = NULL;
		LcSetKey = NULL;
		LcCheckKey = NULL;
		LcRemoveKey = NULL;
		LcImport = NULL;
		LcExport = NULL;
		LcGetNumKey = NULL;
		LcGetKeyByIndex = NULL;
		LcGetKeyByName = NULL;
		LcLoad = NULL;
		LcSave = NULL;
		LcGetMachineKey = NULL;
		LcSetMachineKey = NULL;
		hLibLc = NULL;
		if(UseLocking)
			NV_UNLOCKLOCK(&ProtectLc);
		return false;
	}


	if(UseLocking)
		NV_UNLOCKLOCK(&ProtectLc);

	return true;
}



void NvLcLib::FreeLcLib(bool UseLocking)
{
	if(UseLocking)
		NV_LOCKLOCK(&ProtectLc);
	if(hLibLc == NULL)
	{
		if(UseLocking)
			NV_UNLOCKLOCK(&ProtectLc);
		return;
	}

	(LcLibFree)();
	MyFreeLibrary(hLibLc);
	LcLibInit = NULL;
	LcLibFree = NULL;
	LcOpen = NULL;
	LcClose = NULL;
	LcSetLicenseFile = NULL;
	LcSetKey = NULL;
	LcCheckKey = NULL;
	LcRemoveKey = NULL;
	LcImport = NULL;
	LcExport = NULL;
	LcGetNumKey = NULL;
	LcGetKeyByIndex = NULL;
	LcGetKeyByName = NULL;
	LcLoad = NULL;
	LcSave = NULL;
	LcGetMachineKey = NULL;
	LcSetMachineKey = NULL;
	hLibLc = NULL;
	if(UseLocking)
		NV_UNLOCKLOCK(&ProtectLc);
}



bool NvLcLib::IsLcLibLoaded(bool UseLocking)
{
	bool RetVal = true;
	if(UseLocking)
		NV_LOCKLOCK(&ProtectLc);
	if(hLibLc == NULL)
		RetVal = false;
	if(UseLocking)
		NV_UNLOCKLOCK(&ProtectLc);
	return RetVal;
}


/////////////////////////////////////////////////////////////////////////////
// NvLicenseKey

NvLicenseKey::NvLicenseKey()
{
	m_sService = "";
	m_sServiceId = "";
	m_sKey = "";
	m_sValue = "";
	m_sDate = "";
}

NvLicenseKey::~NvLicenseKey()
{
}



/////////////////////////////////////////////////////////////////////////////
// NvLicense

NvLicense::NvLicense()
{
	m_oLcIdent = NULL;
}

NvLicense::~NvLicense()
{
	NV_LOCKOBJECT(this);
	Close();
	NV_UNLOCKOBJECT(this);
}

bool NvLicense::Open()
{
	Close();
	m_oLcIdent = (GetMain()->LcInterface.LcOpen)();
	if(m_oLcIdent == NULL)
		return false;
	return true;
}

void NvLicense::Close()
{
	if(m_oLcIdent != NULL)
		(GetMain()->LcInterface.LcClose)(m_oLcIdent);
	m_oLcIdent = NULL;
}


void NvLicense::SetLicenseFile(string FileName)
{
	if(m_oLcIdent == NULL)
		return;
	NV_LOCKOBJECT(this);
	(GetMain()->LcInterface.LcSetLicenseFile)(m_oLcIdent, FileName.c_str());
	NV_UNLOCKOBJECT(this);
}



NvLicenseKey* NvLicense::GetKey(string Service, string ServiceId, string KeyName)
{
	if(m_oLcIdent == NULL)
		return NULL;

	NvlKey Key;

	if(!(GetMain()->LcInterface.LcGetKeyByName)(m_oLcIdent, Service.c_str(), ServiceId.c_str(), KeyName.c_str(), &Key))
		return NULL;

	NvLicenseKey* LicenseKey = new NvLicenseKey;
	LicenseKey->m_sDate = Key.m_sDate;
	LicenseKey->m_sKey = Key.m_sKey;
	LicenseKey->m_sService = Key.m_sService;
	LicenseKey->m_sServiceId = Key.m_sServiceId;
	LicenseKey->m_sValue = Key.m_sValue;
	return LicenseKey;
}


bool NvLicense::Load()
{
	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	bool RetVal = (GetMain()->LcInterface.LcLoad)(m_oLcIdent);
	
	NV_UNLOCKOBJECT(this);
	return RetVal;
}


bool NvLicense::Import(string FileName)
{
	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	bool RetVal = (GetMain()->LcInterface.LcImport)(m_oLcIdent, FileName.c_str());
	
	NV_UNLOCKOBJECT(this);
	return RetVal;
}


bool NvLicense::Export(string FileName, string Service)
{
	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	bool RetVal = (GetMain()->LcInterface.LcExport)(m_oLcIdent, FileName.c_str(), Service.c_str());
	
	NV_UNLOCKOBJECT(this);
	return RetVal;
}


bool NvLicense::Save()
{
	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	bool RetVal = (GetMain()->LcInterface.LcSave)(m_oLcIdent);
	
	NV_UNLOCKOBJECT(this);
	return RetVal;
}


bool NvLicense::SetKey(string MachineId, string Service, string ServiceId, string KeyName, string KeyValue, string KeyDate)
{
	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	bool RetVal = (GetMain()->LcInterface.LcSetKey)(m_oLcIdent, MachineId.c_str(), Service.c_str(), ServiceId.c_str(), KeyName.c_str(), KeyValue.c_str(), KeyDate.c_str());
	
	NV_UNLOCKOBJECT(this);
	return RetVal;
}


// Remove a key
bool NvLicense::RemoveKey(string Service, string ServiceId, string KeyName)
{
	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	bool RetVal = (GetMain()->LcInterface.LcRemoveKey)(m_oLcIdent, Service.c_str(), ServiceId.c_str(), KeyName.c_str());
	
	NV_UNLOCKOBJECT(this);
	return RetVal;
}

string NvLicense::GetMachineKey()
{
	if(m_oLcIdent == NULL)
		return "";

	char Buffer[50];
	Buffer[0] = '\0';
	NV_LOCKOBJECT(this);
	(GetMain()->LcInterface.LcGetMachineKey(m_oLcIdent, Buffer));
	NV_UNLOCKOBJECT(this);
	string MachineKey = Buffer;

	return MachineKey;
}

void NvLicense::SetMachineKey(string MachineKey)
{
	if(m_oLcIdent == NULL)
		return;
	NV_LOCKOBJECT(this);
	(GetMain()->LcInterface.LcSetMachineKey(m_oLcIdent, MachineKey.c_str()));
	NV_UNLOCKOBJECT(this);
}

bool NvLicense::CheckKey(string Service, string ServiceId, string KeyName, string &Value, bool VerifyDate)
{

	if(m_oLcIdent == NULL)
		return false;
	
	Value = "";

	char Buffer[256];
	Buffer[0] = '\0';

	NV_LOCKOBJECT(this);
	bool RetVal = (GetMain()->LcInterface.LcCheckKey)(m_oLcIdent, Service.c_str(), ServiceId.c_str(), KeyName.c_str(), Buffer, sizeof(Buffer), VerifyDate);
	NV_UNLOCKOBJECT(this);

	if(RetVal)
		Value = Buffer;

	return RetVal;
}



bool NvLicense::GetInfo(string Service, NvObjTable *LicenseInfo, NvObjString *MachineId)
{
	LicenseInfo->Clear();
	MachineId->SetValue(GetMachineKey());
	CleanString(Service);
	MakeUpper(Service);

	LicenseInfo->InsertColumn("SERVICE");
	LicenseInfo->InsertColumn("KEY");
	LicenseInfo->InsertColumn("VALUE");
	LicenseInfo->InsertColumn("DATE");

	if(m_oLcIdent == NULL)
		return false;

	NV_LOCKOBJECT(this);

	int NumKeys = (GetMain()->LcInterface.LcGetNumKey)(m_oLcIdent);
	int i,j;
	NvlKey Key;
	for(i=0, j=0; i<NumKeys; i++)
	{
		if(!(GetMain()->LcInterface.LcGetKeyByIndex)(m_oLcIdent, i, &Key))
			return false;
	
		string ServiceHere = Key.m_sService;

		if(Service.length() && Service != ServiceHere)
			continue;

		LicenseInfo->InsertRow();
		LicenseInfo->InsertCellLine(j, 0, Key.m_sService);
		LicenseInfo->InsertCellLine(j, 1, Key.m_sKey);
		LicenseInfo->InsertCellLine(j, 2, Key.m_sValue);
		LicenseInfo->InsertCellLine(j, 3, Key.m_sDate);
		j++;
	}


	NV_UNLOCKOBJECT(this);

	return true;
}





