// ***************************************************************************************
// File: license.h
// Description: license management
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _LICENSE_H
#define _LICENSE_H


// Classes declared in this file
class NvLcLib;
class NvLicenseKey;
class NvLicense;

#include "task.h"
#include <vector>
#include <string>
#include "../nvlc/elc.h"

#define LICENSE_FILE_IDENTIFIER		"NIRVA SERVER LICENSE FILE"


/////////////////////////////////////////////////////////////////////////////
// NvLcLib
// Interface to license manager. There is only one instance of this class in the main nvs class
class NvLcLib
{
public:

	NvMutex ProtectLc;							// Mutex for license interface protection
	HMODULE hLibLc;								// Handle to the nvlc dynamic library
	LC_OPEN LcOpen;								// nvlc library Open function pointer
	LC_CLOSE LcClose;							// nvlc library Close function pointer
	LC_SETLICENSEFILE LcSetLicenseFile;			// nvlc library SetLicenseFile function pointer
	LC_SETKEY LcSetKey;							// nvlc library SetKey function pointer
	LC_CHECKKEY LcCheckKey;						// nvlc library CheckKey function pointer
	LC_REMOVEKEY LcRemoveKey;					// nvlc library RemoveKey function pointer
	LC_IMPORT LcImport;							// nvlc library Import function pointer
	LC_EXPORT LcExport;							// nvlc library Export function pointer
	LC_GETNUMKEY LcGetNumKey;					// nvlc library GetNumKey function pointer
	LC_GETKEYBYINDEX LcGetKeyByIndex;			// nvlc library GetKeyByIndex function pointer
	LC_GETKEYBYNAME LcGetKeyByName;				// nvlc library GetKeyByName function pointer
	LC_LOAD LcLoad;								// nvlc library Load function pointer
	LC_SAVE LcSave;								// nvlc library Save function pointer
	LC_GETMACHINEKEY LcGetMachineKey;			// nvlc library GetMachineKey function pointer
	LC_SETMACHINEKEY LcSetMachineKey;			// nvlc library SetMachineKey function pointer
	INITLCLIB LcLibInit;						// nvlc library InitLcLib function pointer
	EXITLCLIB LcLibFree;						// nvlc library ExitLcLib function pointer
	bool m_bAvailable;


	NvLcLib();
	virtual ~NvLcLib();

	bool LoadLcLib(const char *NirvaDir, bool UseLocking = true);
	void FreeLcLib(bool UseLocking = true);
	bool IsLcLibLoaded(bool UseLocking = true);
};




/////////////////////////////////////////////////////////////////////////////
// NvLicenseKey
class NvLicenseKey
{
public:
	std::string m_sService;				// Service name
	std::string m_sServiceId;			// Service license identifier
	std::string m_sKey;					// License key name
	std::string m_sValue;				// License key value
	std::string m_sDate;				// License key expiration date
	
	NvLicenseKey();
	virtual ~NvLicenseKey();
};


class NvLicense : public NvObject
{
private:
	NVLCIDENT m_oLcIdent;				// License library object

public:

	NvLicense();
	virtual ~NvLicense();
	bool Open();
	void Close();


public:
	void SetLicenseFile(std::string FileName);
	bool SetKey(std::string MachineId, std::string Service, std::string ServiceId, std::string KeyName, std::string KeyValue = "", std::string KeyDate = "");
	bool CheckKey(std::string Service, std::string ServiceId, std::string KeyName, std::string &Value, bool VerifyDate = true);
	bool RemoveKey(std::string Service, std::string ServiceId, std::string KeyName);
	bool Import(std::string FileName);
	bool Export(std::string FileName, std::string Service = "");
	bool GetInfo(std::string Service, NvObjTable *LicenseInfo, NvObjString *MachineId);
	NvLicenseKey* GetKey(std::string Service, std::string ServiceId, std::string KeyName);
	bool Load();
	bool Save();
	std::string GetMachineKey();
	void SetMachineKey(std::string MachineKey);
};


#endif //_LICENSE_H
