// ***************************************************************************************
// File: util.h
// Description: utility functions
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _UTIL_H
#define _UTIL_H

#ifdef WIN32
#include <windows.h>
#else
#include <sys/param.h>
#endif

#include <string>
#include <vector>


#ifndef WIN32
#ifndef HMODULE
#define HMODULE void *
typedef int (*FARPROC)();
#endif
#endif

#ifdef WIN32
#define gmtime_r(tp, tm)    ((gmtime_s((tm), (tp)) == 0) ? (tp) : NULL)
#define localtime_r(tp, tm)    ((localtime_s((tm), (tp)) == 0) ? (tp) : NULL)
#endif /* _WIN32 */

#ifdef WIN32
#define NvLocale	std::string
#else
#define NvLocale	locale_t
#endif


int CopyFileEx(const char *SourceFile, const char *DestFile, NvInt64 Offset=0, NvInt64 BytesToCopy = -1);
int CopyFileEx(FILE *FileSource, FILE *FileDest, NvInt64 Offset=0, NvInt64 BytesToCopy = -1);
int AppendFileEx(const char *SourceFile, const char *DestFile, NvInt64 Offset=0, NvInt64 BytesToCopy = -1);
int AppendFileEx(FILE *FileSource, FILE *FileDest, NvInt64 Offset=0, NvInt64 BytesToCopy = -1);
bool GetTemporaryFile(std::string &FileName, std::string Dir, std::string Extension, std::string Prefix, std::string Suffix);
void RemoveFile(const char *FileName);
int FileExist(const char *FileName);
NvInt64 GetFileLength(const char* FileName);
NvInt64 GetFileLength(FILE *File);
int DirExist(const char *DirName);
bool DirList(const char *DirName, const char *Root, std::vector<std::string> *List);
bool IsDirEmpty(const char *DirName);
bool FileList(const char *DirName, const char *Root, std::vector<std::string> *List);
bool CreateDir(const char *DirName, bool LastDirOnly = true);
bool RemoveDir(const char *DirName, bool Self = true, const char *FileRoot = "*", const char *SubDirRoot = "*");
bool CopyDir(const char *SrcDir, const char *DstDir, bool WithSubDir = true);
bool GetDirLastModifiedCheckSum(std::string DirName, std::string& Result, bool WithSubDir = true);
bool GetNoCaseRealFileName(std::string& FileName);
int freadline(char *Line, int LineLength, FILE *stream);
size_t fwritelinemax(const char *data, size_t length, FILE* file, size_t maxlinelength);
int findstring(FILE *stream, const char *StringToFind);
int findstring(const char *Data, int DataSize, const char *StringToFind);
char *strstrnocase(const char *string, const char *strCharSet);
int strcmpnocase(const char *string1, const char *string2);
int strcmpnocase(std::string& string1, std::string& string2);
int strncmpnocase(const char *string1, const char *string2, int len);
void CleanString(char *String, bool NoSpace = false, bool ConcatSpace = true);
void CleanString(std::string& StringToClean, bool NoSpace = false, bool ConcatSpace = true);
bool CheckAsciiName(std::string& StringToCheck);
void AddDirSuffix(std::string& DirName);
void AddEndLine(std::string& String);
void RemoveDirSuffix(std::string& DirName);
bool CompareWorkDir(std::string Dir1, std::string Dir2);
void RemoveFirstAndEndBlanks(std::string& String);
void RemoveDoubleQuotes(std::string& String);
void RemoveSimpleQuotes(std::string& String);
void RemoveChar(std::string& String, char CharToRemove);
void SetDirName(std::string& DirName);
void GetFromProfile(const char *FileName, char *Output, unsigned int size, const char *Section, const char *Entry, const char *Default);
void RemoveFiles(const char *WorkDir, const char *Root);
std::string GetFirstFileDir(const char *DirName, const char *Root);
unsigned int GetAllocatedMemory();
unsigned int GetProcessAllocatedMemory();
int ExecPrgm(const char *CommandLine, int *RetCode, int TimeOut);
bool ArrangeDate(std::string& DateToArrange);
bool ArrangeHour(std::string& HourToArrange);
#ifndef WIN32
char *strlwr(char *string);	// These very usefull functions unfornatly doesn't exist on unix
char *strupr(char *string);	// So, we do them
#endif
HMODULE MyLoadLibrary(const char *LibraryPath, bool Global = false);
void MyFreeLibrary(HMODULE hModule);
FARPROC MyGetLibFunction(HMODULE hModule, const char *FunctionName);
std::string EncodeStringForXML(std::string StringToEncode, bool strict = false);
void SetAsciiFile(const char *FileName);
void EncodeUrlString(std::string& StringToEncode, bool bUnicode = true, bool bUrl = true);
void DecodeUrlString(std::string& StringToDecode, bool bUnicode = true, bool bUrl = true, bool bAutoDetectEncoding = false);
bool EncodeString(std::string& StringToEncode, bool bSrcUnicode, bool bDestUnicode);
bool IsUnicodeString(std::string& StringToCheck);
char *mystrptime (const char *buf, const char *format, struct tm *timeptr);
int axtoi(const char *hexStg);
std::string CreateGuid();
std::string MyGetFullPathName(const char* FileName);
std::string GetFileLastModifiedTime(const char* FileName);
std::string StripChars(std::string str, const std::string &chars);
std::string KeepChars(std::string str, const std::string &chars);
int SplitString(std::string str, std::string sep, std::vector<std::string> *Splitted);
int Mygettimeofday(struct timeval *tv, struct timezone *tz);
void SplitAddress(std::string &Address, int &Port);
void MyGetTime(char *Buffer, const struct tm *timeptr, bool bGMT);
std::string UTF16to8(const wchar_t * in);
std::wstring UTF8to16(const char * in);
char* iso88591_to_utf8(const char *str);
int myRand();
void myRandSeed(const void *buf, int num);
NvLocale MyGetLocale(int Category);
NvLocale MySetLocale(int Category, std::string Locale = "");
void MyRestoreLocale(int Category, NvLocale Locale);
void MyFreeLocale(NvLocale Locale);
std::string sha256(const std::string str);

/* -*- mode: c; c-file-style: "k&r" -*-

  strnatcmp.c -- Perform 'natural order' comparisons of strings in C.
  Copyright (C) 2000, 2004 by Martin Pool <mbp sourcefrog net>

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/


/* CUSTOMIZATION SECTION
 *
 * You can change this typedef, but must then also change the inline
 * functions in strnatcmp.c */
typedef wchar_t nat_char;

int strnatcmpnocase(const char *a, const char *b);

#endif //_UTIL_H
