


package com.allaire.cfx ;

import java.util.Vector ;


/**
 * a debugging implementation of the com.allaire.cfx.Query interface
 */
public class DebugQuery implements Query
{
    /** initialize a query with name and columns */
	public DebugQuery( String name, String[] columns ) throws IllegalArgumentException
	{
      // check for null arguments
      if ( name == null )
      {
         throw new IllegalArgumentException(
            "null passed as name argument to DebugQuery constructor" ) ;
      }
      if ( columns == null )
      {
         throw new IllegalArgumentException(
            "null passed as columns argument to DebugQuery constructor" ) ;
      }

      // initialize members
      m_strName = name ;
      setColumnsAsUpper( columns ) ;
   }


    /** initialize a query with name, columns, and data */
	public DebugQuery( String name, String[] columns, String[][] data )
	         throws IllegalArgumentException
	{
	   // call base constructor
	   this( name, columns ) ;

	   // initialize data (if any was passed to us)
	   if ( (data != null) && (data.length > 0) )
	   {
	      // verify that the column counts of columns and data match up
	      int nColumns = data[0].length ;
	      if ( nColumns != columns.length )
	      {
	         throw new IllegalArgumentException(
	            "Invalid arguments to constructor of DebugQuery: The size " +
	            "of the columns array is inconsistent with the number of " +
	            "columns passed in the data argument."  ) ;
	      }

	      // use the passed data to initialize our query
	      int nRows = data.length ;
	      for ( int i=0; i<nRows; i++ )
	         m_rows.addElement(data[i]) ;
	   }
	}


   // get the query name
	public String getName()
	{
	    return m_strName ;
	}


   // get the number of rows in the query
	public int getRowCount()
	{
	    return m_rows.size() ;
	}


	// get the index of a column given its' name
	public int getColumnIndex( String name )
	{
		// search the columns array for a match (return -1 if none found)
		for ( int i=0; i<m_columns.length; i++ )
		{
			if ( m_columns[i].equalsIgnoreCase(name) )
				return (i+1) ;
		}
		return -1 ;
	}


   // get the column array
	public String[] getColumns()
	{
	    return m_columns ;
	}


   // get a data element
	public String getData( int iRow, int iCol ) throws IndexOutOfBoundsException
	{
	   // validate arguments
      validateRowColIndex( iRow, iCol ) ;

      // get the requested data
      String[] row = (String[]) m_rows.elementAt(iRow-1) ;
	   return row[iCol-1] ;
	}


   // add a row (return the new rows index)
	public int addRow()
	{
	   m_rows.addElement( new String[m_columns.length] ) ;
	   return getRowCount() ;
	}


   // set a data element
	public void setData( int iRow, int iCol, String data ) throws IndexOutOfBoundsException
	{
	   // validate arguments
      validateRowColIndex( iRow, iCol ) ;

	   // convert a null data element to an empty string
	   String strData = data ;
	   if ( strData == null )
	      strData = "" ;

	   // set the data
      String[] row = (String[]) m_rows.elementAt(iRow-1) ;
	   row[iCol-1] = strData ;
	}


	// helper to set the columns member (converts to upper-case)
	private void setColumnsAsUpper( String[] columns )
	{
		m_columns = new String[columns.length] ;
		for ( int i=0; i<columns.length; i++ )
			m_columns[i] = columns[i].toUpperCase() ;
	}


	// helper to verify that the passed indexes are valid
	private void validateRowColIndex( int iRow, int iCol ) throws IndexOutOfBoundsException
	{
	   if ( (iRow < 1) || (iRow > getRowCount()) )
	   {
	      throw new IndexOutOfBoundsException(
	         "Invalid iRow index passed to getData or setData method " +
	         "(row indexes are 1-based): " +
	         "iRow=" + Integer.toString(iRow) + ", RowCount=" +
	         Integer.toString(getRowCount()) ) ;
	   }
	   if ( (iCol < 1) || (iCol > m_columns.length) )
	   {
	      throw new IndexOutOfBoundsException(
	         "Invalid iCol index passed to getData or setData method " +
	         "(column indexes are 1-based): " +
	         "iCol=" + Integer.toString(iCol) + ", Columns=" +
	         Integer.toString(m_columns.length) ) ;
	   }
	}

        //for future sorting column
        public void sort(int iCol, String sort_order){}
        public void sort(String sort_string){}

   // implementation data
   private String m_strName ;
   private String[] m_columns ;
   private Vector m_rows = new Vector() ;
}

