
package com.allaire.cfx ;

import java.util.Enumeration ;
import java.util.Hashtable ;
import java.util.Vector ;


//
// DebugRequest -- a debugging implementation of the com.allaire.cfx.Request interface
//
public class DebugRequest implements Request
{
   // Initialize a debug request with attributes
   public DebugRequest( Hashtable attributes ) 
      throws IllegalArgumentException
   {
      if ( attributes != null )
        initializeVariables(attributes, m_attributes) ;
   }
 
 
   // Initialize a debug request with attributes and a query
   public DebugRequest( Hashtable attributes, Query query ) 
      throws IllegalArgumentException
   {
      // call base constructor
      this( attributes ) ;
      
      // set our query member
      m_query = query ;      
   }
   
    
   // Initialize a debug request with attributes, settings, and a query
	public DebugRequest( Hashtable attributes, Query query, Hashtable settings )
	   throws IllegalArgumentException
	{
	   // call base constructor
	   this( attributes, query ) ;
	   
      // initialize variables passed to us
      if ( settings != null )
        initializeVariables(settings, m_settings ) ;    
   }
 
   
   // get a custom setting for this tag
	public String getSetting( String name ) 
	{
	   if ( name != null )
	   {
	      String strNameUpper = name.toUpperCase() ;
	      String strSetting = (String) m_settings.get(strNameUpper) ; 
	      if ( strSetting != null )
	         return strSetting ;
	      else
	         return "";	      
	   }
	   else
	   {
	      return "";
	   }	    
	}
	
	
	// get the query for this tag (returns null if there is none)
	public Query getQuery() 
	{
	    return m_query ;
	}


   // check to see whether an attribute exists
	public boolean attributeExists( String name ) 
	{
	   if ( name != null )
	   {
	      return m_attributes.containsKey(name) ;          
	   }
	   else
	   {
	      return false ;
	   }
	}
	
	
   // get the value of an attribute passed to this tag
	public String getAttribute( String name ) 
	{
	   if ( name != null )
	   {
	      String strNameUpper = name.toUpperCase() ;
	      String strAttribute = (String) m_attributes.get(strNameUpper) ; 
	      if ( strAttribute != null )
	         return strAttribute ;
	      else
	         return "";	      
	   }
	   else
	   {
	      return "";
	   }	    
	}

   
	// get a string attribute with a default
	public String getAttribute( String name, String def )
	{
		if ( attributeExists(name) )
			return getAttribute(name) ;
		else
			return def;
	}

	// get an integer attribute
	public int getIntAttribute( String name ) throws NumberFormatException
	{
		if ( attributeExists(name) )
		{
			return Integer.parseInt( getAttribute(name) ) ;	
		}
		else
		{
			return -1 ;
		}
	}


	// get an integer attribute with a default
	public int getIntAttribute( String name, int def )
	{
		if ( attributeExists(name) )
		{
			try
			{
				return Integer.parseInt( getAttribute(name) ) ;
			}
			catch( NumberFormatException e )
			{
				return def ;
			}
		}
		else
		{
			return def ;
		}
	}


   // get the list of attributes held by this tag
	public String[] getAttributeList() 
	{
	   // initialize attributes to return
	   int nAttribs = m_attributes.size() ;
	   String[] attribs = new String[nAttribs] ;
	   
	   // populate the attribute array
	   int i = 0 ;
	   Enumeration e = m_attributes.keys() ;
	   while ( e.hasMoreElements() ) 
	   {
	      attribs[i] = (String) e.nextElement() ;
	      i++ ;  
	   }	   
	   
	   // return the attribs
	   return attribs ;
	}


   // since this is a debug interface we are always in debug mode
	public boolean debug() 
	{
	    return true ;
	}

	
	// helper function to initialize the contents of a variable list
	// from another variable list, upper casing the variable names
	private void initializeVariables( Hashtable input, Hashtable target ) 
	{
	   Enumeration e = input.keys() ;
	   while ( e.hasMoreElements() )
	   {
	      String strKey = (String) e.nextElement() ;
	      target.put(strKey.toUpperCase(), input.get(strKey) ) ;     
	   } 
	}	


   // implementation data
   Hashtable m_attributes = new Hashtable() ;
   Hashtable m_settings = new Hashtable() ;
   Query m_query = null ;
}


