// ***************************************************************************************
// File: session1.cpp
// Description: session management
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: The session.cpp file was too big for some compilers (Solaris)
//           So it has been divided into session.cpp, session1.cpp,
//           session2.cpp and so on...
// ***************************************************************************************


#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;
	
// SYSTEM COMMAND SET_PARAMETER command
bool NvSession::CommandSetParameter(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Get parameter name
	string ParameterName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ParameterName);

	// Get parameter value
	string ParameterValue = CommandToExecute->Parameters.GetValue("VALUE");

	if(ParameterName.length() && (OriginalCommand != NULL))
	{
		// Change the parameter
		OriginalCommand->Parameters.SetValue(ParameterName, ParameterValue);

		// Change also the variable name
		m_oVariables.SetValue(ParameterName, ParameterValue);

		// If mOpenParameters is not null change it also
		if(mOpenParameters != NULL)
			(*mOpenParameters)[ParameterName] = ParameterValue;
	}

	return true;
}
	
	
// SYSTEM COMMAND REMOVE_PARAMETER command
bool NvSession::CommandRemoveParameter(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Get parameter name
	string ParameterName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ParameterName);

	if(ParameterName.length() && (OriginalCommand != NULL))
	{
		// Change the parameter
		OriginalCommand->Parameters.Remove(ParameterName);

		// Change also the variable name
		m_oVariables.Remove(ParameterName);

		// If mOpenParameters is not null change it also
		if(mOpenParameters != NULL)
			mOpenParameters->erase(ParameterName);
	}

	return true;
}
	
// SYSTEM COMMAND SET_OUTPUT_BUFFER command
bool NvSession::CommandSetOutputBuffer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from procedure or service
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:COMMAND:SET_OUTPUT_BUFFER");
		return false;
	}
	// Get parameter value
	string ParameterValue = CommandToExecute->Parameters.GetValue("VALUE");

	GetStringValue(CommandToExecute, ParameterValue.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}
	
// SYSTEM SYSTEM GET_MIME_TYPES command
bool NvSession::SystemGetMimeTypes(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	OutContainer->RemoveObject("MIMES");
	if(!OutContainer->CreateObject("MIMES", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "MIMES");
		return false;
	}

	NvObjIndStringList *Mimes = (NvObjIndStringList *)(OutContainer->GetObject("MIMES"));
	if(Mimes == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, "MIMES");
		return false;
	}

	GetMain()->GetMimeTable(Mimes);
	
	return true;
}
	
	
// SYSTEM SYSTEM SET_MIME_TYPE command
bool NvSession::SystemSetMimeType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get extension
	string Extension = CommandToExecute->Parameters.GetValue("EXTENSION");

	// Get mime type
	string Mime = CommandToExecute->Parameters.GetValue("MIME");

	GetMain()->SetMimeType(Extension, Mime);
	return true;
}
	
	
// SYSTEM SYSTEM GET_MIME_TYPE command
bool NvSession::SystemGetMimeType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get extension
	string Extension = CommandToExecute->Parameters.GetValue("EXTENSION");

	// Get mime type
	string Mime = GetMain()->GetMimeType(Extension);
	GetStringValue(CommandToExecute, Mime.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}
	
	
// SYSTEM SYSTEM REMOVE_MIME_TYPE command
bool NvSession::SystemRemoveMimeType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get extension
	string Extension = CommandToExecute->Parameters.GetValue("EXTENSION");

	// Remove mime type
	GetMain()->RemoveMimeType(Extension);

	return true;
}


	
	
// SYSTEM SYSTEM GET_ALIASES command
bool NvSession::SystemGetAliases(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	OutContainer->RemoveObject("ALIASES");
	if(!OutContainer->CreateObject("ALIASES", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "ALIASES");
		return false;
	}

	NvObjIndStringList *Aliases = (NvObjIndStringList *)(OutContainer->GetObject("ALIASES"));
	if(Aliases == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, "ALIASES");
		return false;
	}

	GetMain()->GetAliasTable(Aliases);
	
	return true;
}
	
	
// SYSTEM SYSTEM SET_ALIAS command
bool NvSession::SystemSetAlias(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get path
	string Path = CommandToExecute->Parameters.GetValue("PATH");

	// Get alias
	string Alias = CommandToExecute->Parameters.GetValue("ALIAS");

	GetMain()->SetAlias(Path, Alias);
	return true;
}
	
	
// SYSTEM SYSTEM GET_ALIAS command
bool NvSession::SystemGetAlias(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get path
	string Path = CommandToExecute->Parameters.GetValue("PATH");

	// Get alias
	string Alias = GetMain()->GetAlias(Path);
	GetStringValue(CommandToExecute, Alias.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}
	
	
// SYSTEM SYSTEM REMOVE_ALIAS command
bool NvSession::SystemRemoveAlias(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get path
	string Path = CommandToExecute->Parameters.GetValue("PATH");

	// Remove alias
	GetMain()->RemoveAlias(Path);

	return true;
}

	
	
// SYSTEM SYSTEM GET_VHOSTS command
bool NvSession::SystemGetVhosts(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	OutContainer->RemoveObject("VHOSTS");
	if(!OutContainer->CreateObject("VHOSTS", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "VHOSTS");
		return false;
	}

	NvObjTable *Vhosts = (NvObjTable *)(OutContainer->GetObject("VHOSTS"));
	if(Vhosts == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, "VHOSTS");
		return false;
	}

	GetMain()->VirtualHosts.GetList(Vhosts);
	
	return true;
}

	
	
// SYSTEM SYSTEM ADD_VHOST command
bool NvSession::SystemAddVhost(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get name
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	// Get description
	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	// Get Addresses
	string Addresses = CommandToExecute->Parameters.GetValue("ADDRESSES");
	// Get Mode
	string Mode = CommandToExecute->Parameters.GetValue("MODE");

	GetMain()->VirtualHosts.AddHost(Name.c_str(), Description.c_str(), Addresses.c_str(), Mode.c_str());
	
	return true;
}

	
	
// SYSTEM SYSTEM REMOVE_VHOST command
bool NvSession::SystemRemoveVhost(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get name
	string Name = CommandToExecute->Parameters.GetValue("NAME");

	GetMain()->VirtualHosts.RemoveHost(Name.c_str());
	
	return true;
}

	
	
// SYSTEM SYSTEM GET_VERSION command
bool NvSession::SystemGetVersion(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sVersion = GetMain()->m_sVersion;
	GetStringValue(CommandToExecute, sVersion.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}
	
	
// SYSTEM SYSTEM GET_VERSION_TYPE command
bool NvSession::SystemGetVersionType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sVersionType = GetMain()->m_sVersionType;
	GetStringValue(CommandToExecute, sVersionType.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}
	
	
// SYSTEM SYSTEM GET_NUM_PROCS command
bool NvSession::SystemGetNumProcs(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char Buff[20];
	sprintf(Buff, "%d", GetMain()->m_iNumProcs);
	GetStringValue(CommandToExecute, Buff, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}
	
	
// SYSTEM SYSTEM GET_NUM_EXCEPTIONS command
bool NvSession::SystemGetNumExceptions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char Buff[20];
	sprintf(Buff, "%d", GetMain()->GetNumExceptions());
	GetStringValue(CommandToExecute, Buff, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}

	
// SYSTEM SYSTEM GET_PLATFORM command
bool NvSession::SystemGetPlatform(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string Platform = "WIN32";
#ifdef WIN32
#ifdef X64
		Platform = "WIN64";
#endif
#endif
#ifdef AIX
	Platform = "AIX";
#endif
#ifdef LINUX
	Platform = "LINUX";
#ifdef X64
	Platform = "LINUX64";
#endif
#endif
#ifdef HPUX
	Platform = "HPUX";
#endif
#ifdef HPUXI
	Platform = "HPUXI";
#endif
#ifdef SOLARIS
	Platform = "SOLARIS";
#endif
	GetStringValue(CommandToExecute, Platform.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}
	
	
// SYSTEM SYSTEM GET_ENCODING command
bool NvSession::SystemGetEncoding(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sEncoding = "ISO-8859-1";
	if(GetMain()->IsUnicode())
		sEncoding = "UTF-8";
	GetStringValue(CommandToExecute, sEncoding.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}


	
	
// SYSTEM SYSTEM THREAD_LIST command
bool NvSession::SystemThreadList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	OutContainer->RemoveObject("THREAD_LIST");
	if(!OutContainer->CreateObject("THREAD_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "THREAD_LIST");
		return false;
	}

	NvObjTable *ThreadList = (NvObjTable *)(OutContainer->GetObject("THREAD_LIST"));
	if(ThreadList == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, "THREAD_LIST");
		return false;
	}

	GetMain()->Threads.List(ThreadList);
	
	return true;
}

// SYSTEM COMMAND SET_HTTP_HEADER command
bool NvSession::CommandSetHttpHeader(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(OriginalCommand->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:COMMAND:SET_HTTP_HEADER");
		return false;
	}
	
	// Get parameter name
	string ParameterName = CommandToExecute->Parameters.GetValue("HEADER");
	CleanString(ParameterName);

	// Get parameter value
	string ParameterValue = CommandToExecute->Parameters.GetValue("VALUE");

	// Get append parameter
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend, true);
	MakeUpper(sAppend);
	bool bAppend = false;
	if(sAppend == "YES")
		bAppend = true;

	if(ParameterName.length() && (OriginalCommand != NULL) && (OriginalCommand->m_iCommandSource == NV_SOURCE_BROWSER))
	{
		// Change the parameter
		if(bAppend)
		{
			string PreviousValue = OriginalCommand->HttpOutHeaders.GetValue(ParameterName);
			if(PreviousValue.length())
			{
				if(ParameterValue.length())
					ParameterValue = PreviousValue + "\n" + ParameterValue;
				else
					ParameterValue = PreviousValue;
			}
		}
		if(ParameterValue.length())
		{
			OriginalCommand->HttpOutHeaders.SetValue(ParameterName, ParameterValue);
		}
		else if(!bAppend)
			OriginalCommand->HttpOutHeaders.Remove(ParameterName);
	}

	return true;
}
	



// SYSTEM COMMAND GET_HTTP_HEADERS command
bool NvSession::CommandGetHttpHeaders(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command has meaning only from a browser original command
	if(OriginalCommand->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:COMMAND:GET_HTTP_HEADERS");
		return false;
	}
	
	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:GET_HTTP_HEADERS:EXEC");
		return false;
	}

	OutContainer->RemoveObject("HTTP_HEADERS");
	if(!OutContainer->CreateObject("HTTP_HEADERS", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "HTTP_HEADERS");
		return false;
	}

	NvObjIndStringList *HttpHeaders = (NvObjIndStringList *)(OutContainer->GetObject("HTTP_HEADERS"));
	if(HttpHeaders == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, "HTTP_HEADERS");
		return false;
	}

	HttpHeaders->CopyFrom(&(OriginalCommand->HttpInHeaders));

	return true;
}



// SYSTEM COMMAND SET_HTTP_RET_CODE command
bool NvSession::CommandSetHttpRetCode(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if((OriginalCommand == NULL) || (OriginalCommand->m_iCommandSource != NV_SOURCE_BROWSER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:COMMAND:SET_HTTP_RET_CODE");
		return false;
	}

	// Get value
	string Value = CommandToExecute->Parameters.GetValue("VALUE");
	OriginalCommand->m_sHttpRet = Value;

	return true;
}
	


// System TEST BROWSER command
bool NvSession::TestBrowser(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:TEST:TEST_BROWSER");
		return false;
	}

	// This command must block the standard xml output
	CommandToExecute->iOutputType = OUTPUT_BINARY;
	
	// get current time and date
	time_t CurrentTime;
	time(&CurrentTime);
	struct tm tCurrentTime;
	char sTime[200];
	if(GetMain()->GetLocalTime())
	{
		localtime_r(&CurrentTime, &tCurrentTime);
//		strftime(sTime, sizeof(sTime), "%a, %d %b %Y %H:%M:%S", &tCurrentTime);
		MyGetTime(sTime, &tCurrentTime, false);
	}
	else
	{
		gmtime_r(&CurrentTime, &tCurrentTime);
//		strftime(sTime, sizeof(sTime), "%a, %d %b %Y %H:%M:%S GMT", &tCurrentTime);
		MyGetTime(sTime, &tCurrentTime, true);
	}
	
	// and write a simple HTML page
	char buffer[2000];
	strcpy(buffer, "<html>\r\n");
	strcat(buffer, "<body>\r\n");
	strcat(buffer, "<h1>NIRVA Server");
	strcat(buffer, " : ");
	strcat(buffer, sTime);
	strcat(buffer, "</h1>\r\n");
	strcat(buffer, "<h1>");
	strcat(buffer, CommandToExecute->Parameters.GetValue("TEXT").c_str());
	strcat(buffer, "</h1>\r\n");
	strcat(buffer, "</body>\r\n");
	strcat(buffer, "</html>\r\n");

	char *Data = (char *)malloc(strlen(buffer));
	memcpy(Data, buffer, strlen(buffer));

	CommandToExecute->Data = Data;
	CommandToExecute->DataSize = (int)(strlen(buffer));

	CommandToExecute->HttpOutHeaders.SetValue("Cache-Control", "no-cache");
	CommandToExecute->HttpOutHeaders.SetValue("Pragma", "no-cache");
	CommandToExecute->HttpOutHeaders.SetValue("Expires", "Wed, 26 Feb 1997 08:21:57 GMT"); // old date

	CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "text/html" );

	return true;
}
	

// System TEST SET_LIST command
bool NvSession::TestSetList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SET_LIST");
	if(!OutContainer->CreateObject("SET_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SET_LIST");
		return false;
	}

	NvObjTable *Result = (NvObjTable *)(OutContainer->GetObject("SET_LIST"));

	string Set = CommandToExecute->Parameters.GetValue("SET");
	CleanString(Set);

	bool bService = false;
	bool bSystem = false;
	bool bApplication = true;

	NvService *Service = NULL;
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplication = false;
		if(ServiceName == "SYSTEM")
			bSystem = true;
		else
			bService = true;
	}
	if(bService && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvTestSetList *SetList = &(Application->TestSets);
	if(bSystem)
		SetList = &(GetMain()->TestSets);
	if(bService)
		SetList = &(Service->TestSets);

	SetList->GetInfo(Set.c_str(), Result);

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}
	

// System TEST LIST command
bool NvSession::TestList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("TEST_LIST");
	if(!OutContainer->CreateObject("TEST_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "TEST_LIST");
		return false;
	}

	NvObjTable *Result = (NvObjTable *)(OutContainer->GetObject("TEST_LIST"));

	string Set = CommandToExecute->Parameters.GetValue("SET");
	CleanString(Set);
	if(!Set.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TEST:LIST:SET");
		return false;
	}

	string Test = CommandToExecute->Parameters.GetValue("TEST");
	CleanString(Test);

	bool bService = false;
	bool bSystem = false;
	bool bApplication = true;

	NvService *Service = NULL;
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplication = false;
		if(ServiceName == "SYSTEM")
			bSystem = true;
		else
			bService = true;
	}
	if(bService && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvTestSetList *SetList = &(Application->TestSets);
	if(bSystem)
		SetList = &(GetMain()->TestSets);
	if(bService)
		SetList = &(Service->TestSets);

	SetList->GetTestInfo(Set.c_str(), Test.c_str(), Result);

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}
	
// System TEST CRASH command
bool NvSession::Crash(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// do a real creash
	char* x = NULL;
	*x = 42;

	return true;
}
	
	
// System TEST ZUT command
// used for testing some code
bool NvSession::Zut(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char bufrand[100];
	sprintf(bufrand, "tototartempionzutdezutetrzutencor%08x_%08x", myRand(), getpid());
	myRandSeed(bufrand, (int)strlen(bufrand));

	return true;
}

	
// System MISC NOP command
// This command does nothing but it can be used for refreshing the time out
// for executing a procedure or for creating a container
bool NvSession::Nop(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	return true;
}

	
// System MISC SLEEP command
bool NvSession::SleepSession(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get sleep time (milliseconds) from command parameters
	string SleepTime = CommandToExecute->Parameters.GetValue("TIME");
	CleanString(SleepTime);
	int iSleepTime = 0;
	if(SleepTime.length())
		iSleepTime = atoi(SleepTime.c_str());
	if(iSleepTime < 0)
		iSleepTime = 0;
	if(iSleepTime > 0)
		SLEEP(iSleepTime);

	return true;
}

	
// SYSTEM MISC EXEC command
bool NvSession::ExecFilter(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:MISC:EXEC");
		return false;
	}

	// The function uses only the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get path name of executable that should be in command parameters
	string PathName = CommandToExecute->Parameters.GetValue("PATH");
	CleanString(PathName, false, false);
	if(!PathName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MISC:EXEC:PATH");
		return false;
	}

	// Get optional lock name
	string LockName = CommandToExecute->Parameters.GetValue("LOCK");
	CleanString(LockName);

	// Get optional time out
	string TimeOutInCommand = CommandToExecute->Parameters.GetValue("TIME_OUT");
	CleanString(TimeOutInCommand);
	MakeUpper(TimeOutInCommand);
	int iTimeOutInCommand = -1;	// Infinite time out by default
	if(TimeOutInCommand.length())
		iTimeOutInCommand = atoi(TimeOutInCommand.c_str());
	if(iTimeOutInCommand < 0)
		iTimeOutInCommand = -1;


	// Get optional return code object name
	string sRetCodeObjectname = CommandToExecute->Parameters.GetValue("RET");
	CleanString(sRetCodeObjectname);
	if(!sRetCodeObjectname.length())
		sRetCodeObjectname = "RETURN_CODE"; 

	OutContainer->RemoveObject(sRetCodeObjectname.c_str());
	if(!OutContainer->CreateObject(sRetCodeObjectname.c_str(), NV_OBJTYPE_INTEGER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, sRetCodeObjectname);
		return false;
	}

	NvObj *ObjectRetCode = OutContainer->GetObject(sRetCodeObjectname.c_str());
	((NvObjInteger *)ObjectRetCode)->SetValue(0);


	// Get optional return code validation string
	string sRetCodeOk= CommandToExecute->Parameters.GetValue("RET_OK");
	CleanString(sRetCodeOk);

	// Now, we can replace the parameters of the path by the correct values
	string::size_type index;
	string RealPath = "";
	while(PathName.length())
	{
		index = PathName.find('%');
		if(index == string::npos)
		{
			RealPath += PathName;
			PathName = "";
			continue;
		}
		RealPath += PathName.substr(0, index);
		PathName = PathName.substr(index);
		if(PathName.length() == 1)
		{
			// This single end character has nothing to do here
			PathName = "";
			continue;
		}
		if(PathName[1] == '%')
		{
			// We have a double % sign so we must remove one
			RealPath += "%";
			if(PathName.length() == 2)
				PathName == "";
			else
				PathName = PathName.substr(2);
			continue;
		}
		// Now we have a real parameter value to replace
		// The parameter string should also end with a % sign

		PathName.erase(0, 1);

		string ParameterName = "";
		index = PathName.find('%');
		if(index == string::npos)
		{
			// This parameter doesn't exist in command
			CommandToExecute->Error.SetError("SYSTEM", "PRGM", NV_PRGM_BAD_PATH_PARAM, PathName);
			return false;
		}

		else
		{
			ParameterName = PathName.substr(0, index);
			PathName = PathName.substr(index+1);
		}

		// The real ParameterName is OK and cannot be empty at this stage
		if(!CommandToExecute->Parameters.KeyExist(ParameterName))
		{
			// This parameter doesn't exist in command
			CommandToExecute->Error.SetError("SYSTEM", "PRGM", NV_PRGM_BAD_PATH_PARAM, ParameterName);
			return false;
		}

		RealPath += CommandToExecute->Parameters.GetValue(ParameterName);
	}

	if(!RealPath.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MISC:EXEC:PATH");
		return false;
	}

	// If we have a lock name, we lock it
	// There is a special lock list for executables
	if(LockName.length())
		GetMain()->FilterLockList.Lock(LockName, m_sIdent);

	if(ActualSystemLogLevel)
	{
		string Message = "execute prgm";
		GetMain()->Logs.Write("System", Message.c_str(), RealPath.c_str(), NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
	}

	int RetCode = 0;
	int RetVal = ExecPrgm(RealPath.c_str(), &RetCode, iTimeOutInCommand);

	if(LockName.length())
		GetMain()->FilterLockList.Unlock(LockName, m_sIdent);

	if(RetVal == -1)
	{
		CommandToExecute->Error.SetError("SYSTEM", "PRGM", NV_PRGM_CANNOT_RUN_PRGM, RealPath);
		return false;
	}

	if(RetVal == -2)
	{
		CommandToExecute->Error.SetError("SYSTEM", "PRGM", NV_PRGM_TIME_OUT_PRGM, RealPath);
		return false;
	}

	// the program terminated normally (or continue execution if requested)

	// We first set the return code in the corresponding object
	((NvObjInteger *)ObjectRetCode)->SetValue(RetCode);

	if(iTimeOutInCommand == 0)
		return true;	// We don't wait for the end of program

	
	bool bRetCodeOk = false;
	if(!sRetCodeOk.length())
		bRetCodeOk = true;

	while(sRetCodeOk.length())
	{
		// We must check the return code
		// Several checks can be in the list
		// They are separated by a semicolumn
		string sRetCode = sRetCodeOk;
		string::size_type index = sRetCode.find(';');
		if(index != string::npos)
		{
			sRetCode = sRetCodeOk.substr(0, index);
			sRetCodeOk = sRetCodeOk.substr(index+1);
		}
		else
			sRetCodeOk = "";

		// An object name has been found
		CleanString(sRetCode);
		
		// It should not be empty
		if(!sRetCode.length())
			continue;

		int iCode = 0;
		if(sRetCode[0] == '>')
		{
			sRetCode = sRetCode.erase(0, 1);
			if(!sRetCode.length())
				continue;
			iCode = atoi(sRetCode.c_str());
			if(RetCode > iCode)
			{
				bRetCodeOk = true;
				break;
			}
		}
		else if(sRetCode[0] == '<')
		{
			sRetCode = sRetCode.erase(0, 1);
			if(!sRetCode.length())
				continue;
			iCode = atoi(sRetCode.c_str());
			if(RetCode < iCode)
			{
				bRetCodeOk = true;
				break;
			}
		}
		else
		{
			iCode = atoi(sRetCode.c_str());
			if(RetCode == iCode)
			{
				bRetCodeOk = true;
				break;
			}
		}
	}

	if(!bRetCodeOk)
	{
		CommandToExecute->Error.SetError("SYSTEM", "PRGM", NV_PRGM_ERROR_PRGM, RealPath);
		return false;
	}

	return true;
}


//SYSTEM MISC INFOEX
bool NvSession::InfoEx(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SYSTEM_INFO");
	if(!OutContainer->CreateObject("SYSTEM_INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SYSTEM_INFO");
		return false;
	}

	NvObjIndStringList *SystemInfo = (NvObjIndStringList *)(OutContainer->GetObject("SYSTEM_INFO"));

	if(!GetMain()->GetInfoEx(SystemInfo))
	{
		OutContainer->RemoveObject("SYSTEM_INFO");
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SYSTEM_CANNOT_GET_INFO);
		return false;
	}

	return true;
}


	

//SYSTEM MISC GET_CRASH
bool NvSession::GetCrash(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char Buf[20];
	if(GetMain()->b_Crash)
		strcpy(Buf, "YES");
	else
		strcpy(Buf, "NO");
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

	

//SYSTEM MISC HASH
bool NvSession::MiscHash(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sResult = "";
	if(CommandToExecute->Parameters.KeyExist("FILENAME"))
	{
		string sFileName = CommandToExecute->Parameters.GetValue("FILENAME");
		sResult = MD5_file(sFileName.c_str());
	}
	else
	{
		string sString = CommandToExecute->Parameters.GetValue("STRING");
		sResult = MD5_string(sString);
	}
	MakeUpper(sResult);
	GetStringValue(CommandToExecute, sResult.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

	

//SYSTEM MISC UNIQ
bool NvSession::MiscUniq(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sString = GetMain()->GetUniqString();
	GetStringValue(CommandToExecute, sString.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

	

//SYSTEM MISC SEQUENCE
bool NvSession::MiscSequence(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(sName, true);
	MakeUpper(sName);
	if(!sName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MISC:SEQUENCE:NAME");
		return false;
	}
	
	string sNum = CommandToExecute->Parameters.GetValue("NUM");
	CleanString(sNum, true);
	MakeUpper(sNum);
	int iNum = atoi(sNum.c_str());
	if(iNum <= 0)
		iNum = 1;

	string sReset = CommandToExecute->Parameters.GetValue("RESET");
	CleanString(sReset, true);
	MakeUpper(sReset);
	int iReset = atoi(sReset.c_str());
	if(iReset <= 0)
		iReset = 0;
	
	string sMax = CommandToExecute->Parameters.GetValue("MAX");
	CleanString(sMax, true);
	MakeUpper(sMax);
	int iMax = atoi(sMax.c_str());
	if(iMax <= 0)
		iMax = 2000000000;
	if(iMax > 2000000000)
		iMax = 2000000000;

	if(iNum > iMax)
		iNum = iMax;

	string sErrorNoExist = CommandToExecute->Parameters.GetValue("ERROR_IF_NOT_EXIST");
	CleanString(sErrorNoExist, true);
	MakeUpper(sErrorNoExist);
	bool bErrorNoExist = false;
	if(sErrorNoExist == "YES")
		bErrorNoExist = true;

	string sMinDigits = CommandToExecute->Parameters.GetValue("MIN_DIGITS");
	CleanString(sMinDigits, true);
	MakeUpper(sMinDigits);
	int iMinDigits = atoi(sMinDigits.c_str());
	if(iMinDigits <= 0)
		iMinDigits = 0;

	string sSystem = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystem, true);
	MakeUpper(sSystem);
	bool bSystemLevel = false;
	if(sSystem == "YES")
		bSystemLevel = true;

	// Get object result
	string ObjResult = CommandToExecute->Parameters.GetValue("RESULT");
	CleanString(ObjResult);
	if(!ObjResult.length())
		ObjResult = "RESULT";

	NvObjStringList* Result = (NvObjStringList *)(OutContainer->GetObject(ObjResult.c_str()));

	if(Result != NULL)
	{
		// The object exists so we remove it
		OutContainer->RemoveObject(ObjResult.c_str());
		Result = NULL;
	}
		
	if(!OutContainer->CreateObject(ObjResult.c_str(), NV_OBJTYPE_STRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}
	
	Result = (NvObjStringList *)(OutContainer->GetObject(ObjResult.c_str()));
	if(Result == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}

	NvRegistryContainer *RegistryContainer =  &(Application->AppRegistry);
	if(bSystemLevel)
		RegistryContainer =  &(GetMain()->SystemRegistry);	// This is the system registry

	string LockName = "NV_SEQUENCE_";
	if(bSystemLevel)
		LockName = "NV_SEQUENCE_SYS_";
	LockName += sName;
	bool RetVal = false;
	if(bSystemLevel)
		RetVal = GetMain()->LockList.Lock(LockName, m_sIdent, "", -1, 5);
	else
		RetVal = Application->LockList.Lock(LockName, m_sIdent, "", -1, 5);

	if(!RetVal)
	{
		OutContainer->RemoveObject(ObjResult.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "LOCK", NV_LOCK_CANNOT_LOCK, LockName);
		return false;
	}

	int iOldValue = 0;
	NvContainer TempContainer;
	NvObjInteger *SequenceObj = NULL;
	if(RegistryContainer->GetKey("system.sequences", &TempContainer, NV_CONTAINER_COPY_APPEND, sName.c_str()))
	{
		// An old value exists
		SequenceObj = (NvObjInteger*)(TempContainer.GetObject(sName.c_str()));
		if(SequenceObj != NULL)
			iOldValue = SequenceObj->GetValue();	// Should be always the case
		if(iOldValue <0)
			iOldValue = 0;
	}
	else
	{
		// The registry entry has not been found
		if(bErrorNoExist)
		{
			OutContainer->RemoveObject(ObjResult.c_str());
			if(bSystemLevel)
				GetMain()->LockList.Unlock(LockName, m_sIdent, true);
			else
				Application->LockList.Unlock(LockName, m_sIdent, true);
			CommandToExecute->Error.SetError("SYSTEM", "SEQUENCE", NV_SEQUENCE_NOT_EXIST, sName);
			return false;
		}
		else
		{
			TempContainer.CreateObject(sName.c_str(), NV_OBJTYPE_INTEGER);
			SequenceObj = (NvObjInteger*)(TempContainer.GetObject(sName.c_str()));
		}
	}

	if(iReset)
	{
		if((iMax > 0) && (iReset > iMax))
			iReset = 1;
		iOldValue = iReset-1;
	}
	if((iMax > 0) && (iOldValue >= iMax))
		iOldValue = 0;

	int iStartValue1 = iOldValue + 1;
	int iEndValue1 = iOldValue + iNum;
	int iStartValue2 = 0;
	int iEndValue2 = 0;
	int iNewValue = iEndValue1;
	if((iMax > 0) && (iNewValue > iMax))
	{
		iEndValue1 = iMax;
		iStartValue2 = 1;
		iEndValue2 = iNewValue-iMax;
		iNewValue = iEndValue2;
	}
	SequenceObj->SetValue(iNewValue);
	if(!RegistryContainer->SetKey("system.sequences", &TempContainer, NV_CONTAINER_COPY_APPEND, sName.c_str()))
	{
		OutContainer->RemoveObject(ObjResult.c_str());
		if(bSystemLevel)
			GetMain()->LockList.Unlock(LockName, m_sIdent, true);
		else
			Application->LockList.Unlock(LockName, m_sIdent, true);
		CommandToExecute->Error.SetError("SYSTEM", "SEQUENCE", NV_SEQUENCE_CANNOT_SAVE, sName);
		return false;
	}
	
	// We do not need any more the lock so we can release it
	if(bSystemLevel)
		GetMain()->LockList.Unlock(LockName, m_sIdent, true);
	else
		Application->LockList.Unlock(LockName, m_sIdent, true);

	// Now we can set the return values
	string Format = "%d";
	char Buf[50];
	if(iMinDigits)
	{
		Format = "%0";
		sprintf(Buf, "%d", iMinDigits);
		Format += Buf;
		Format += "d";
	}
	sprintf(Buf, "%d", iStartValue1);
	if((iMinDigits > 0) && ((int)strlen(Buf) < iMinDigits))
		sprintf(Buf, Format.c_str(), iStartValue1);
	string Values = Buf;
	int i;
	for(i=iStartValue1+1;i<=iEndValue1;i++)
	{
		sprintf(Buf, "%d", i);
		if((iMinDigits > 0) && ((int)strlen(Buf) < iMinDigits))
			sprintf(Buf, Format.c_str(), i);
		Values += ";";
		Values += Buf;
	}
	if(iStartValue2 > 0)
	{
		for(i=iStartValue2;i<=iEndValue2;i++)
		{
			sprintf(Buf, "%d", i);
			if((iMinDigits > 0) && ((int)strlen(Buf) < iMinDigits))
				sprintf(Buf, Format.c_str(), i);
			Values += ";";
			Values += Buf;
		}
	}
	Result->Add(Values, ";");

	return true;
}



	

//SYSTEM MISC REST
bool NvSession::MiscRest(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from browser
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:MISC:REST");
		return false;
	}

	// Normaly already done before but we do it again
	if(!CheckPermission(CommandToExecute, "SYSTEM", "REST_CONNECTOR"))
		return false;


	// This does nothing except executing procedures defined in NV_PROC
	// We just have to process the output
	string HttpOutputObjectName = "";
	bool bHttpOutputObjectTypeFile = false;
	string sOutputType = CommandToExecute->Parameters.GetValue("NV_OUTPUT_TYPE");
	CleanString(sOutputType);
	MakeUpper(sOutputType);
	if(sOutputType.find("FILE") == 0)
	{
		bHttpOutputObjectTypeFile = true;
		if(sOutputType.find("FILE:") == 0)
			HttpOutputObjectName = sOutputType.substr(5);
		if(!HttpOutputObjectName.length())
			HttpOutputObjectName = "OUTPUT_FILE";
	}
	else if(sOutputType.find("STRING") == 0)
	{
		if(sOutputType.find("STRING:") == 0)
			HttpOutputObjectName = sOutputType.substr(5);
		if(!HttpOutputObjectName.length())
			HttpOutputObjectName = "OUTPUT_STRING";
	}

	CommandToExecute->FreeData();

	if(HttpOutputObjectName.length())
	{
		// The function uses the output container
		if(!CheckOutputContainer(CommandToExecute))
			return false;

		// Now, the container has been found

		NvObj* Object = OutContainer->GetObject(HttpOutputObjectName.c_str());
		if(Object == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, HttpOutputObjectName);
			return false;
		}

		int iObjectType = Object->GetType();
		if(bHttpOutputObjectTypeFile && (iObjectType != NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_TYPE, HttpOutputObjectName);
			return false;
		}
		if(!bHttpOutputObjectTypeFile && (iObjectType != NV_OBJTYPE_STRING))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_TYPE, HttpOutputObjectName);
			return false;
		}

		if(iObjectType == NV_OBJTYPE_FILE)
		{
			NvInt64 FileSize = ((NvObjFile *)Object)->GetSize();
			bool bFileMessage = false;
			if(FileSize > (NvInt64)(GetMain()->GetHttpMaxMemBody()))	// for files grater than 1024 kbytes
				bFileMessage = true;
			
			if(bFileMessage)
			{
				string ObjectFileName = ((NvObjFile *)Object)->GetFilePathName();

				// We must work with a copy of the object because the session can be closed
				// before sending the file object to the client
				string DirHere = GetMain()->GetNirvaDir();
				DirHere += "Work";
				string FileNameHere;
				if(!GetTemporaryFile(FileNameHere, DirHere, "tmp", "", ""))
					return false;
				if(!CopyFileEx(ObjectFileName.c_str(), FileNameHere.c_str()))
				{
					RemoveFile(FileNameHere.c_str());
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, HttpOutputObjectName);
					return false;
				}
				CommandToExecute->DataFile = FileNameHere;
				CommandToExecute->bDeleteDataFile = true;
			}
			else
			{
				// Not a too big file so we use memory data
				if(!((NvObjFile *)Object)->GetData(&(CommandToExecute->Data), &(CommandToExecute->DataSize)))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, HttpOutputObjectName);
					return false;
				}
			}
		}
		else
		{
			// This is a string object, so we take the object data into memory
			NvObjString* RetObj = (NvObjString*)Object;
			size_t ObjSize = RetObj->m_sValue.length();
			if(ObjSize)
			{
				CommandToExecute->Data = (char*)malloc(ObjSize);
				if(CommandToExecute->Data != NULL)
				{
					memcpy(CommandToExecute->Data, RetObj->m_sValue.data(), ObjSize);
					CommandToExecute->DataSize = (unsigned long)ObjSize;
				}
			}
		}
	}


	return true;
}




//SYSTEM TIME GET
bool NvSession::GetTime(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sGmt = CommandToExecute->Parameters.GetValue("GMT");
	CleanString(sGmt);
	MakeUpper(sGmt);
	bool bGmt = false;
	if(sGmt == "YES")
		bGmt = true;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("TIME");
	if(!OutContainer->CreateObject("TIME", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "TIME");
		return false;
	}

	NvObjIndStringList *TimeObject = (NvObjIndStringList *)(OutContainer->GetObject("TIME"));

	time_t CurrentTime;
	time(&CurrentTime);
	struct tm tCurrentTime;
	char Buff[64];
	if(bGmt)
		gmtime_r(&CurrentTime, &tCurrentTime);
	else
		localtime_r(&CurrentTime, &tCurrentTime);

	sprintf(Buff, "%04d", tCurrentTime.tm_year + 1900);
	TimeObject->SetValue("YEAR", Buff);
	sprintf(Buff, "%02d", tCurrentTime.tm_mon + 1);
	TimeObject->SetValue("MONTH", Buff);
	sprintf(Buff, "%02d", tCurrentTime.tm_mday);
	TimeObject->SetValue("DAY", Buff);
	sprintf(Buff, "%02d", tCurrentTime.tm_hour);
	TimeObject->SetValue("HOUR", Buff);
	sprintf(Buff, "%02d", tCurrentTime.tm_min);
	TimeObject->SetValue("MIN", Buff);
	sprintf(Buff, "%02d", tCurrentTime.tm_sec);
	TimeObject->SetValue("SEC", Buff);

	return true;
}


//SYSTEM MAIL SEND
bool NvSession::SendMail(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "MAIL_SEND"))
		return false;

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;


	// Get the TO parameter
	string sTo = CommandToExecute->Parameters.GetValue("TO");
	CleanString(sTo);
	if(!sTo.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MAIL:SEND:TO");
		return false;
	}

	// Get the MAILHOST parameter
	string sMailHost = CommandToExecute->Parameters.GetValue("MAILHOST");
	CleanString(sMailHost);
	if(!sMailHost.length())	// If no mailhost given, use the default one
		sMailHost = GetMain()->GetDefaultMailServer();
	if(!sMailHost.length())
		sMailHost = "localhost";
	CommandToExecute->Parameters.SetValue("MAILHOST", sMailHost);


	// Get the FROM parameter
	string sFrom = CommandToExecute->Parameters.GetValue("FROM");
	CleanString(sFrom);
	if(!sFrom.length())
	{
		// We first try to get the USER parameter
		if(CommandToExecute->Parameters.KeyExist("USER"))
		{
			sFrom = CommandToExecute->Parameters.GetValue("USER");
			CleanString(sFrom);
			string FromHere = sFrom;
			MakeUpper(FromHere);
			if(FromHere == "NV_DEFAULT")
				sFrom = GetMain()->GetDefaultMailUser();
		}
		if(!sFrom.length())
		{
			// The user is the mailhost and the default user
			sFrom = GetMain()->GetDefaultMailUser();
		}
		if(!sFrom.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "MAIL", NV_MAIL_NO_SENDER);
			return false;
		}
		
		// We add the mailhost to the user name
		sFrom += "@";
		sFrom += sMailHost;
		CommandToExecute->Parameters.SetValue("FROM", sFrom);
	}
	
	// Get the AUTH flag
	string sAuth = CommandToExecute->Parameters.GetValue("AUTH");
	CleanString(sAuth);
	MakeUpper(sAuth);
	if(!sAuth.length())
		sAuth = "YES";
	if(sAuth == "YES")
	{
		CommandToExecute->Parameters.SetValue("AUTH", "YES");
		if(CommandToExecute->Parameters.KeyExist("USER"))
		{
			string AuthUser = CommandToExecute->Parameters.GetValue("USER");
			CleanString(AuthUser);
			string AuthUserHere = AuthUser;
			MakeUpper(AuthUserHere);
			if((AuthUserHere == "NV_DEFAULT") || (!AuthUser.length()))
			{
				// The USER parameter has been given blank so we use the default one
				AuthUser = GetMain()->GetDefaultMailUser();
				CommandToExecute->Parameters.SetValue("USER", AuthUser);
			}
		}
		if(!(CommandToExecute->Parameters.KeyExist("PASSWORD")))
		{
			string AuthPassword = GetMain()->GetDefaultMailPassword();
			CommandToExecute->Parameters.SetValue("PASSWORD", AuthPassword);
		}
	}

	// Try to execute the procedure but do nothing if it doesn't exist
	if(!ExecuteProcedure(CommandToExecute, "(perl:System/mail)", false, "", NULL, NULL, NULL, iEncoding))
		return false;

	return true;
}
	



// System XML SET_XML command
// This command put the input container or part of it into
// an XML file embended in a NvObjFile object
// The resulting object is named "XML" by default
// The OBJECTS parameter contains the name of objects to get separated by a semicolomn
// if OBJECTS is empty, no object is returned (except eventual subcontainer objects)
// if OBJECTS not exists,  all the objects are returned
// The SUBCONTAINERS parameter, if set to yes tells to get also all subcontainers objects
bool NvSession::XmlSetXml(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object names that may be in parameters
	string ObjNames = CommandToExecute->Parameters.GetValue("OBJECTS");
	CleanString(ObjNames);

	// Get object result name
	string ObjResult = CommandToExecute->Parameters.GetValue("XMLOBJ");
	CleanString(ObjResult);
	if(!ObjResult.length())
		ObjResult = "XML";

	// Get output type
	bool bWithBinaryOutput = false;
	string sWithBinaryOutput = CommandToExecute->Parameters.GetValue("OUTPUT");
	CleanString(sWithBinaryOutput);
	MakeUpper(sWithBinaryOutput);
	if(sWithBinaryOutput == "BINARY")
		bWithBinaryOutput = true;

	// Get subcontainer option
	bool bWithSubContainers = false;
	string sWithSubContainers = CommandToExecute->Parameters.GetValue("SUBCONTAINERS");
	CleanString(sWithSubContainers);
	MakeUpper(sWithSubContainers);
	if(sWithSubContainers == "YES")
		bWithSubContainers = true;

	// Get with data option option
	bool bWithData = true;
	bool bWithFileData = false;
	string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
	CleanString(sWithData);
	MakeUpper(sWithData);
	if(sWithData == "NO")
		bWithData = false;
	if(sWithData == "ALL")
	{
		bWithData = true;
		bWithFileData = true;
	}


	// Get http headers option
	bool bWithHttpHeaders = false;
	string sWithHttpHeaders = CommandToExecute->Parameters.GetValue("HEADERS");
	CleanString(sWithHttpHeaders);
	MakeUpper(sWithHttpHeaders);
	if(sWithHttpHeaders == "YES")
		bWithHttpHeaders = true;

	// Get variables option
	bool bWithVariables = false;
	string sWithVariables = CommandToExecute->Parameters.GetValue("VARIABLES");
	CleanString(sWithVariables);
	MakeUpper(sWithVariables);
	if(sWithVariables == "YES")
		bWithVariables = true;

	bool bUnicode = false;
	string sEncoding = CommandToExecute->Parameters.GetValue("ENCODING");
	CleanString(sEncoding);
	MakeUpper(sEncoding);
	if((sEncoding.length()) && (sEncoding != "ISO-8859-1") && (sEncoding != "UTF-8"))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_UNSUPPORTED_ENCODING, sEncoding);
		return false;
	}
	if(!sEncoding.length())
	{
		if(GetMain()->IsUnicode())
			sEncoding = "UTF-8";
		else
			sEncoding = "ISO-8859-1";
	}
	if(sEncoding == "UTF-8")
		bUnicode = true;
	NvXml XmlObjectForCoding;
	
	// Get strict coding option
	bool bStrict = false;
	string sStrict = CommandToExecute->Parameters.GetValue("STRICT");
	CleanString(sStrict, true);
	MakeUpper(sStrict);
	if(sStrict == "YES")
		bStrict = true;
	XmlObjectForCoding.bStrict = bStrict;


	// Get simple coding output option
	int iXmlCoding = XML_MODEL_NORMAL;
	string sModel = CommandToExecute->Parameters.GetValue("MODEL");
	CleanString(sModel, true);
	MakeUpper(sModel);
	string sSimpleCoding = CommandToExecute->Parameters.GetValue("SIMPLE");
	CleanString(sSimpleCoding, true);
	MakeUpper(sSimpleCoding);
	if(sModel.length())
	{
		if(sModel == "SIMPLE")
			iXmlCoding = XML_MODEL_SIMPLE;
		if(sModel == "NORMAL")
			iXmlCoding = XML_MODEL_NORMAL;
		if(sModel == "COMPACT")
			iXmlCoding = XML_MODEL_COMPACT;
		if(sModel == "TINY")
			iXmlCoding = XML_MODEL_TINY;
	}
	else
	{
		if(sSimpleCoding == "YES")
			iXmlCoding = XML_MODEL_SIMPLE;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Now, the containers has been found


	NvObjStringList ObjectsToGet;
	if(CommandToExecute->Parameters.KeyExist("OBJECTS"))
	{
		while(ObjNames.length())
		{
			// Several objects can be in the list
			// They are separated by a semicolumn
			string ObjectName = ObjNames;
			string::size_type index = ObjectName.find(';');
			if(index != string::npos)
			{
				ObjectName = ObjNames.substr(0, index);
				ObjNames = ObjNames.substr(index+1);
			}
			else
				ObjNames = "";

			// An object name has been found
			CleanString(ObjectName);
			
			// It should not be empty
			if(!ObjectName.length())
				continue;

			ObjectsToGet.Add(ObjectName);
		}
	}
	else
	{
		map<string,void *>::iterator pos;
		for(pos = InContainer->m_mObjects.begin(); pos != InContainer->m_mObjects.end(); ++pos)
		{
			NvObj *Object = (NvObj*)(pos->second);
			ObjectsToGet.Add(Object->GetName());
		}
	}

	NvObj *ObjectResult = OutContainer->GetObject(ObjResult.c_str());

	if(ObjectResult != NULL)
	{
		if(bWithBinaryOutput)
		{
			// We must remove the object
			OutContainer->RemoveObject(ObjResult.c_str());
			ObjectResult = NULL;
		}
		else
		{
			// We must remove the object if it's not a file object or if it doesn't have file name
			if(ObjectResult->GetType() == NV_OBJTYPE_FILE)
			{
				if(!(((NvObjFile*)ObjectResult)->GetFileName().length()))
				{
					OutContainer->RemoveObject(ObjResult.c_str());
					ObjectResult = NULL;
				}
			}
			else
			{
				OutContainer->RemoveObject(ObjResult.c_str());
				ObjectResult = NULL;
			}
		}
	}

	if(ObjectResult == NULL)
	{
		int ObjectType = NV_OBJTYPE_FILE;
		if(bWithBinaryOutput)
			ObjectType = NV_OBJTYPE_BINARY;
		if(!OutContainer->CreateObject(ObjResult.c_str(), ObjectType))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
			return false;
		}
		ObjectResult = OutContainer->GetObject(ObjResult.c_str());

		if(!bWithBinaryOutput)
		{
			string sWorkDir = "";
			string sPrefix = "";
			sWorkDir = Application->GetWorkDir();
			sPrefix = Application->GetAppNameForFiles();
			if(!((NvObjFile*)ObjectResult)->SetAutoFileName(sWorkDir, "xml", sPrefix))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, Application->GetWorkDir());
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
		}
	}

	if(!bWithBinaryOutput)
	{
		if(!((NvObjFile*)ObjectResult)->Open("w"))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_OPEN_FILE, ObjectResult->GetName());
			OutContainer->RemoveObject(ObjResult.c_str());
			return false;
		}
	}

	string StringResult;

	int i;
	int NumTabs = 1;
	
	string XmlData = "<?xml version=\"1.0\"";
	ReserveString(XmlData, 1000);
	if(sEncoding.length())
	{
		XmlData +=  " encoding=\"";
		XmlData += sEncoding;
		XmlData += "\"";
	}
	XmlData += " ?>\n";
	XmlData += "<";
	
	string NsPrefix = "";
	string NsHref = "";
	if(CommandToExecute->Parameters.KeyExist("NSREF"))
	{
		NsPrefix = CommandToExecute->Parameters.GetValue("NSPREFIX");
		CleanString(NsPrefix, true);
		NsHref = CommandToExecute->Parameters.GetValue("NSREF");
		CleanString(NsHref, true);
		if(!NsHref.length())
		{
			NsPrefix = "";
			NsHref = "";
		}
	}

	if(NsHref.length() && NsPrefix.length())
	{
		XmlData += NsPrefix;
		XmlData += ":";
	}
	if(iXmlCoding == XML_MODEL_SIMPLE)
		XmlData += "nirva";
	else if(iXmlCoding == XML_MODEL_COMPACT)
		XmlData += "nirva";
	else if(iXmlCoding == XML_MODEL_TINY)
		XmlData += "nirva";
	else
		XmlData += "NIRVA";
	if(NsHref.length())
	{
		XmlData += " xmlns";
		if(NsPrefix.length())
		{
			XmlData += ":";
			XmlData += NsPrefix;
		}
		XmlData += "=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(NsHref, bUnicode);
		XmlData += "\"";
	}
	if(NsPrefix.length())
		NsPrefix += ":";

	XmlData += " session=\"";
	XmlData += XmlObjectForCoding.EncodeOutputString(m_sIdent, bUnicode);
	XmlData += "\" application=\"";
	XmlData += XmlObjectForCoding.EncodeOutputString(GetAppName(), bUnicode);
	XmlData += "\" source=\"";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
		XmlData += "CLIENT";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
		XmlData += "BROWSER";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE)
		XmlData += "PROCEDURE";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
		XmlData += "SERVICE";
	XmlData += "\" host=\"";
	XmlData +=XmlObjectForCoding.EncodeOutputString(CommandToExecute->m_sHost, bUnicode);
	XmlData += "\">\n";
	if(bWithBinaryOutput)
	{
		StringResult += XmlData;
		ReserveString(StringResult);
	}
	else
	{
		if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
			((NvObjFile*)ObjectResult)->Close();
			OutContainer->RemoveObject(ObjResult.c_str());
			return false;
		}
	}
	XmlData = "";
	
	map<string,string>::iterator pos;
	if(iXmlCoding == XML_MODEL_COMPACT)
		bWithHttpHeaders = false; // No headers in compact mode
	if(iXmlCoding == XML_MODEL_TINY)
		bWithHttpHeaders = false; // No headers in tiny mode
	if(bWithHttpHeaders)
	{
		for(pos = CommandToExecute->HttpInHeaders.m_mList.begin(); pos != CommandToExecute->HttpInHeaders.m_mList.end(); ++pos)
		{
			XmlData = "\t<";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "httpheader";
			else
				XmlData += "NVHTTPHEADER";
			XmlData += " key=\"";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->first, bUnicode);
			XmlData += "\">";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->second, bUnicode);
			XmlData += "</";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "httpheader";
			else
				XmlData += "NVHTTPHEADER";
			XmlData += ">\n";
			if(bWithBinaryOutput)
			{
				StringResult += XmlData;
				ReserveString(StringResult);
			}
			else
			{
				if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
				{
					CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
					((NvObjFile*)ObjectResult)->Close();
					OutContainer->RemoveObject(ObjResult.c_str());
					return false;
				}
			}
		}
		XmlData = "\n";
		if(bWithBinaryOutput)
		{
			StringResult += XmlData;
			ReserveString(StringResult);
		}
		else
		{
			if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
				((NvObjFile*)ObjectResult)->Close();
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
		}
	}
	XmlData = "";

	if(bWithVariables)
	{
		for(pos = m_oVariables.m_mList.begin(); pos != m_oVariables.m_mList.end(); ++pos)
		{
			XmlData = "\t<";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_COMPACT)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_TINY)
				XmlData += "variable";
			else
				XmlData += "NVSESSIONVAR";
			XmlData +=" key=\"";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->first, bUnicode);
			XmlData += "\">";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->second, bUnicode);
			XmlData += "</";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_COMPACT)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_TINY)
				XmlData += "variable";
			else
				XmlData += "NVSESSIONVAR";
			XmlData += ">\n";
			if(bWithBinaryOutput)
			{
				StringResult += XmlData;
				ReserveString(StringResult);
			}
			else
			{
				if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
				{
					CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
					((NvObjFile*)ObjectResult)->Close();
					OutContainer->RemoveObject(ObjResult.c_str());
					return false;
				}
			}
		}
		XmlData = "\n";
		
		if(bWithBinaryOutput)
		{
			StringResult += XmlData;
			ReserveString(StringResult);
		}
		else
		{
			if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
				((NvObjFile*)ObjectResult)->Close();
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
		}
	}
	XmlData = "";

	if(iXmlCoding == XML_MODEL_NORMAL)
	{
		XmlData = "\t<";
		XmlData += NsPrefix;
		XmlData += "NVCONTAINER name=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(InContainer->GetName(), bUnicode);
		XmlData += "\">\n";
		if(bWithBinaryOutput)
		{
			StringResult += XmlData;
			ReserveString(StringResult);
		}
		else
		{
			if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
				((NvObjFile*)ObjectResult)->Close();
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
		}
	}

	for(i=0; i<ObjectsToGet.GetSize(); i++)
	{
		string sObjectInList = ObjectsToGet.GetValue(i);
		if(InContainer == OutContainer)
		{
			// We must not take the just created object
			string sResultObjectName = ObjResult;
			MakeLower(sResultObjectName);
			MakeLower(sObjectInList);
			if(sObjectInList == sResultObjectName)
				continue;
		}
		// First get requested objects
		if(bWithBinaryOutput)
		{
			if(!(InContainer->GetXmlObjectDataToString(ObjectsToGet.GetValue(i).c_str(), StringResult, NumTabs+1, false, bWithData, bWithFileData, NsPrefix, iXmlCoding, NULL, &XmlObjectForCoding, bUnicode)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, ObjectsToGet.GetValue(i));
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
			ReserveString(StringResult);
		}
		else
		{
			if(!(InContainer->GetXmlObjectDataToStream(ObjectsToGet.GetValue(i).c_str(), ((NvObjFile*)ObjectResult)->GetFileDescriptor(), NumTabs+1, false, bWithData, bWithFileData, NsPrefix, iXmlCoding, NULL, &XmlObjectForCoding, bUnicode)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, ObjectsToGet.GetValue(i));
				((NvObjFile*)ObjectResult)->Close();
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
		}
	}

	// Then get enventual subcontainers
	if(bWithSubContainers)
	{
		if(bWithBinaryOutput)
		{
			if(!(InContainer->GetXmlObjectDataToString(NULL, StringResult, NumTabs+1, bWithSubContainers, bWithData, bWithFileData, NsPrefix, iXmlCoding, NULL, &XmlObjectForCoding, bUnicode)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
			ReserveString(StringResult);
		}
		else
		{
			if(!(InContainer->GetXmlObjectDataToStream(NULL, ((NvObjFile*)ObjectResult)->GetFileDescriptor(), NumTabs+1, bWithSubContainers, bWithData, bWithFileData, NsPrefix, iXmlCoding, NULL, &XmlObjectForCoding, bUnicode)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
				((NvObjFile*)ObjectResult)->Close();
				OutContainer->RemoveObject(ObjResult.c_str());
				return false;
			}
		}
	}
	XmlData = "";

	if(iXmlCoding == XML_MODEL_NORMAL)
	{
		XmlData = "\t</";
		XmlData += NsPrefix;
		XmlData += "NVCONTAINER>\n";
	}

	XmlData += "</";
	XmlData += NsPrefix;
	if(iXmlCoding == XML_MODEL_SIMPLE)
		XmlData += "nirva>\n";
	else if(iXmlCoding == XML_MODEL_COMPACT)
		XmlData += "nirva>\n";
	else if(iXmlCoding == XML_MODEL_TINY)
		XmlData += "nirva>\n";
	else
		XmlData += "NIRVA>\n";
	if(bWithBinaryOutput)
	{
		StringResult += XmlData;
		char *OutputData = (char *)(StringResult.c_str());
		if(!((NvObjBinary*)ObjectResult)->Populate(OutputData, (int)(StringResult.length())))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
			OutContainer->RemoveObject(ObjResult.c_str());
			return false;
		}
	}
	else
	{
		if(fwrite(XmlData.c_str(), 1, XmlData.length(), ((NvObjFile*)ObjectResult)->GetFileDescriptor()) != XmlData.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, InContainer->GetName());
			((NvObjFile*)ObjectResult)->Close();
			OutContainer->RemoveObject(ObjResult.c_str());
			return false;
		}
		((NvObjFile*)ObjectResult)->Close();
	}


	CommandToExecute->FreeData();
	return true;
}


bool NvSession::XmlGetXml(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get object source name
	string ObjSrc = CommandToExecute->Parameters.GetValue("XMLOBJ");
	CleanString(ObjSrc);
	if(!ObjSrc.length())
		ObjSrc = "XML";

	// Test if object exist and is of file type
	NvObj *InObject = InContainer->GetObject(ObjSrc.c_str());
	if(InObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjSrc);
		return false;
	}
	if((InObject->GetType() != NV_OBJTYPE_FILE) && (InObject->GetType() != NV_OBJTYPE_BINARY))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjSrc);
		return false;
	}

	if(InObject->GetType() == NV_OBJTYPE_FILE)
	{
		NvObjFile *FileObject = (NvObjFile *)InObject;
		string ObjectFileName = FileObject->GetFilePathName();
		if(!ObjectFileName.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_NO_FILE, ObjSrc);
			return false;
		}

		CommandToExecute->DataFile = ObjectFileName;
	}
	if(InObject->GetType() == NV_OBJTYPE_BINARY)
	{
		NvObjBinary *BinaryObject = (NvObjBinary *)InObject;
		CommandToExecute->Data = BinaryObject->m_cdata;
		CommandToExecute->DataSize = BinaryObject->m_isize;
	}

	NvXml XmlParser;
	string NameSpace;
	int XmlCoding;
	bool RetVal = XmlParser.SetInputData(OutContainer, CommandToExecute, NameSpace, XmlCoding);

	CommandToExecute->DataFile = "";
	CommandToExecute->Data = NULL;
	CommandToExecute->DataSize = 0;
	
	return RetVal;
}


bool NvSession::XmlTransform(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object names that may be in parameters
	string ObjXmlSrc = CommandToExecute->Parameters.GetValue("XMLSRC");
	CleanString(ObjXmlSrc);
	if(!ObjXmlSrc.length())
		ObjXmlSrc = "XML";

	// Get object result name
	string ObjXmlDest = CommandToExecute->Parameters.GetValue("XMLDEST");
	CleanString(ObjXmlDest);
	if(!ObjXmlDest.length())
		ObjXmlDest = "XMLDEST";

	// Get output type
	bool bWithBinaryOutput = false;
	string sWithBinaryOutput = CommandToExecute->Parameters.GetValue("OUTPUT");
	CleanString(sWithBinaryOutput);
	MakeUpper(sWithBinaryOutput);
	if(sWithBinaryOutput == "BINARY")
		bWithBinaryOutput = true;

	// Get object xsl
	string ObjXsl = CommandToExecute->Parameters.GetValue("XSL");
	CleanString(ObjXsl);
	if(!ObjXsl.length())
		ObjXsl = "XSL";
	
	string Xsl = CommandToExecute->Parameters.GetValue("XSL_NAME");
	CleanString(Xsl);
	if(Xsl.length())
		ObjXsl = "";

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;


	NvObj *ObjectXmlSrc = InContainer->GetObject(ObjXmlSrc.c_str());
	if(ObjectXmlSrc == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjXmlSrc);
		return false;
	}
	if((ObjectXmlSrc->GetType() != NV_OBJTYPE_FILE) && (ObjectXmlSrc->GetType() != NV_OBJTYPE_BINARY))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjXmlSrc);
		return false;
	}
	bool bBinaryInput = false;
	if(ObjectXmlSrc->GetType() == NV_OBJTYPE_BINARY)
		bBinaryInput = true;
	
	
	string XslFile = "";
	if(ObjXsl.length())
	{
		//Get the file name form object xsl
		NvObjFile *ObjectXsl = (NvObjFile *)(InContainer->GetObject(ObjXsl.c_str()));
		if(ObjectXsl == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjXsl);
			return false;
		}
		if(ObjectXsl->GetType() != NV_OBJTYPE_FILE)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjXsl);
			return false;
		}
		XslFile = ObjectXsl->GetFilePathName();
	}
	else
	{
		// Get the filename directly
		XslFile = Application->GetFileDir();
		
		// Check if this is a system or service Xsl (name is enclosed in parentheses preceded eventually by the service name)
		// ex: Toto(zut) -> file zut.xsl for service toto
		string::size_type index = Xsl.find('(');
		if(index != string::npos)
		{
			if(Xsl[Xsl.length() - 1] != ')')
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_NAME, Xsl);
				return false;
			}

			string ServiceName = "";
			// this is a system or service xsl
			if(index != 0)
			{
				// it's probably a service xsl
				// we must get the service name
				ServiceName = Xsl.substr(0, index);
				CleanString(ServiceName);
				MakeUpper(ServiceName);
				Xsl = Xsl.substr(index);
			}

			// We must remove the parentheses
			Xsl = Xsl.substr(1, Xsl.length() - 2);
			
			// And get the xsl file from system or service Files directory
			if(ServiceName.length())
			{
				NvService *Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
				if(Service == NULL)
				{
					CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
					return false;
				}
				XslFile = Service->GetFileDir();
				GetMain()->Services.FreeService(ServiceName.c_str());
				AddDirSuffix(XslFile);
			}
			else
			{
				XslFile = GetMain()->GetNirvaDir();
				AddDirSuffix(XslFile);
				XslFile += "Files";
				AddDirSuffix(XslFile);
			}
		}

		// if no extension, we give the .xsl extension
		index = Xsl.rfind('.');
		if(index == string::npos)
			Xsl += ".xsl";


		XslFile += Xsl;
		if(!GetNoCaseRealFileName(XslFile))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_XSL_FILE, XslFile);
			return false;
		}
	}

	
	NvObj *ObjectXmlDest = OutContainer->GetObject(ObjXmlDest.c_str());

	if(ObjectXmlDest != NULL)
	{
		if(bWithBinaryOutput)
		{
			// We must remove the object
			OutContainer->RemoveObject(ObjXmlDest.c_str());
			ObjectXmlDest = NULL;
		}
		else
		{
			// We must remove the object if it's not a file object or if it doesn't have file name
			if(ObjectXmlDest->GetType() == NV_OBJTYPE_FILE)
			{
				if(!(((NvObjFile*)ObjectXmlDest)->GetFileName().length()))
				{
					OutContainer->RemoveObject(ObjXmlDest.c_str());
					ObjectXmlDest = NULL;
				}
			}
			else
			{
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				ObjectXmlDest = NULL;
			}
		}
	}

	if(ObjectXmlDest == NULL)
	{
		int ObjectType = NV_OBJTYPE_FILE;
		if(bWithBinaryOutput)
			ObjectType = NV_OBJTYPE_BINARY;
		if(!OutContainer->CreateObject(ObjXmlDest.c_str(), ObjectType))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjXmlDest);
			return false;
		}
		ObjectXmlDest = OutContainer->GetObject(ObjXmlDest.c_str());

		if(!bWithBinaryOutput)
		{
			string sWorkDir = "";
			string sPrefix = "";
			sWorkDir = Application->GetWorkDir();
			sPrefix = Application->GetAppNameForFiles();
			if(!((NvObjFile*)ObjectXmlDest)->SetAutoFileName(sWorkDir, "xml", sPrefix))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, Application->GetWorkDir());
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				return false;
			}
		}
	}

	// Prepare the list of xslt parameter
	NvObjIndStringList XsltParameters;
	XsltParameters.CopyFrom(&(CommandToExecute->Parameters));
	XsltParameters.Remove("XMLSRC");
	XsltParameters.Remove("XMLDEST");
	XsltParameters.Remove("OUTPUT");
	XsltParameters.Remove("XSL");
	XsltParameters.Remove("XSL_NAME");
	// Put them between cotes
	map<string,string>::iterator pos;
	for(pos = XsltParameters.m_mList.begin(); pos != XsltParameters.m_mList.end(); ++pos)
	{
		string NewValue = "\'";
		NewValue += pos->second.c_str();
		NewValue += "\'";
		pos->second = NewValue;
	}


	// Get the skip DTD option
	bool bSkipDtd = false;
	string sSkipDtd = CommandToExecute->Parameters.GetValue("SKIP_DTD");
	CleanString(sSkipDtd);
	MakeUpper(sSkipDtd);
	if(sSkipDtd == "YES")
		bSkipDtd = true;

	// Now, all the objects are here and are created

	if(bWithBinaryOutput)
	{
		if(bBinaryInput)
			((NvObjBinary*)ObjectXmlDest)->CopyFrom((NvObjBinary*)ObjectXmlSrc);
		else
			((NvObjBinary*)ObjectXmlDest)->Load(((NvObjFile*)ObjectXmlSrc)->GetFilePathName().c_str());
		char **OutputData = &((NvObjBinary*)ObjectXmlDest)->m_cdata;
		unsigned long *OutputDataSize = (unsigned long *)&((NvObjBinary*)ObjectXmlDest)->m_isize;
		NvXml XmlParser;
		if(!XmlParser.XsltTransform(OutputData, OutputDataSize, XslFile.c_str(), &XsltParameters, bSkipDtd))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
			OutContainer->RemoveObject(ObjXmlDest.c_str());
			return false;
		}
	}
	else
	{
		if(bBinaryInput)
		{
			NvObjBinary ProviBinary;
			ProviBinary.CopyFrom((NvObjBinary*)ObjectXmlSrc);
			char **OutputData = &ProviBinary.m_cdata;
			unsigned long *OutputDataSize = (unsigned long *)&ProviBinary.m_isize;
			NvXml XmlParser;
			if(!XmlParser.XsltTransform(OutputData, OutputDataSize, XslFile.c_str(), &XsltParameters, bSkipDtd))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				return false;
			}
			if(!ProviBinary.Save(((NvObjFile*)ObjectXmlDest)->GetFilePathName().c_str()))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				return false;
			}
		}
		else
		{
			if(!ParseXml(CommandToExecute, ((NvObjFile*)ObjectXmlSrc)->GetFilePathName().c_str(), ((NvObjFile*)ObjectXmlDest)->GetFilePathName().c_str(), XslFile.c_str(), &XsltParameters, bSkipDtd))
			{
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				return false;
			}
		}
	}
	

	return true;

}



bool NvSession::XmlValidate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object names that may be in parameters
	string ObjXmlDoc = CommandToExecute->Parameters.GetValue("XML");
	CleanString(ObjXmlDoc);
	if(!ObjXmlDoc.length())
		ObjXmlDoc = "XML";

	// Get mode (SCHEMA or DTD)
	string Mode = CommandToExecute->Parameters.GetValue("MODE");
	CleanString(Mode);
	MakeUpper(Mode);
	bool bModeDtd = false;
	if(Mode == "DTD")
		bModeDtd = true;

	string ObjSchema = "";
	string ObjDtd = "";
	if(bModeDtd)
	{
		// Get dtd file object name
		ObjDtd = CommandToExecute->Parameters.GetValue("DTD");
		CleanString(ObjDtd);
		if(!ObjDtd.length())
			ObjDtd = "DTD";
	}
	else
	{
		// Get schema file object name
		ObjSchema = CommandToExecute->Parameters.GetValue("SCHEMA");
		CleanString(ObjSchema);
		if(!ObjSchema.length())
			ObjSchema = "SCHEMA";
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;


	NvObj *ObjectXmlDoc = InContainer->GetObject(ObjXmlDoc.c_str());
	if(ObjectXmlDoc == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjXmlDoc);
		return false;
	}
	if((ObjectXmlDoc->GetType() != NV_OBJTYPE_FILE) && (ObjectXmlDoc->GetType() != NV_OBJTYPE_BINARY))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjXmlDoc);
		return false;
	}
	bool bBinaryInput = false;
	if(ObjectXmlDoc->GetType() == NV_OBJTYPE_BINARY)
		bBinaryInput = true;
	
	NvObjFile *ObjectSchema = NULL;
	NvObjFile *ObjectDtd = NULL;

	if(bModeDtd)
	{
		ObjectDtd = (NvObjFile *)(InContainer->GetObject(ObjDtd.c_str()));
		if(ObjectDtd == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjDtd);
			return false;
		}
		if(ObjectDtd->GetType() != NV_OBJTYPE_FILE)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjDtd);
			return false;
		}
	}
	else
	{
		ObjectSchema = (NvObjFile *)(InContainer->GetObject(ObjSchema.c_str()));
		if(ObjectSchema == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjSchema);
			return false;
		}
		if(ObjectSchema->GetType() != NV_OBJTYPE_FILE)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjSchema);
			return false;
		}
	}


	// Now, all the objects are here

	NvXml XmlParser;
	bool RetVal = true;
	string ResultData = "";
	if(bBinaryInput)
	{
		char *Data = ((NvObjBinary*)ObjectXmlDoc)->m_cdata;
		unsigned long DataSize = ((NvObjBinary*)ObjectXmlDoc)->m_isize;
		if(bModeDtd)
			RetVal = XmlParser.ValidateDtd(Data, DataSize, ObjectDtd->GetFilePathName().c_str(), ResultData);
		else
			RetVal = XmlParser.ValidateSchema(Data, DataSize, ObjectSchema->GetFilePathName().c_str(), ResultData);
	}
	else
	{
		if(bModeDtd)
			RetVal = XmlParser.ValidateDtd(((NvObjFile*)ObjectXmlDoc)->GetFilePathName().c_str(), ObjectDtd->GetFilePathName().c_str(), ResultData);
		else
			RetVal = XmlParser.ValidateSchema(((NvObjFile*)ObjectXmlDoc)->GetFilePathName().c_str(), ObjectSchema->GetFilePathName().c_str(), ResultData);
	}

	if(!RetVal)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_VALIDATION_ERROR, ResultData);
		return false;
	}

	return true;

}

bool NvSession::XmlClearCache(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized from procedure or service
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:XML:CLEAR_CACHE");
		return false;
	}
	NvXml::EmptyCache();

	return true;
}


bool NvSession::XmlLoadXsl(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object names that may be in parameters
	string Xsl = CommandToExecute->Parameters.GetValue("XSL_NAME");
	CleanString(Xsl);
	if(!Xsl.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "XML:LOAD_XSL:XSL_NAME");
		return false;
	}


	// Get object xsl
	string ObjXsl = CommandToExecute->Parameters.GetValue("XSL");
	CleanString(ObjXsl);
	if(!ObjXsl.length())
		ObjXsl = "XSL";

	string XslFile = "";
	XslFile = Application->GetFileDir();
	
	// Check if this is a system or service Xsl (name is enclosed in parentheses preceded eventuall by the service name)
	// ex: Toto(zut) -> file zut.xsl for service toto
	string::size_type index = Xsl.find('(');
	if(index != string::npos)
	{
		if(Xsl[Xsl.length() - 1] != ')')
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_NAME, Xsl);
			return false;
		}

		string ServiceName = "";
		// this is a system or service xsl
		if(index != 0)
		{
			// it's probably a service xsl
			// we must get the service name
			ServiceName = Xsl.substr(0, index);
			CleanString(ServiceName);
			MakeUpper(ServiceName);
			Xsl = Xsl.substr(index);
		}

		// We must remove the parentheses
		Xsl = Xsl.substr(1, Xsl.length() - 2);
		
		// And get the xsl file from system or service Files directory
		if(ServiceName.length())
		{
			NvService *Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
			if(Service == NULL)
			{
				CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
				return false;
			}
			XslFile = Service->GetFileDir();
			GetMain()->Services.FreeService(ServiceName.c_str());
			AddDirSuffix(XslFile);
		}
		else
		{
			XslFile = GetMain()->GetNirvaDir();
			AddDirSuffix(XslFile);
			XslFile += "Files";
			AddDirSuffix(XslFile);
		}
	}

	// if no extension, we give the .xsl extension
	index = Xsl.rfind('.');
	if(index == string::npos)
		Xsl += ".xsl";


	XslFile += Xsl;
	if(!GetNoCaseRealFileName(XslFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_XSL_FILE, XslFile);
		return false;
	}

	// Now we have the complete Xsl file path in XslFile

	NvObjFile *ObjectXsl = (NvObjFile *)(OutContainer->GetObject(ObjXsl.c_str()));
	if(ObjectXsl != NULL)
	{
		// We must remove the object if it's not a file object or if it doesn't have file name
		if(ObjectXsl->GetType() == NV_OBJTYPE_FILE)
		{
			if(!(ObjectXsl->GetFileName().length()))
			{
				OutContainer->RemoveObject(ObjXsl.c_str());
				ObjectXsl = NULL;
			}
		}
		else
		{
			OutContainer->RemoveObject(ObjXsl.c_str());
			ObjectXsl = NULL;
		}
	}

	if(ObjectXsl == NULL)
	{
		if(!OutContainer->CreateObject(ObjXsl.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjXsl);
			return false;
		}

		ObjectXsl = (NvObjFile *)(OutContainer->GetObject(ObjXsl.c_str()));
		string sWorkDir = "";
		string sPrefix = "";
		sWorkDir = Application->GetWorkDir();
		sPrefix = Application->GetAppNameForFiles();
		if(!ObjectXsl->SetAutoFileName(sWorkDir, "xsl", sPrefix))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, Application->GetWorkDir());
			OutContainer->RemoveObject(ObjXsl.c_str());
			return false;
		}
	}

	if(!CopyFileEx(XslFile.c_str(), ObjectXsl->GetFilePathName().c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_XSL_FILE, XslFile);
		return false;
	}

	return true;
}



// Change the XSL output of the original command for a web command
bool NvSession::XmlSetXsl(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command does nothing if the original command doesn't come from browser
	if(OriginalCommand->m_iCommandSource != NV_SOURCE_BROWSER)
		return true;

	// This command is authorized only from procedure or service
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:XML:SET_XSL");
		return false;
	}

	string NewXsl = CommandToExecute->Parameters.GetValue("XSL_NAME");

	OriginalCommand->Parameters.SetValue("NV_XML_XSL", NewXsl);

	return true;
}


// Sends XML data over HTTP and get back result XML
bool NvSession::XmlSend(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get URL
	string Url = CommandToExecute->Parameters.GetValue("URL");
	CleanString(Url);
	if(!Url.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "URL");
		return false;
	}
	string Address = Url;
	string Command = "";
	string::size_type index;
	// replace backslashes with slashes
	while((index = Address.find('\\')) != string::npos)
		Address.replace(index, 1, 1, '/');
	index = Address.find("//");
	if(index != string::npos)
		index = Address.find('/', index+2);
	if(index != string::npos)
	{
		Command = Address.substr(index);
		Address =  Address.substr(0, index);
	}
	if(!Command.length())
		Command = "/";

	// Get PROXY
	string Proxy = CommandToExecute->Parameters.GetValue("PROXY");
	CleanString(Proxy);
	bool bUseProxy = false;
	bool bProxySsl = false;
	string sProxyServer = "";
	int iProxiPort = 4480;
	if(Proxy.length())
	{
		string ProxyHere = Proxy;
		MakeUpper(ProxyHere);
		CleanString(ProxyHere);
		index = ProxyHere.find("HTTP://");
		if(index != string::npos)
		{
			Proxy = Proxy.substr(index+7);
			bUseProxy = true;
			bProxySsl = false;
		}
		if(!bUseProxy)
		{
			index = ProxyHere.find("HTTPS://");
			if(index != string::npos)
			{
				Proxy = Proxy.substr(index+8);
				bUseProxy = true;
				bProxySsl = true;
			}
		}
		if(bUseProxy)
		{
			sProxyServer = Proxy;
			iProxiPort = 4480;
			SplitAddress(sProxyServer, iProxiPort);
/*			index = sProxyServer.find(':');
			if(index != string::npos)
			{
				string Provistr = sProxyServer.substr(index+1);
				CleanString(Provistr);
				iProxiPort = atoi(Provistr.c_str());
				sProxyServer.erase(index);
			}*/
		}
	}

	// Get PROXY user
	string ProxyUser = CommandToExecute->Parameters.GetValue("PROXY_USER");
	// Get PROXY password
	string ProxyPassword = CommandToExecute->Parameters.GetValue("PROXY_PASSWORD");

	// Get HTTP user
	string HttpUser = CommandToExecute->Parameters.GetValue("HTTP_USER");
	// Get HTTP password
	string HttpPassword = CommandToExecute->Parameters.GetValue("HTTP_PASSWORD");

	bool b_UseSSL = false;
	bool bHttpFound = false;
	string Provistr = Address;
	MakeUpper(Provistr);
	index = Provistr.find("HTTPS://");
	if(index == 0)
	{
		Address =  Address.substr(index+8);
		b_UseSSL = true;
		bHttpFound = true;
	}
	index = Provistr.find("HTTP://");
	if(index == 0)
	{
		Address =  Address.substr(index+7);
		bHttpFound = true;
	}
	if(!bHttpFound)
	{
		string Ssl = CommandToExecute->Parameters.GetValue("SSL");
		CleanString(Ssl);
		MakeUpper(Ssl);
		if(Ssl == "YES")
			b_UseSSL = true;
	}

	// Get port
	int Port = 80;
	if(b_UseSSL)
		Port = 443;
	string sHost = Address;
	SplitAddress(Address, Port);
/*	index = Address.find(':');
	if(index == string::npos)
	{
		string sPort = CommandToExecute->Parameters.GetValue("PORT");
		CleanString(sPort);
		MakeUpper(sPort);
		if(sPort.length())
			Port = atoi(sPort.c_str());
		if(Port <= 0)
		{
			CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "PORT");
			return false;
		}
	}
	else
	{
		string Provistr = Address.substr(index+1);
		CleanString(Provistr);
		Port = atoi(Provistr.c_str());
		Address.erase(index);
	}
*/

	string sPort = CommandToExecute->Parameters.GetValue("PORT");
	CleanString(sPort);
	MakeUpper(sPort);
	if(sPort.length())
		Port = atoi(sPort.c_str());
	if(!Address.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "URL");
		return false;
	}

	int iServerConnectionTimeOut = 10;
	string sServerConnectionTimeOut = CommandToExecute->Parameters.GetValue("CTIMEOUT");
	CleanString(sServerConnectionTimeOut);
	MakeUpper(sServerConnectionTimeOut);
	if(sServerConnectionTimeOut.length())
		iServerConnectionTimeOut = atoi(sServerConnectionTimeOut.c_str());
	if(iServerConnectionTimeOut < 0)
		iServerConnectionTimeOut = 0;

	// Get object names that may be in parameters
	string ObjXmlSrc = CommandToExecute->Parameters.GetValue("XMLIN");
	CleanString(ObjXmlSrc);
	if(!ObjXmlSrc.length())
		ObjXmlSrc = "XMLIN";

	// Get object result name
	string ObjXmlDest = CommandToExecute->Parameters.GetValue("XMLOUT");
	CleanString(ObjXmlDest);
	if(!ObjXmlDest.length())
		ObjXmlDest = "XMLOUT";

	// Get output type
	bool bWithBinaryOutput = false;
	string sWithBinaryOutput = CommandToExecute->Parameters.GetValue("OUTPUT");
	CleanString(sWithBinaryOutput);
	MakeUpper(sWithBinaryOutput);
	if(sWithBinaryOutput == "BINARY")
		bWithBinaryOutput = true;

	// Get close option
	bool bCloseAfter = false;
	string sCloseAfter = CommandToExecute->Parameters.GetValue("CLOSE");
	CleanString(sCloseAfter);
	MakeUpper(sCloseAfter);
	if(sCloseAfter == "YES")
		bCloseAfter = true;

	// Get reuse conenction option
	bool bReuse = true;
	string sReuse = CommandToExecute->Parameters.GetValue("REUSE");
	CleanString(sReuse);
	MakeUpper(sReuse);
	if(sReuse == "NO")
		bReuse = false;

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;


	NvObj *ObjectXmlSrc = InContainer->GetObject(ObjXmlSrc.c_str());
	if(ObjectXmlSrc == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjXmlSrc);
		return false;
	}
	if((ObjectXmlSrc->GetType() != NV_OBJTYPE_FILE) && (ObjectXmlSrc->GetType() != NV_OBJTYPE_BINARY))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjXmlSrc);
		return false;
	}
	bool bBinaryInput = false;
	if(ObjectXmlSrc->GetType() == NV_OBJTYPE_BINARY)
		bBinaryInput = true;
	
	
	NvObj *ObjectXmlDest = OutContainer->GetObject(ObjXmlDest.c_str());

	if(ObjectXmlDest != NULL)
	{
		if(bWithBinaryOutput)
		{
			// We must remove the object
			OutContainer->RemoveObject(ObjXmlDest.c_str());
			ObjectXmlDest = NULL;
		}
		else
		{
			// We must remove the object if it's not a file object or if it doesn't have file name
			if(ObjectXmlDest->GetType() == NV_OBJTYPE_FILE)
			{
				if(!(((NvObjFile*)ObjectXmlDest)->GetFileName().length()))
				{
					OutContainer->RemoveObject(ObjXmlDest.c_str());
					ObjectXmlDest = NULL;
				}
			}
			else
			{
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				ObjectXmlDest = NULL;
			}
		}
	}

	if(ObjectXmlDest == NULL)
	{
		int ObjectType = NV_OBJTYPE_FILE;
		if(bWithBinaryOutput)
			ObjectType = NV_OBJTYPE_BINARY;
		if(!OutContainer->CreateObject(ObjXmlDest.c_str(), ObjectType))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjXmlDest);
			return false;
		}
		ObjectXmlDest = OutContainer->GetObject(ObjXmlDest.c_str());

		if(!bWithBinaryOutput)
		{
			string sWorkDir = "";
			string sPrefix = "";
			sWorkDir = Application->GetWorkDir();
			sPrefix = Application->GetAppNameForFiles();
			if(!((NvObjFile*)ObjectXmlDest)->SetAutoFileName(sWorkDir, "xml", sPrefix))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, Application->GetWorkDir());
				OutContainer->RemoveObject(ObjXmlDest.c_str());
				return false;
			}
		}
	}

	// Get eventual client certificate
	string ClientCertificateFile = "";

	// Get object names that may be in parameters
	string ObjCertificate = CommandToExecute->Parameters.GetValue("CERTIFICATE");
	CleanString(ObjCertificate);
	if(ObjCertificate.length())
	{
		NvObj *ObjectCertificate = InContainer->GetObject(ObjCertificate.c_str());
		if(ObjectCertificate == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjCertificate);
			return false;
		}
		ClientCertificateFile = ((NvObjFile *)ObjectCertificate)->GetFilePathName();
	}

	// Get eventual CA certificate
	string CACertificateFile = "";

	// Get object names that may be in parameters
	string ObjCACertificate = CommandToExecute->Parameters.GetValue("CACERT");
	CleanString(ObjCACertificate);
	if(ObjCACertificate.length())
	{
		NvObj *ObjectCACertificate = InContainer->GetObject(ObjCACertificate.c_str());
		if(ObjectCACertificate == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjCACertificate);
			return false;
		}
		CACertificateFile = ((NvObjFile *)ObjectCACertificate)->GetFilePathName();
	}


	// Get object names that may be in parameters
	string CertificatePassword = CommandToExecute->Parameters.GetValue("CERTIFICATE_PASSWORD");

	// Now, all the objects are here and are created
	// We can send the HTTP order

	NvHttpConnection* Connection = NULL;
	
	int ProxyMode = NO_PROXY;
	if(bUseProxy)
	{
		if(b_UseSSL)
			ProxyMode = PROXY_FINAL_SSL;
		else
			ProxyMode = PROXY_FINAL_NORMAL;
	}

	if(bUseProxy)
		Connection = GetMain()->Connections.GetConnection(sProxyServer.c_str(), iProxiPort, iServerConnectionTimeOut, bProxySsl, ProxyMode, Address.c_str(), Port, bReuse);
	else
		Connection = GetMain()->Connections.GetConnection(Address.c_str(), Port, iServerConnectionTimeOut, b_UseSSL, NO_PROXY, "", 1081, bReuse);
	if(Connection == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_CONNECT_SERVER, Address);
		OutContainer->RemoveObject(ObjXmlDest.c_str());
		return false;
	}

	// We get a connected connection
	Connection->AuthInfo.m_sUser = HttpUser;
	Connection->AuthInfo.m_sPassword = HttpPassword;
	Connection->ProxyAuthInfo.m_sUser = ProxyUser;
	Connection->ProxyAuthInfo.m_sPassword = ProxyPassword;
	if(bUseProxy && b_UseSSL)
		Connection->b_ProxyHttps = true;
	else
		Connection->b_ProxyHttps = false;

	Connection->sSocket.CertificateFile = ClientCertificateFile;
	Connection->sSocket.CACertFile = CACertificateFile;
	Connection->sSocket.PrivateKeyPassword = CertificatePassword;

	// Set an eventual log file
	Connection->sSocket.LogFile = CommandToExecute->Parameters.GetValue("SOCKET_LOG");

	// Get method type
	string sMethod = CommandToExecute->Parameters.GetValue("METHOD");
	CleanString(sMethod);
	MakeUpper(sMethod);
	if(!sMethod.length())
		sMethod = "POST";

	// Get full path option
	bool bFull = false;
	string sFull = CommandToExecute->Parameters.GetValue("FULL");
	CleanString(sFull);
	MakeUpper(sFull);
	if(sFull == "YES")
		bFull = true;

	if(bUseProxy)
	{
		bFull = true;
		if(b_UseSSL)
			bFull = false;
	}

	bool RetVal = false;
	
	// we clear the input message
	Connection->InMessage.Clear();
	
	// we clear the outpput message
	Connection->OutMessage.Clear();

	NvData *Data = NVNEW NvData;
	Data->sInfo = sMethod;
	Data->sInfo += " ";
	if(bFull)
		Data->sInfo += Url;
	else
		Data->sInfo += Command;
	Data->sInfo += " HTTP/1.1";
	Connection->OutMessage.vDatas.push_back(Data);
	
	// by default, we have no content in the body
	Connection->OutMessage.AddHeader("Content-Length", "0");
	
	// Set user agent
	Connection->OutMessage.AddHeader("User-Agent", "Nirva");
	
	// For HTTP 1.1 we must include the host header
	// In fact, it is not used on the server side
	Connection->OutMessage.AddHeader("Host", sHost);

	// We prepare the output global headers

	// Set content-type
	string sContentType = CommandToExecute->Parameters.GetValue("CONTENT_TYPE");
	if(!sContentType.length())
		sContentType = "text/xml";
	Connection->OutMessage.AddHeader("Content-Type", sContentType);

	// Set soap action
	if(CommandToExecute->Parameters.KeyExist("SOAP_ACTION"))
	{
		string sSoapAction = CommandToExecute->Parameters.GetValue("SOAP_ACTION");
		Connection->OutMessage.AddHeader("SOAPAction", sSoapAction);
	}

	// Now we set input data
	if(bBinaryInput)
	{
		Connection->OutMessage.vDatas[0]->Data = ((NvObjBinary *)ObjectXmlSrc)->m_cdata;
		Connection->OutMessage.vDatas[0]->DataSize = ((NvObjBinary *)ObjectXmlSrc)->m_isize;
	}
	else
	{
		string ObjectFileName = ((NvObjFile *)ObjectXmlSrc)->GetFilePathName();
		Connection->OutMessage.vDatas[0]->DataFile = ObjectFileName;
		Connection->OutMessage.vDatas[0]->bDeleteDataFile = false;	// Don't remove the file because we directly use the object file
	}

	// Set the real body size
	NvInt64 BodySize = Connection->OutMessage.GetWriteBodySize();
	char buffer[20];
	sprintf(buffer, "%llu", BodySize);
	Connection->OutMessage.AddHeader("Content-Length", buffer);

	Connection->OutMessage.AddHeader("Connection", "Keep-Alive");
	Connection->OutMessage.AddHeader("Proxy-Connection", "Keep-Alive");




	// Now send the order
	string TempWorkDir = GetMain()->GetNirvaDir();
	AddDirSuffix(TempWorkDir);
	TempWorkDir += "Work";
	RetVal = true;
	int OrderResult = Connection->SendOrder(true, GetMain()->GetHttpMaxMemBody(), TempWorkDir.c_str());
	
	// we clear the output message
	if(bBinaryInput)
	{
		// We should not delete the binary input data
		Connection->OutMessage.vDatas[0]->Data = NULL;
		Connection->OutMessage.vDatas[0]->DataSize = 0;
	}
	Connection->OutMessage.Clear();

	
	if(OrderResult != 1)
	{
		// An error occured
		// we clear the input message
		Connection->InMessage.Clear();
		if(OrderResult == -2)
			CommandToExecute->Error.SetError("SYSTEM", "HTTP", NV_HTTP_PROXY_AUTHENTICATION_ERROR, sProxyServer);
		else if(OrderResult == -1)
			CommandToExecute->Error.SetError("SYSTEM", "HTTP", NV_HTTP_AUTHENTICATION_ERROR, Address);
		else
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_NETWORK_ERROR, Address);
		if(bCloseAfter)
			Connection->sSocket.Close();
		Connection->sSocket.CloseLog();
		GetMain()->Connections.FreeConnection(Connection);
		OutContainer->RemoveObject(ObjXmlDest.c_str());
		return false;
	}
	
	// We populate the object
	if(Connection->InMessage.vDatas[0]->DataFile.length())
	{
		// We got a file result
		if(bWithBinaryOutput)
		{
			// We populate the binary object with file data
			if(!((NvObjBinary *)ObjectXmlDest)->Load(Connection->InMessage.vDatas[0]->DataFile.c_str()))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_POPULATE, ObjXmlDest);
				RetVal = false;
			}
		}
		else
		{
			// We populate the file object directly from the data file in command
			string ObjectFileName = ((NvObjFile *)ObjectXmlDest)->GetFilePathName();
			if(!CopyFileEx(Connection->InMessage.vDatas[0]->DataFile.c_str(), ObjectFileName.c_str()))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_POPULATE, ObjXmlDest);
				RetVal = false;
			}
		}
	}
	else
	{
		// We got a binary result
		if(bWithBinaryOutput)
		{
			// We let binary object pointing to data
			((NvObjBinary *)ObjectXmlDest)->m_cdata = Connection->InMessage.vDatas[0]->Data;
			((NvObjBinary *)ObjectXmlDest)->m_isize = Connection->InMessage.vDatas[0]->DataSize;
			Connection->InMessage.vDatas[0]->Data = NULL;
			Connection->InMessage.vDatas[0]->DataSize = 0;
		}
		else
		{
			// We populate the file object directly from the data
			string ObjectFileName = ((NvObjFile *)ObjectXmlDest)->GetFilePathName();
			if(!((NvObjFile *)ObjectXmlDest)->Populate(Connection->InMessage.vDatas[0]->Data, Connection->InMessage.vDatas[0]->DataSize))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_POPULATE, ObjXmlDest);
				RetVal = false;
			}
		}
	}

	// we clear the input message
	Connection->InMessage.Clear();
	if(bCloseAfter)
		Connection->sSocket.Close();
	Connection->sSocket.CloseLog();
	GetMain()->Connections.FreeConnection(Connection);

	return RetVal;
}


//SYSTEM SESSION GET_SESSION_ID
bool NvSession::GetSessionId(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(CommandToExecute, m_sIdent.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}


//SYSTEM SESSION GET_USER
bool NvSession::GetSessionUser(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(CommandToExecute, GetUserName().c_str(), Buffer, BufferSize, DataSize, iEncoding);
	string All = CommandToExecute->Parameters.GetValue("ALL_INFO");
	CleanString(All);
	MakeUpper(All);
	if(All == "YES")
	{
		// The function uses the output container
		if(!CheckOutputContainer(CommandToExecute))
			return false;

		// Prepare result object
		OutContainer->RemoveObject("USER_INFO");
		if(!OutContainer->CreateObject("USER_INFO", NV_OBJTYPE_INDSTRINGLIST))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "USER_INFO");
			return false;
		}

		NvObjIndStringList *UserInfo = (NvObjIndStringList *)(OutContainer->GetObject("USER_INFO"));
		UserInfo->SetValue("NAME", GetUserName());
		UserInfo->SetValue("FULLNAME", GetUserFullName());
	}
	
	return true;
}


//SYSTEM SESSION GET_PASSWORD
bool NvSession::GetSessionPassword(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:SESSION:GET_PASSWORD");
		return false;
	}

	if(!CheckPermission(CommandToExecute, "SYSTEM", "PASSWORD_GET"))
		return false;


	GetStringValue(CommandToExecute, m_sPassword.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}


//SYSTEM SESSION GET_NUM
bool NvSession::SessionGetNum(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetMain()->Sessions.Update();
	NV_LOCKOBJECT(&(GetMain()->Sessions));
	int NumSessions = (int)(GetMain()->Sessions.m_mSessions.size());
	NV_UNLOCKOBJECT(&(GetMain()->Sessions));
	char Buf[20];
	sprintf(Buf, "%d", NumSessions);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	return true;
}


//SYSTEM SESSION SERVICES
bool NvSession::SessionServices(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SERVICES");
	if(!OutContainer->CreateObject("SERVICES", NV_OBJTYPE_STRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SERVICES");
		return false;
	}

	NvObj *Object = OutContainer->GetObject("SERVICES");
	NvObjStringList *IndSessionServices = (NvObjStringList *)Object;
	NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
	IndSessionServices->CopyFrom(&m_oServices);
	NV_UNLOCKLOCK(&ProtectInUse);

	return true;
}


//SYSTEM SESSION LIST
bool NvSession::SessionList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SESSION_INFO"))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SESSIONS");
	if(!OutContainer->CreateObject("SESSIONS", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SESSIONS");
		return false;
	}

	NvObjTable *Result = (NvObjTable *)(OutContainer->GetObject("SESSIONS"));

	string WhatToList = CommandToExecute->Parameters.GetValue("WHAT");
	CleanString(WhatToList);
	string WithCurrent = CommandToExecute->Parameters.GetValue("WITH_CURRENT");
	CleanString(WithCurrent);
	MakeUpper(WithCurrent);
	if(!WithCurrent.length())
	{
		WithCurrent = CommandToExecute->Parameters.GetValue("WITH_ACTUAL");
		CleanString(WithCurrent);
		MakeUpper(WithCurrent);
	}
	NvSession *ActualSession = this;
	if(WithCurrent == "YES")
		ActualSession = NULL;
	GetMain()->Sessions.List(Result, WhatToList.c_str(), ActualSession);

	return true;
}


//SYSTEM SESSION CLOSE_OTHER
bool NvSession::SessionCloseOther(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Authorized from service or procedure but not from clients.
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SESSION_CLOSE_OTHER"))
			return false;
	}

	string sSessionToClose = CommandToExecute->Parameters.GetValue("SESSION");
	CleanString(sSessionToClose);
	MakeUpper(sSessionToClose);
	if(sSessionToClose == m_sIdent)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_CLOSE, sSessionToClose);
		return false;
	}
	if(!sSessionToClose.length())
		return true;

	int iWaitTime = 10;
	string sWaitTime = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(sWaitTime);
	if(sWaitTime.length())
		iWaitTime = atoi(sWaitTime.c_str());
	if(iWaitTime < 0)
		iWaitTime = 0;
	GetMain()->Sessions.RemoveSession(sSessionToClose.c_str(), true, iWaitTime);

	return true;
}


//SYSTEM SESSION CREATE
// Create a new session. This can be user or application session
bool NvSession::SessionCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Only authorized from procedure, service or server not from clients
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:SESSION:CREATE");
		return false;
	}

	string AppliNameHere = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(AppliNameHere);
	// Default application is current one if not provided
	if(!AppliNameHere.length() && (!CommandToExecute->Parameters.KeyExist("APPLICATION")))
		AppliNameHere = GetAppName();
	// Default user is current one if not provided
	string UserHere = CommandToExecute->Parameters.GetValue("USER");
	CleanString(UserHere);
	if(!UserHere.length() && (!CommandToExecute->Parameters.KeyExist("USER")))
		UserHere = m_sUser;
	// Default password is current one if not provided
	string PasswordHere = CommandToExecute->Parameters.GetValue("PASSWORD");
	CleanString(PasswordHere);
	if(!PasswordHere.length() && (!CommandToExecute->Parameters.KeyExist("PASSWORD")))
		PasswordHere = m_sPassword;
	
	bool bWithNewPassword = false;
	if(CommandToExecute->Parameters.KeyExist("NEW_PASSWORD"))
		bWithNewPassword = true;
	string NewPasswordHere = CommandToExecute->Parameters.GetValue("NEW_PASSWORD");
	CleanString(NewPasswordHere);
	string NewPasswordHereConfirm = CommandToExecute->Parameters.GetValue("NEW_PASSWORD_CONFIRM");
	CleanString(NewPasswordHereConfirm);
	bool bWithNewPasswordConfirm = false;
	if(bWithNewPassword)
	{
		if(CommandToExecute->Parameters.KeyExist("NEW_PASSWORD_CONFIRM"))
			bWithNewPasswordConfirm = true;
	}

	string Hash = AppliNameHere + UserHere + PasswordHere;
	Hash = MD5_string(Hash);
	MakeUpper(Hash);
	int iSessionTimeOut = -1;	// application life time out by default
	if(CommandToExecute->Parameters.KeyExist("TIMEOUT"))
	{
		string sSessionTimeOut = CommandToExecute->Parameters.GetValue("TIMEOUT");
		CleanString(sSessionTimeOut);
		iSessionTimeOut = atoi(sSessionTimeOut.c_str());
	}

	NvSession *Session = NULL;


	// User session
	if(GetMain()->CheckOverTime())
	{
		CommandToExecute->Error.SetError("SYSTEM", "SYSTEM", NV_SYSTEM_OVER_TIME);
		return false;
	}
	Session = GetMain()->Sessions.GetSession(NULL);
	if(Session == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOTCREATE);
		int MaxSessions = GetMain()->GetMaxSessions();
		if((MaxSessions != -1) && (GetMain()->Sessions.GetNumSessions() >= MaxSessions))
			CommandToExecute->Error.SetError("SYSTEM", "SYSTEM", NV_SYSTEM_MAX_SESSIONS);
		return false;
	}
	// The session creation was OK
	// Now, we must login to it
	string SessionName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(SessionName);
	MakeLower(SessionName);
	Session->m_sName = SessionName;
	string OpenProc = CommandToExecute->Parameters.GetValue("OPEN");
	CleanString(OpenProc);
	if(!OpenProc.length())
		OpenProc = "session_open";
	if(OpenProc == "NV_SESSION_OPEN_NONE")
		OpenProc = "";
	Session->m_sOpenProc = OpenProc;
	string CloseProc = CommandToExecute->Parameters.GetValue("CLOSE");
	CleanString(CloseProc);
	if(!CloseProc.length())
		CloseProc = "session_close";
	if(CloseProc == "NV_SESSION_CLOSE_NONE")
		CloseProc = "";
	Session->m_sCloseProc = CloseProc;
	string SessionType = SESSION_TYPE_INTERNAL;
	if(SessionName.length())
		SessionType = SESSION_TYPE_NAMED;
	int AttachType = ATTACH_REQUEST_TYPE_INTERNAL;
	if(SessionType == SESSION_TYPE_NAMED)
		AttachType = ATTACH_REQUEST_TYPE_NAMED;
	GetMain()->AttachRequestList.Get(Session, AttachType, AppliNameHere);
	int LoginRetVal = Session->Login(UserHere.c_str(), PasswordHere.c_str(), AppliNameHere.c_str(), SessionType.c_str(), NewPasswordHere.c_str(), NewPasswordHereConfirm.c_str(), bWithNewPassword, bWithNewPasswordConfirm, iSessionTimeOut, Hash.c_str(), NULL, &CommandToExecute->Parameters.m_mList, CommandToExecute);
	if(LoginRetVal != 1)
	{
		// Login failed
		// We prepare error information
		string Info = UserHere;
		Info += " - ";
		Info += AppliNameHere;
		if(LoginRetVal == 0)
			CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOTLOGIN, Info);
		if(LoginRetVal == -1)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_USER, Info);
		if(LoginRetVal == -2)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_CONTEXT, Info);
		if(LoginRetVal == -3)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_PASSWORD, Info);
		if(LoginRetVal == -4)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_APPLICATION, Info);
		if(LoginRetVal == -5)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CHANGE_PASSWORD, Info);
		if(LoginRetVal == -6)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_PASSWORD_SYNTAX, Info);

		// We free the session
		GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());

		// And remove it
		GetMain()->Sessions.RemoveSession(Session->m_sIdent.c_str());
		Session = NULL;
		return false;
	}

	// Login is OK
	string UserHereForCheck = UserHere;
	CleanString(UserHereForCheck, true);
	MakeLower(UserHereForCheck);
	if(Session != NULL)
		if(UserHereForCheck == "nvadmin")
			Session->m_bAdministrator = true; // Administrator user

	// Now we return the session ID in the output buffer
	GetStringValue(CommandToExecute, Session->m_sIdent.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	// We free the session
	GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());

	return true;
}




//SYSTEM SESSION GET_NAMED
// Gets a named session for a given session
bool NvSession::SessionGetNamed(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Do nothing here since all the code is at the start of the NvSession:Command function

	return true;
}





//SYSTEM SESSION FREE_NAMED
// Gets a named session for a given session
bool NvSession::SessionFreeNamed(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Do nothing here since all the code is at the start of the NvSession:Command function

	return true;
}



//SYSTEM SESSION CHECK_CLOSE_REQUEST
bool NvSession::SessionCheckCloseRequest(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// First check the global nirva stop request
	bool bCloseRequest = GetMain()->GetStopRequest();

	// If no global stop request, check if the session has been requested to close
	// at this time, the time out is reset to 0
	if(!(bCloseRequest) && (GetTimeOut() == 0))
		bCloseRequest = true;

	// If this is a listener thread session
	if(m_sSessionType == SESSION_TYPE_LISTENER)
		bCloseRequest = GetListenerCloseRequest();

	// Return the result to caller
	if(bCloseRequest)
		GetStringValue(CommandToExecute, "YES", Buffer, BufferSize, DataSize, iEncoding);
	else
		GetStringValue(CommandToExecute, "NO", Buffer, BufferSize, DataSize, iEncoding);
	return true;
}





//SYSTEM SESSION SET_DEFAULT_ERROR_PROC
bool NvSession::SessionSetDefaultErrorProc(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string DefErrorProc = CommandToExecute->Parameters.GetValue("NAME");
	
	m_sDefErrorProcName = DefErrorProc;
	
	return true;
}



//SYSTEM SESSION SET_DEFAULT_ERROR_XSL
bool NvSession::SessionSetDefaultErrorXsl(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string DefErrorXsl = CommandToExecute->Parameters.GetValue("NAME");
	
	SetErrorXsl(DefErrorXsl.c_str());
	
	return true;
}



//SYSTEM SESSION SET_LANGUAGE
bool NvSession::SessionSetLanguage(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string LanguageHere = CommandToExecute->Parameters.GetValue("LANGUAGE");
	MakeUpper(LanguageHere);
	CleanString(LanguageHere);
	if(!LanguageHere.length())
		LanguageHere = "ENGLISH";
	m_sLanguage = LanguageHere;
	
	return true;
}

//SYSTEM SESSION USE_ID_COOKIE
bool NvSession::SessionUseIdCookie(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Only authorized from procedure
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:SESSION:USE_ID_COOKIE");
		return false;
	}

	// Only authorized in open session
	if(!bInOpenProc)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED, "SYSTEM:SESSION:USE_ID_COOKIE");
		return false;
	}

	string sIdCookie = CommandToExecute->Parameters.GetValue("ID_COOKIE");
	CleanString(sIdCookie);
	MakeUpper(sIdCookie);
	if(sIdCookie == "NO")
		m_bUseSessionCookie = false;
	else
		m_bUseSessionCookie = true;

	string sTimeOut = CommandToExecute->Parameters.GetValue("TIMEOUT");
	CleanString(sTimeOut);
	MakeUpper(sTimeOut);
	int iTimeOut = 0;
	if(sTimeOut == "SESSION")
		iTimeOut = GetTimeOut();
	else
	{
		iTimeOut = atoi(sTimeOut.c_str());
		if(iTimeOut < 0)
			iTimeOut = 0;
	}
	// if time out is 0 there will be no expiration date (session cookie)
	if(m_bUseSessionCookie)
		m_iSessionCookieTimeOut = iTimeOut;

	return true;
}


//SYSTEM SESSION SET_DEFAULT_VAR_IDENT
// Undocumented command
bool NvSession::SessionSetDefaultVarIdent(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Only authorized from procedure
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:SESSION:SET_DEFAULT_VAR_IDENT");
		return false;
	}

	string sValue = CommandToExecute->Parameters.GetValue("VALUE");
	CleanString(sValue, true);
	if(sValue == "NV_DEFAULT")
		m_sVariableIdent = Application->GetVariableIdent();
	else
		m_sVariableIdent = sValue;

	return true;
}


//SYSTEM SERVICE START
bool NvSession::OpenRequest(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string RequestName = CommandToExecute->Parameters.GetValue("NAME");
	bool bReopen = false;
	string sReopen = CommandToExecute->Parameters.GetValue("REOPEN");
	MakeUpper(sReopen);
	CleanString(sReopen);
	if(sReopen == "YES")
		bReopen = true;

	NvRequest *Request = Requests.AddRequest(RequestName.c_str(), bReopen);
	if(Request == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REQUEST", NV_REQUEST_CANNOT_CREATE);
		return false;
	}

	string Server = CommandToExecute->Parameters.GetValue("SERVER");
	Request->m_sAppli = CommandToExecute->Parameters.GetValue("APPLICATION");
	Request->m_sSessionId = CommandToExecute->Parameters.GetValue("SESSION");
	Request->m_sOpen = CommandToExecute->Parameters.GetValue("OPEN");
	Request->m_sClose = CommandToExecute->Parameters.GetValue("CLOSE");
	Request->m_sUser = CommandToExecute->Parameters.GetValue("USER");
	Request->m_sPassword = CommandToExecute->Parameters.GetValue("PASSWORD");
	if(!CommandToExecute->Parameters.KeyExist("APPLICATION"))
		Request->m_sAppli = m_sAppName;
	if(!CommandToExecute->Parameters.KeyExist("USER"))
		Request->m_sUser = m_sUser;
	if(!CommandToExecute->Parameters.KeyExist("PASSWORD"))
		Request->m_sPassword = m_sPassword;

	Request->bWithNewPassword = false;
	if(CommandToExecute->Parameters.KeyExist("NV_NEW_PASSWORD"))
		Request->bWithNewPassword = true;
	Request->m_sNewPassword = CommandToExecute->Parameters.GetValue("NV_NEW_PASSWORD");
	Request->bWithNewPasswordConfirm = false;
	if(Request->bWithNewPassword)
	{
		if(CommandToExecute->Parameters.KeyExist("NV_NEW_PASSWORD_CONFIRM"))
			Request->bWithNewPasswordConfirm = true;
	}
	Request->m_sNewPasswordConfirm = CommandToExecute->Parameters.GetValue("NV_NEW_PASSWORD_CONFIRM");

	CleanString(Server);
	CleanString(Request->m_sAppli);
	CleanString(Request->m_sSessionId);
	CleanString(Request->m_sUser);
	CleanString(Request->m_sPassword);
	CleanString(Request->m_sNewPassword);
	CleanString(Request->m_sNewPasswordConfirm);
	Request->m_sHash = Request->m_sAppli + Request->m_sUser + Request->m_sPassword;
	Request->m_sHash = MD5_string(Request->m_sHash);
	MakeUpper(Request->m_sHash);

	if(!Server.length())
		Server = "127.0.0.1";

	bool bVirtualModeLoadBalancing = false;
	int iVirtualNumAddresses = 0;
	if((Server[0] == '(') && (Server[Server.length()-1] == ')'))
	{
		// This is a virtual host address
		Server = Server.substr(1); 
		Server = Server.substr(0, Server.length()-1); 
		iVirtualNumAddresses = GetMain()->VirtualHosts.GetNumAddresses(Server.c_str());
		if(iVirtualNumAddresses != 0)
		{
			if(GetMain()->VirtualHosts.GetMode(Server.c_str()) == "LOAD_BALANCING")
				bVirtualModeLoadBalancing = true;
		}
	}
	if(!iVirtualNumAddresses)
		Request->m_sServer = Server;

	string ProviStr = CommandToExecute->Parameters.GetValue("CLOSE_MODE");
	CleanString(ProviStr);
	MakeUpper(ProviStr);
	if(ProviStr == "MANUAL")
		Request->m_iAutoClose = NV_REQUEST_CLOSE_MANUAL;
	if(ProviStr == "COMMAND")
		Request->m_iAutoClose = NV_REQUEST_CLOSE_COMMAND;
	if(ProviStr == "SESSION")
		Request->m_iAutoClose = NV_REQUEST_CLOSE_SESSION;

	ProviStr = CommandToExecute->Parameters.GetValue("SSL");
	CleanString(ProviStr);
	MakeUpper(ProviStr);
	if(ProviStr == "YES")
		Request->b_UseSSL = true;
	else
		Request->b_UseSSL = false;

	if(CommandToExecute->Parameters.KeyExist("SESSION_TIME_OUT"))
	{
		ProviStr = CommandToExecute->Parameters.GetValue("SESSION_TIME_OUT");
		int SessionTimeOut = atoi(ProviStr.c_str());
		Request->m_iSessionTimeOut = SessionTimeOut;
		if(Request->m_iSessionTimeOut < 0)
			Request->m_iSessionTimeOut = 0;
	}

	if(CommandToExecute->Parameters.KeyExist("CONNECTION_TIME_OUT"))
	{
		ProviStr = CommandToExecute->Parameters.GetValue("CONNECTION_TIME_OUT");
		int ConnectionTimeOut = atoi(ProviStr.c_str());
		Request->m_iServerConnectionTimeOut = ConnectionTimeOut;
		if(Request->m_iServerConnectionTimeOut < 0)
			Request->m_iServerConnectionTimeOut = 0;
	}
	char Buf[50];
	sprintf(Buf, "%lu", (unsigned long)Request);
	string RequestId = Buf;
	MakeUpper(RequestId);

	// Set the unicode parameter
	Request->b_ClientUnicode = GetMain()->IsUnicode();
	
	// Set an eventual log file
	Request->m_sSocketLogFile = CommandToExecute->Parameters.GetValue("SOCKET_LOG");

	if(iVirtualNumAddresses)
	{
		// We must get the real address and test the server
		NvCommand TestCommand;
		TestCommand.m_pSession = this;
		TestCommand.m_iCommandSource = NV_SOURCE_CLIENT;
		TestCommand.m_sService = "SYSTEM";
		TestCommand.m_sClass = "MISC";
		TestCommand.m_sCommand = "NOP";
		TestCommand.m_sLanguage = CommandToExecute->m_sLanguage;
		bool bConnected = false;
		if(!bVirtualModeLoadBalancing)
		{
			// In failover mode, we first try to connect the active server
			Request->m_sServer = GetMain()->VirtualHosts.GetActiveAddress(Server.c_str());
			if(!Request->m_sServer.length())
				Request->m_sServer = "127.0.0.1";
			if(Request->RequestCommand(&TestCommand))
				bConnected = true;
			else
			{
				// If the error is not a network error
				// the peer server is running so we must return the error
				if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
					(TestCommand.Error.m_sErrorClass != "REQUEST") ||
					(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
				{
					CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
					return false;
				}
			}
		}
		while(!bConnected)
		{
			// We must try a connection to next address but no more than the number of addresses defined
			if(!iVirtualNumAddresses)
				break;
			iVirtualNumAddresses--;
			Request->m_sServer = GetMain()->VirtualHosts.GetNextAddress(Server.c_str());
			if(!Request->m_sServer.length())
				Request->m_sServer = "127.0.0.1";
			if(Request->RequestCommand(&TestCommand))
				bConnected = true;
			else
			{
				// If the error is not a network error
				// the peer server is running so we must return the error
				if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
					(TestCommand.Error.m_sErrorClass != "REQUEST") ||
					(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
				{
					CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
					return false;
				}
			}
		}
		if(!bConnected)
		{
			CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
			return false;
		}
	}

	Request->SetServerAddress();

	GetStringValue(CommandToExecute, RequestId.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//SYSTEM SERVICE START
bool NvSession::CloseRequest(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	NvRequest *Request = NULL;
	string RequestName = CommandToExecute->Parameters.GetValue("NV_REQUEST");
	CleanString(RequestName);
	MakeUpper(RequestName);
	Requests.RemoveRequest(RequestName.c_str());
	return true;
}
	
	
//SYSTEM SERVICE START
bool NvSession::ServiceStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_RUN"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:START:SERVICE");
		return false;
	}

	// Get immediate option
	int iImmediate = START_MODE_FLAG;
	if(CommandToExecute->Parameters.KeyExist("IMMEDIATE"))
	{
		string sImmediate = CommandToExecute->Parameters.GetValue("IMMEDIATE");
		CleanString(sImmediate);
		MakeUpper(sImmediate);
		if(sImmediate == "YES")
			iImmediate = START_MODE_START;
		if(sImmediate == "NO")
			iImmediate = START_MODE_NOSTART;
	}


	if(!GetMain()->Services.EnableService(ServiceName.c_str(), iImmediate))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_INIT, ServiceName);
		return false;
	}


	return true;
}


//SYSTEM SERVICE STOP
bool NvSession::ServiceStop(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_RUN"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:STOP:SERVICE");
		return false;
	}

	// Get wait time option
	string WaitTimeInCommand = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(WaitTimeInCommand);
	MakeUpper(WaitTimeInCommand);
	int iWaitTimeInCommand = 60;	// by default we wait 60 seconds
	if(WaitTimeInCommand.length())
		iWaitTimeInCommand = atoi(WaitTimeInCommand.c_str());
	if(iWaitTimeInCommand < 0)
		iWaitTimeInCommand = 0;	// no wait time



	if(!GetMain()->Services.CheckRequiredForApplications(ServiceName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_DISABLE, ServiceName);
		return false;
	}

	if(!GetMain()->Services.DisableService(ServiceName.c_str(), m_sIdent.c_str(), iWaitTimeInCommand))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_DISABLE, ServiceName);
		return false;
	}


	return true;
}


//SYSTEM SERVICE SESSIONS
bool NvSession::ServiceSessions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_SESSION_LIST"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:SESSIONS:SERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SESSIONS");
	if(!OutContainer->CreateObject("SESSIONS", NV_OBJTYPE_STRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SESSIONS");
		return false;
	}

	NvObj *Object = OutContainer->GetObject("SESSIONS");
	NvObjStringList *ServiceSessions = (NvObjStringList *)Object;

	if(!GetMain()->Services.GetSessions(ServiceName.c_str(), ServiceSessions))
	{
		OutContainer->RemoveObject("SESSIONS");
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET_SESSION_LIST, ServiceName);
		return false;
	}

	return true;
}




//SYSTEM SERVICE MOUNT
bool NvSession::ServiceMount(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_MOUNT"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(!CheckAsciiName(ServiceName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_BAD_NAME, ServiceName);
		return false;
	}

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:MOUNT:SERVICE");
		return false;
	}

	// Get variable name that should be in command parameters
	string ServicePath = CommandToExecute->Parameters.GetValue("PATH");
	CleanString(ServicePath);


	// Get variable name that should be in command parameters
	string ServiceDesc = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	CleanString(ServiceDesc);

	// Get variable name that should be in command parameters
	string sMultithread = CommandToExecute->Parameters.GetValue("MULTITHREAD");
	CleanString(sMultithread);
	MakeUpper(sMultithread);
	bool bMultithread = true;
	if(sMultithread == "NO")
		bMultithread = false;

	// Get variable name that should be in command parameters
	string sLoadOnStart = CommandToExecute->Parameters.GetValue("LOAD_ON_START");
	CleanString(sLoadOnStart);
	MakeUpper(sLoadOnStart);
	bool bLoadOnStart = false;
	if(sLoadOnStart == "YES")
		bLoadOnStart = true;


	// Get variable name that should be in command parameters
	string sLanguage = CommandToExecute->Parameters.GetValue("LANGUAGE");
	CleanString(sLanguage);
	MakeUpper(sLanguage);
	int iLanguage = LANGUAGE_CPP;
	if(sLanguage == "JAVA")
		iLanguage = LANGUAGE_JAVA;
	if(sLanguage == "DOTNET")
		iLanguage = LANGUAGE_DOTNET;


	if(!GetMain()->Services.CreateService(ServiceName.c_str(), ServiceDesc.c_str(), ServicePath.c_str(), bMultithread, bLoadOnStart, iLanguage))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_MOUNT, ServiceName);
		return false;
	}


	return true;
}


//SYSTEM SERVICE UNMOUNT
bool NvSession::ServiceUnmount(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_MOUNT"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:UNMOUNT:SERVICE");
		return false;
	}

	// Get wait time option
	string WaitTimeInCommand = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(WaitTimeInCommand);
	MakeUpper(WaitTimeInCommand);
	int iWaitTimeInCommand = 60;	// by default we wait 60 seconds
	if(WaitTimeInCommand.length())
		iWaitTimeInCommand = atoi(WaitTimeInCommand.c_str());
	if(iWaitTimeInCommand < 0)
		iWaitTimeInCommand = 0;	// no wait time


	if(!GetMain()->Services.RemoveService(ServiceName.c_str(), m_sIdent.c_str(), iWaitTimeInCommand))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_UNMOUNT, ServiceName);
		return false;
	}


	return true;
}


//SYSTEM SERVICE SKELETON
bool NvSession::ServiceSkeleton(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_SKELETON"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(!CheckAsciiName(ServiceName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_BAD_NAME, ServiceName);
		return false;
	}

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:SKELETON:SERVICE");
		return false;
	}

	// Get variable name that should be in command parameters
	string sWithSessions = CommandToExecute->Parameters.GetValue("WITH_SESSIONS");
	CleanString(sWithSessions);
	MakeUpper(sWithSessions);
	bool bWithSessions = true;
	if(sWithSessions == "NO")
		bWithSessions = false;


	// Get variable name that should be in command parameters
	string sLanguage = CommandToExecute->Parameters.GetValue("LANGUAGE");
	CleanString(sLanguage);
	MakeUpper(sLanguage);
	int iLanguage = LANGUAGE_CPP;
	if(sLanguage == "JAVA")
		iLanguage = LANGUAGE_JAVA;
	if(sLanguage == "DOTNET")
		iLanguage = LANGUAGE_DOTNET;

	if(!GetMain()->Services.CreateSourceSkeleton(ServiceName.c_str(), bWithSessions, iLanguage, m_sIdent.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_CREATE_SKELETON, ServiceName);
		return false;
	}


	return true;
}


//SYSTEM SERVICE INFO
bool NvSession::ServiceInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LIST"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}

	NvObjTable *ServiceInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));

	// We must update the session list because the time out sessions are checked only each 30 seconds
	GetMain()->Sessions.Update();

	if(!GetMain()->Services.GetInfo(ServiceName.c_str(), ServiceInfo))
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET_INFO, ServiceName);
		return false;
	}

	return true;
}


//SYSTEM SERVICE INFOEX
bool NvSession::ServiceInfoEx(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:INFOEX:SERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SERVICE_INFO");
	if(!OutContainer->CreateObject("SERVICE_INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SERVICE_INFO");
		return false;
	}

	NvObjIndStringList *ServiceInfo = (NvObjIndStringList *)(OutContainer->GetObject("SERVICE_INFO"));

	if(!GetMain()->Services.GetInfoEx(ServiceName.c_str(), ServiceInfo))
	{
		OutContainer->RemoveObject("SERVICE_INFO");
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET_INFO, ServiceName);
		return false;
	}

	return true;
}


//SYSTEM SERVICE CONFIG
bool NvSession::ServiceConfig(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_CONFIG"))
		return false;

	// This command is authorized only from browser
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:SERVICE:CONFIG");
		return false;
	}
	
	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:CONFIG:SERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	

	NvService *Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
	if(Service == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
		return false;
	}

	string ServiceProcedureName = ServiceName;
	ServiceProcedureName += "(Config/config)";


	// Try to execute the procedure but do nothing if it doesn't exist
	if(!ExecuteProcedure(CommandToExecute, ServiceProcedureName, false, "", NULL, NULL, NULL,iEncoding))
	{
		GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(!CommandToExecute->Parameters.KeyExist("NV_XML_XSL"))
	{
		string ServiceXslName = ServiceName;
		ServiceXslName += "(Config/config)";
		CommandToExecute->Parameters.SetValue("NV_XML_XSL", ServiceXslName);		// Service standard xsl file
	}

	GetMain()->Services.FreeService(ServiceName.c_str());
	return true;
}


//SYSTEM SERVICE IDENT
bool NvSession::ServiceIdent(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	char Buff[128];
	sprintf(Buff, "%d%d%d", myRand(), myRand(), myRand());
	string sIdentifier = Buff;
	sIdentifier += ServiceName;
	sIdentifier += m_sIdent;
	sIdentifier = MD5_string(sIdentifier);
	MakeUpper(sIdentifier);
	OutContainer->RemoveObject("IDENTIFIER");
	if(!OutContainer->CreateObject("IDENTIFIER", NV_OBJTYPE_STRING))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "IDENTIFIER");
		return false;
	}

	NvObjString *Object = (NvObjString *)OutContainer->GetObject("IDENTIFIER");
	Object->SetValue(sIdentifier);
	
	
	return true;
}


//SYSTEM SERVICE PACKAGE
bool NvSession::ServicePackage(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_PACKAGE"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:PACKAGE:SERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}
		FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
		FileObject->SetAutoFileName(Application->GetWorkDir());
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	// Get variable name that should be in command parameters
	string DescriptionFile = CommandToExecute->Parameters.GetValue("DESC_FILE");
	CleanString(DescriptionFile);

	string Info = "";
	if(!GetMain()->Services.Package(ServiceName.c_str(), DescriptionFile.c_str(), FileObject->GetFilePathName().c_str(), Info))
	{
		string ErrorMessage = ServiceName;
		ErrorMessage += " ";
		ErrorMessage += Info;
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_CREATE_PACKAGE, ErrorMessage);
		return false;
	}

	return true;
}


//SYSTEM SERVICE INSTALL
bool NvSession::ServiceInstall(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_INSTALL"))
		return false;

	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
		return false;
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	string ServiceNameInPackage = "";
	string PlatformInPackage = "";
	// Get service name and platform from package
	
	NvPackage Package;
	vector<std::string> HeaderLines;
	if(!Package.GetHeaderLines(FileObject->GetFilePathName().c_str(), &HeaderLines))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_INSTALL, Package.Info);
		return false;
	}
	int i;
	string::size_type index;
	for(i=0; i<(int)(HeaderLines.size()); i++)
	{
		string Line = HeaderLines[i];
		index = Line.find('=');
		if(index == string::npos)
			continue;	// Bad header entry
		string EntryName = Line.substr(0, index);
		string EntryValue = Line.substr(index+1);
		CleanString(EntryName);
		CleanString(EntryValue);
		MakeUpper(EntryName);
		MakeUpper(EntryValue);
		if(!ServiceNameInPackage.length())
			if(EntryName == "SERVICE")
				ServiceNameInPackage = EntryValue;
		if(!PlatformInPackage.length())
			if(EntryName == "PLATFORM")
				PlatformInPackage = EntryValue;
	}
	MakeUpper(ServiceNameInPackage);
	CleanString(ServiceNameInPackage);
	MakeUpper(PlatformInPackage);
	CleanString(PlatformInPackage);

	if(!ServiceName.length())
		ServiceName = ServiceNameInPackage;
	if(!CheckAsciiName(ServiceName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_BAD_NAME, ServiceName);
		return false;
	}

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SERVICE:INSTALL:SERVICE");
		return false;
	}

	string Platform = "WIN32";
#ifdef WIN32
#ifdef X64
	Platform = "WIN64";
#endif
#endif
#ifdef AIX
	Platform = "AIX";
#endif
#ifdef LINUX
	Platform = "LINUX";
#ifdef X64
	Platform = "LINUX64";
#endif
#endif
#ifdef HPUX
	Platform = "HPUX";
#endif
#ifdef HPUXI
	Platform = "HPUXI";
#endif
#ifdef SOLARIS
	Platform = "SOLARIS";
#endif

	// Now check the platform
	if(PlatformInPackage.length())
	{
		if(PlatformInPackage != Platform)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_BAD_PLATFORM, PlatformInPackage);
			return false;
		}
	}

	string Info = "";
	string sMountOption = CommandToExecute->Parameters.GetValue("MOUNT");
	CleanString(sMountOption);
	MakeUpper(sMountOption);
	bool bMountOption = false;
	if(sMountOption == "YES")
		bMountOption = true;

	if(!GetMain()->Services.Install(ServiceName.c_str(), FileObject->GetFilePathName().c_str(), bMountOption, Info))
	{
		string ErrorMessage = ServiceName;
		ErrorMessage += " ";
		ErrorMessage += Info;
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_INSTALL, ErrorMessage);
		return false;
	}

	string ServiceProcedureName = ServiceName;
	ServiceProcedureName += "(install)";


	// Try to execute the procedure but do nothing if it doesn't exist
	if(!ExecuteProcedure(CommandToExecute, ServiceProcedureName, false, "", NULL, NULL, NULL, iEncoding))
		return false;

	return true;
}



//SYSTEM SERVICE PROTECT_CONTAINERS
bool NvSession::ServiceProtectContainers(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE)
		return true; // NOOP in this case
	string ServiceNameSource = CommandToExecute->m_sSourceService;
	if(!ServiceNameSource.length())
		return true;	// Should not occur
	NvService *Service = GetMain()->Services.GetService(ServiceNameSource.c_str(), true, true);
	if(Service == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceNameSource);
		return false;
	}
	Service->SetProtectContainers(true);
	GetMain()->Services.FreeService(ServiceNameSource.c_str());

	return true;
}


//SYSTEM WEBSERVICE EXECUTE
bool NvSession::WebserviceExecute(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:EXECUTE:WEBSERVICE");
		GetMain()->Logs.Write("Webservices", "No web service name", "", NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}


	if(!GetMain()->Webservices.WebserviceExist(WebserviceName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_UNKNOWN, WebserviceName);
		GetMain()->Logs.Write("Webservices", "Web service not found", WebserviceName.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}


	if(!GetMain()->Webservices.IsWebserviceEnable(WebserviceName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_AVAILABLE, WebserviceName);
		GetMain()->Logs.Write("Webservices", "Web service is not running", WebserviceName.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}

	string OperationName = CommandToExecute->Parameters.GetValue("OPERATION");
	CleanString(OperationName, true);


	if(!OperationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:EXECUTE:OPERATION");
		GetMain()->Logs.Write("Webservices", "No operation name given", WebserviceName.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}

	string LogMessage = WebserviceName;
	LogMessage += ":";
	LogMessage += OperationName;
	GetMain()->Logs.Write("Webservices", "Running web service", LogMessage.c_str(), NV_LOG_INFO, 1, m_sIdent.c_str());

	NvWebs* WebService = GetMain()->Webservices.GetWebservice(WebserviceName.c_str());
	if(WebService == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_GET, WebserviceName);
		GetMain()->Logs.Write("Webservices", "Cannot get the web service", WebserviceName.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}

	NvWebsContent *WebsContent = WebService->CurrentContent;
	if(!WebsContent->OperationExist(OperationName.c_str()))
	{
		GetMain()->Webservices.FreeWebservice(WebserviceName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_INVALID, OperationName);
		string Message = WebserviceName;
		Message += ":";
		Message += OperationName;
		GetMain()->Logs.Write("Webservices", "Web service operation not found", Message.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}
	
	// Check authorization for executing web service operation
	string PermissionService = "(";
	PermissionService += WebserviceName;
	PermissionService += ")";
	string PermissionToCheck = OperationName;
	MakeUpper(PermissionService);
	if(!CheckPermission(CommandToExecute, PermissionService.c_str(), PermissionToCheck.c_str()))
	{
		// Should not occur
		GetMain()->Webservices.FreeWebservice(WebserviceName.c_str());
		string Message = WebserviceName;
		Message += ":";
		Message += OperationName;
		GetMain()->Logs.Write("Webservices", "Not allowed to run this web service operation (check security)", Message.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}

	string Description;
	string ProcedureName;
	string InMessage;
	string OutMessage;
	if(!WebsContent->OperationGetParam(OperationName.c_str(), Description, ProcedureName, InMessage, OutMessage))
	{
		// Should not occur
		GetMain()->Webservices.FreeWebservice(WebserviceName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_INVALID, OperationName);
		string Message = WebserviceName;
		Message += ":";
		Message += OperationName;
		GetMain()->Logs.Write("Webservices", "Cannot get web service parameters", Message.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
		return false;
	}

	NvContainer InDataContainer;
	NvContainer OutDataContainer;
	WebsContent->MessageGetContent(InMessage.c_str(), &InDataContainer);
	WebsContent->MessageGetContent(OutMessage.c_str(), &OutDataContainer);

	// Create eventual input structure if not ever done
	// This command creates the necessary input message structure into the input container
	InContainer->CopyFrom(&InDataContainer, NV_CONTAINER_COPY_APPEND_REPLACE_IF_BAD_TYPE, NULL, true, false, Application->GetWorkDir().c_str(), 0);

	// If the procedure exist, we execute it
	if(ProcedureName.length())
	{
		string ProcDir = WebService->GetProcDir();
		if(!ExecuteProcedure(CommandToExecute, ProcedureName, true, ProcDir, NULL, NULL, NULL, iEncoding))
		{
			GetMain()->Webservices.FreeWebservice(WebserviceName.c_str());
			string Message = WebserviceName;
			Message += ":";
			Message += OperationName;
			Message += ":";
			Message += CommandToExecute->Error.m_sErrorService.c_str();
			Message += " - ";
			Message += CommandToExecute->Error.m_sErrorClass.c_str();
			char sBuf[20];
			sprintf(sBuf, "%d", CommandToExecute->Error.m_iErrorCode);
			Message += " - ";
			Message += sBuf;
			Message += " - ";
			Message += CommandToExecute->Error.GetErrorDesc(CommandToExecute->m_sLanguage.c_str()).c_str();
			Message += " - ";
			Message += CommandToExecute->Error.m_sErrorInfo.c_str();
			GetMain()->Logs.Write("Webservices", "Error in web service procedure", Message.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
			return false;
		}
	}

	// Now we must create the output objects if not ever done
	OutContainer->CopyFrom(&OutDataContainer, NV_CONTAINER_COPY_APPEND_REPLACE_IF_BAD_TYPE, NULL, true, false, Application->GetWorkDir().c_str(), 0);

	bool bSetXmlOutput = false;
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
		bSetXmlOutput = true;
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
		if(CommandToExecute->iXmlInputType)
			bSetXmlOutput = true;
	if(bSetXmlOutput)
	{
		if(!SetXmlOutput(CommandToExecute, false, &OutDataContainer, OutMessage, WebService->GetName()))
		{
			GetMain()->Webservices.FreeWebservice(WebserviceName.c_str());
			string Message = WebserviceName;
			Message += ":";
			Message += OperationName;
			GetMain()->Logs.Write("Webservices", "Cannot set xml output", Message.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str());
			return false;
		}
	}

	// Free the web service
	GetMain()->Webservices.FreeWebservice(WebserviceName.c_str());

	// Block the standard XML output since we directly do it here
	CommandToExecute->iOutputType = OUTPUT_BINARY;
	
	GetMain()->Logs.Write("Webservices", "Web service successful", LogMessage.c_str(), NV_LOG_INFO, 2, m_sIdent.c_str());

	return true;
}

//SYSTEM WEBSERVICE REMOVE
bool NvSession::WebserviceRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_REMOVE"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:REMOVE:WEBSERVICE");
		return false;
	}


	if(!GetMain()->Webservices.RemoveWebservice(WebserviceName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_REMOVE, WebserviceName);
		return false;
	}


	return true;
}


//SYSTEM WEBSERVICE START
bool NvSession::WebserviceStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_RUN"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:START:WEBSERVICE");
		return false;
	}

	if(!GetMain()->Webservices.EnableWebservice(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_START, WebserviceName);
		return false;
	}


	return true;
}


//SYSTEM WEBSERVICE STOP
bool NvSession::WebserviceStop(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_RUN"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:STOP:WEBSERVICE");
		return false;
	}

	int iWaitTime = 10;
	string sWaitTime = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(sWaitTime);
	if(sWaitTime.length())
		iWaitTime = atoi(sWaitTime.c_str());
	if(iWaitTime < 0)
		iWaitTime = 0;

	if(!GetMain()->Webservices.DisableWebservice(WebserviceName.c_str(), iWaitTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_STOP, WebserviceName);
		return false;
	}


	return true;
}


//SYSTEM WEBSERVICE INFO
bool NvSession::WebserviceInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_LIST"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}

	NvObjTable *WebserviceInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));

	string Host = GetMain()->GetHostWebs();

	if(!GetMain()->Webservices.GetInfo(WebserviceName.c_str(), WebserviceInfo, this, Host))
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_GET_INFO, WebserviceName);
		return false;
	}

	return true;
}


//SYSTEM WEBSERVICE INFOEX
bool NvSession::WebserviceInfoEx(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:INFOEX:WEBSERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("WEBSERVICE_INFO");
	if(!OutContainer->CreateObject("WEBSERVICE_INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "WEBSERVICE_INFO");
		return false;
	}

	NvObjIndStringList *WebserviceInfo = (NvObjIndStringList *)(OutContainer->GetObject("WEBSERVICE_INFO"));

	if(!GetMain()->Webservices.GetInfoEx(WebserviceName.c_str(), WebserviceInfo))
	{
		OutContainer->RemoveObject("WEBSERVICE_INFO");
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_GET_INFO, WebserviceName);
		return false;
	}

	return true;
}


//SYSTEM WEBSERVICE CREATE
bool NvSession::WebserviceCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_CREATE"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);
	if(!CheckAsciiName(WebserviceName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_BAD_NAME, WebserviceName);
		return false;
	}

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:CREATE:WEBSERVICE");
		return false;
	}

	string WebserviceDesc = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	string WebserviceNsPrefix = CommandToExecute->Parameters.GetValue("NSPREFIX");
	string WebserviceHttpsOnly = CommandToExecute->Parameters.GetValue("HTTPS_ONLY");
	CleanString(WebserviceHttpsOnly);
	MakeUpper(WebserviceHttpsOnly);
	bool bWebserviceHttpsOnly = false;
	if(WebserviceHttpsOnly == "YES")
		bWebserviceHttpsOnly = true;
	string WebserviceUseDefaultPath = CommandToExecute->Parameters.GetValue("USE_DEFAULT_PATH");
	CleanString(WebserviceUseDefaultPath);
	MakeUpper(WebserviceUseDefaultPath);
	bool bWebserviceUseDefaultPath = true;
	if(WebserviceUseDefaultPath == "NO")
		bWebserviceUseDefaultPath = false;
	string WebservicePath = CommandToExecute->Parameters.GetValue("PATH");
	CleanString(WebservicePath, true);

	if(!GetMain()->Webservices.CreateWebservice(WebserviceName.c_str(), WebserviceDesc.c_str(), WebserviceNsPrefix.c_str(), bWebserviceHttpsOnly, bWebserviceUseDefaultPath, WebservicePath.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_CREATE, WebserviceName);
		return false;
	}

	return true;
}


//SYSTEM WEBSERVICE PACKAGE
bool NvSession::WebservicePackage(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_PACKAGE"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:PACKAGE:WEBSERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}
		FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
		FileObject->SetAutoFileName(Application->GetWorkDir());
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	// Get variable name that should be in command parameters
	string DescriptionFile = CommandToExecute->Parameters.GetValue("DESC_FILE");
	CleanString(DescriptionFile);

	string Info = "";
	if(!GetMain()->Webservices.Package(WebserviceName.c_str(), DescriptionFile.c_str(), FileObject->GetFilePathName().c_str(), Info))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += Info;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_CREATE_PACKAGE, ErrorMessage);
		return false;
	}

	return true;
}


//SYSTEM WEBSERVICE INSTALL
bool NvSession::WebserviceInstall(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_INSTALL"))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
		return false;
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	string WebserviceNameInPackage = "";
	string PlatformInPackage = "";
	// Get Webservice name and platform from package
	
	NvPackage Package;
	vector<std::string> HeaderLines;
	if(!Package.GetHeaderLines(FileObject->GetFilePathName().c_str(), &HeaderLines))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_INSTALL, Package.Info);
		return false;
	}
	int i;
	string::size_type index;
	for(i=0; i<(int)(HeaderLines.size()); i++)
	{
		string Line = HeaderLines[i];
		index = Line.find('=');
		if(index == string::npos)
			continue;	// Bad header entry
		string EntryName = Line.substr(0, index);
		string EntryValue = Line.substr(index+1);
		CleanString(EntryName);
		CleanString(EntryValue);
		MakeUpper(EntryName);
		MakeUpper(EntryValue);
		if(!WebserviceNameInPackage.length())
			if(EntryName == "WEBSERVICE")
				WebserviceNameInPackage = EntryValue;
		if(!PlatformInPackage.length())
			if(EntryName == "PLATFORM")
				PlatformInPackage = EntryValue;
	}
	MakeUpper(WebserviceNameInPackage);
	CleanString(WebserviceNameInPackage);
	MakeUpper(PlatformInPackage);
	CleanString(PlatformInPackage);

	if(!WebserviceName.length())
		WebserviceName = WebserviceNameInPackage;

	if(!CheckAsciiName(WebserviceName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_BAD_NAME, WebserviceName);
		return false;
	}
	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:INSTALL:WEBSERVICE");
		return false;
	}

	string Platform = "WIN32";
#ifdef WIN32
#ifdef X64
	Platform = "WIN64";
#endif
#endif
#ifdef AIX
	Platform = "AIX";
#endif
#ifdef LINUX
	Platform = "LINUX";
#ifdef X64
	Platform = "LINUX64";
#endif
#endif
#ifdef HPUX
	Platform = "HPUX";
#endif
#ifdef HPUXI
	Platform = "HPUXI";
#endif
#ifdef SOLARIS
	Platform = "SOLARIS";
#endif

	// Now check the platform
	if(PlatformInPackage.length())
	{
		if(PlatformInPackage != Platform)
		{
			CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_BAD_PLATFORM, PlatformInPackage);
			return false;
		}
	}


	string Info = "";

	if(!GetMain()->Webservices.Install(WebserviceName.c_str(), FileObject->GetFilePathName().c_str(), Info))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += Info;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_INSTALL, ErrorMessage);
		return false;
	}

	return true;
}



//SYSTEM WEBSERVICE SET_EDIT_MODE
bool NvSession::WebserviceSetEditMode(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	bool bSetMode = false;
	string sSetMode = CommandToExecute->Parameters.GetValue("MODE");
	CleanString(sSetMode);
	MakeUpper(sSetMode);
	if(sSetMode == "ON")
		bSetMode = true;

	if(bSetMode)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_EDIT"))
			return false;

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:EDIT_UPDATE:WEBSERVICE");
		return false;
	}

	if(!GetMain()->Webservices.SetEditMode(WebserviceName.c_str(), this, bSetMode))
	{
		if(bSetMode)
		{
			CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_EDIT, WebserviceName);
			return false;
		}
	}

	return true;
}



//SYSTEM WEBSERVICE EDIT_UPDATE
bool NvSession::WebserviceEditUpdate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_EDIT"))
		return false;
	
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:EDIT_UPDATE:WEBSERVICE");
		return false;
	}

	string Info = "";
	if(!GetMain()->Webservices.EditUpdate(WebserviceName.c_str(), this, Info))
	{
		string MessageInfo = WebserviceName;
		MessageInfo += ":";
		MessageInfo += Info;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_UPDATE, MessageInfo);
		return false;
	}

	return true;
}



//SYSTEM WEBSERVICE IS_EDIT
bool NvSession::WebserviceIsEdit(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:IS_EDIT:WEBSERVICE");
		return false;
	}

	char Buf[20];
	strcpy(Buf, "NO");

	if(GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
		strcpy(Buf, "YES");
	
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//SYSTEM WEBSERVICE CAN_EDIT
bool NvSession::WebserviceCanEdit(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);

	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:CAN_EDIT:WEBSERVICE");
		return false;
	}

	char Buf[20];
	strcpy(Buf, "NO");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_EDIT"))
		return true;

	if(GetMain()->Webservices.CanEdit(WebserviceName.c_str(), this))
		strcpy(Buf, "YES");
	
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//SYSTEM WEBSERVICE OPERATION_LIST
bool NvSession::WebserviceOperationList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_LIST:WEBSERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("OPERATIONS");
	if(!OutContainer->CreateObject("OPERATIONS", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "OPERATIONS");
		return false;
	}

	NvObjTable *OpList = (NvObjTable *)(OutContainer->GetObject("OPERATIONS"));

	if(!GetMain()->Webservices.OperationList(WebserviceName.c_str(), OpList, this))
	{
		OutContainer->RemoveObject("OPERATIONS");
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_GET_INFO, WebserviceName);
		return false;
	}

	return true;
}






//SYSTEM WEBSERVICE OPERATION_ENABLE
bool NvSession::WebserviceOperationEnable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_ENABLE:WEBSERVICE");
		return false;
	}
	
	string OperationName = CommandToExecute->Parameters.GetValue("OPERATION");
	CleanString(OperationName, true);


	if(!OperationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_ENABLE:OPERATION");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	bool bEnable = false;
	string sEnable = CommandToExecute->Parameters.GetValue("ENABLE");
	CleanString(sEnable, true);
	MakeUpper(sEnable);
	if(sEnable == "YES")
		bEnable = true;


	if(!GetMain()->Webservices.OperationEnable(WebserviceName.c_str(), OperationName.c_str(), bEnable))
	{
		if(bEnable)
			CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_ENABLE, WebserviceName);
		else
			CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_DISABLE, WebserviceName);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE OPERATION_CREATE
bool NvSession::WebserviceOperationCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_CREATE:WEBSERVICE");
		return false;
	}
	
	string OperationName = CommandToExecute->Parameters.GetValue("OPERATION");
	CleanString(OperationName, true);


	if(!OperationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_CREATE:OPERATION");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	string InMessage = CommandToExecute->Parameters.GetValue("IN");
	CleanString(InMessage, true);
	string OutMessage = CommandToExecute->Parameters.GetValue("OUT");
	CleanString(OutMessage, true);
	string Procedure = CommandToExecute->Parameters.GetValue("PROCEDURE");
	CleanString(Procedure, true);

	if(!GetMain()->Webservices.OperationCreate(WebserviceName.c_str(), OperationName.c_str(), Description.c_str(), Procedure.c_str(),InMessage.c_str(),OutMessage.c_str()))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += OperationName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_CREATE, ErrorMessage);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE OPERATION_REMOVE
bool NvSession::WebserviceOperationRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_REMOVE:WEBSERVICE");
		return false;
	}
	
	string OperationName = CommandToExecute->Parameters.GetValue("OPERATION");
	CleanString(OperationName, true);


	if(!OperationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_REMOVE:OPERATION");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	if(!GetMain()->Webservices.OperationRemove(WebserviceName.c_str(), OperationName.c_str()))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += OperationName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_REMOVE, ErrorMessage);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE OPERATION_SET_PARAM
bool NvSession::WebserviceOperationSetParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_SET_PARAM:WEBSERVICE");
		return false;
	}
	
	string OperationName = CommandToExecute->Parameters.GetValue("OPERATION");
	CleanString(OperationName, true);


	if(!OperationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_SET_PARAM:OPERATION");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	string sDescription = "";
	string sInMessage = "";
	string sOutMessage = "";
	string sProcedure = "";
	const char *Description = NULL;
	const char *InMessage = NULL;
	const char *OutMessage = NULL;
	const char *Procedure = NULL;
	
	CommandToExecute->Parameters.KeyExist("DESCRIPTION");
	{
		sDescription = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		Description = sDescription.c_str();
	}
	CommandToExecute->Parameters.KeyExist("IN");
	{
		sInMessage = CommandToExecute->Parameters.GetValue("IN");
		InMessage = sInMessage.c_str();
	}
	CommandToExecute->Parameters.KeyExist("OUT");
	{
		sOutMessage = CommandToExecute->Parameters.GetValue("OUT");
		OutMessage = sOutMessage.c_str();
	}
	if(sInMessage.length() && (sInMessage == sOutMessage))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += ":";
		ErrorMessage += OperationName;
		ErrorMessage += ": The input and output messages must have different names";
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_SETPARAM, ErrorMessage);
		return false;
	}

	CommandToExecute->Parameters.KeyExist("PROCEDURE");
	{
		sProcedure = CommandToExecute->Parameters.GetValue("PROCEDURE");
		Procedure = sProcedure.c_str();
	}

	if(!GetMain()->Webservices.OperationSetParam(WebserviceName.c_str(), OperationName.c_str(), Description, Procedure, InMessage, OutMessage))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += OperationName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_SETPARAM, ErrorMessage);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE OPERATION_SOAP
bool NvSession::WebserviceOperationSoap(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_SOAP:WEBSERVICE");
		return false;
	}
	
	string OperationName = CommandToExecute->Parameters.GetValue("OPERATION");
	CleanString(OperationName, true);


	if(!OperationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:OPERATION_SOAP:OPERATION");
		return false;
	}


	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SOAP");
	if(!OutContainer->CreateObject("SOAP", NV_OBJTYPE_BINARY))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SOAP");
		return false;
	}

	NvObjBinary *Output = (NvObjBinary *)(OutContainer->GetObject("SOAP"));

	string AppName = Application->GetAppNameForFiles();
	string Host = GetMain()->GetHostWebs();

	if(!GetMain()->Webservices.OperationSoapIn(WebserviceName.c_str(), OperationName.c_str(), Output, this, AppName, Host))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += OperationName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_OPERATION_CANNOT_SET_SOAP, ErrorMessage);
		return false;
	}

	return true;
}



//SYSTEM WEBSERVICE MESSAGE_LIST
bool NvSession::WebserviceMessageList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_LIST:WEBSERVICE");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("MESSAGES");
	if(!OutContainer->CreateObject("MESSAGES", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "MESSAGES");
		return false;
	}

	NvObjTable *MsgList = (NvObjTable *)(OutContainer->GetObject("MESSAGES"));


	if(!GetMain()->Webservices.MessageList(WebserviceName.c_str(), MsgList, this))
	{
		OutContainer->RemoveObject("MESSAGES");
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_GET_INFO, WebserviceName);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE MESSAGE_CREATE
bool NvSession::WebserviceMessageCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_CREATE:WEBSERVICE");
		return false;
	}
	
	string MessageName = CommandToExecute->Parameters.GetValue("MESSAGE");
	CleanString(MessageName, true);


	if(!MessageName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_CREATE:MESSAGE");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");

	if(!GetMain()->Webservices.MessageCreate(WebserviceName.c_str(), MessageName.c_str(), Description.c_str()))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += MessageName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_MESSAGE_CANNOT_CREATE, ErrorMessage);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE MESSAGE_REMOVE
bool NvSession::WebserviceMessageRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_REMOVE:WEBSERVICE");
		return false;
	}
	
	string MessageName = CommandToExecute->Parameters.GetValue("MESSAGE");
	CleanString(MessageName, true);


	if(!MessageName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_REMOVE:MESSAGE");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	if(!GetMain()->Webservices.MessageRemove(WebserviceName.c_str(), MessageName.c_str()))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += MessageName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_MESSAGE_CANNOT_REMOVE, ErrorMessage);
		return false;
	}

	return true;
}





//SYSTEM WEBSERVICE MESSAGE_SET_PARAM
bool NvSession::WebserviceMessageSetParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_SET_PARAM:WEBSERVICE");
		return false;
	}
		
	string MessageName = CommandToExecute->Parameters.GetValue("MESSAGE");
	CleanString(MessageName, true);


	if(!MessageName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_SET_PARAM:MESSAGE");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	string sDescription = "";
	const char *Description = NULL;
	
	CommandToExecute->Parameters.KeyExist("DESCRIPTION");
	{
		sDescription = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		Description = sDescription.c_str();
	}

	if(!GetMain()->Webservices.MessageSetParam(WebserviceName.c_str(), MessageName.c_str(), Description))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += MessageName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_MESSAGE_CANNOT_SETPARAM, ErrorMessage);
		return false;
	}

	return true;
}



//SYSTEM WEBSERVICE MESSAGE_SET_CONTENT
bool NvSession::WebserviceMessageSetContent(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_SET_CONTENT:WEBSERVICE");
		return false;
	}
		
	string MessageName = CommandToExecute->Parameters.GetValue("MESSAGE");
	CleanString(MessageName, true);


	if(!MessageName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_SET_CONTENT:MESSAGE");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}


	if(!GetMain()->Webservices.MessageSetContent(WebserviceName.c_str(), MessageName.c_str(), InContainer))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += MessageName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_MESSAGE_CANNOT_SET_CONTENT, ErrorMessage);
		return false;
	}

	return true;
}



//SYSTEM WEBSERVICE MESSAGE_GET_CONTENT
bool NvSession::WebserviceMessageGetContent(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_GET_CONTENT:WEBSERVICE");
		return false;
	}
		
	string MessageName = CommandToExecute->Parameters.GetValue("MESSAGE");
	CleanString(MessageName, true);


	if(!MessageName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:MESSAGE_GET_CONTENT:MESSAGE");
		return false;
	}


	if(!GetMain()->Webservices.MessageGetContent(WebserviceName.c_str(), MessageName.c_str(), OutContainer, this))
	{
		string ErrorMessage = WebserviceName;
		ErrorMessage += " ";
		ErrorMessage += MessageName;
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_MESSAGE_CANNOT_GET_CONTENT, ErrorMessage);
		return false;
	}

	return true;
}



//SYSTEM WEBSERVICE GET_HEADERS
bool NvSession::WebserviceGetHeaders(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:GET_HEADERS:WEBSERVICE");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	string Headers = GetMain()->Webservices.GetHeaders(WebserviceName.c_str(), this);
	GetStringValue(CommandToExecute, Headers.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//SYSTEM WEBSERVICE SET_HEADERS
bool NvSession::WebserviceSetHeaders(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:SET_HEADERS:WEBSERVICE");
		return false;
	}

	if(!GetMain()->Webservices.IsInEdit(WebserviceName.c_str(), this))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_IN_EDIT, WebserviceName);
		return false;
	}

	string Headers = CommandToExecute->Parameters.GetValue("HEADERS");
	CleanString(Headers, true);
	MakeLower(Headers);


	GetMain()->Webservices.SetHeaders(WebserviceName.c_str(), Headers.c_str());

	return true;
}


//SYSTEM WEBSERVICE WSDL
bool NvSession::WebserviceWsdl(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string WebserviceName = CommandToExecute->Parameters.GetValue("WEBSERVICE");
	CleanString(WebserviceName);
	MakeUpper(WebserviceName);


	if(!WebserviceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "WEBSERVICE:WSDL:WEBSERVICE");
		return false;
	}

	if(CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		// The function uses the output container
		if(!CheckOutputContainer(CommandToExecute))
			return false;
	}
	
	string AppName = Application->GetAppNameForFiles();
	string Host = GetMain()->GetHostWebs();

	string::size_type index;
	if(CommandToExecute->m_sHost.length())
	{
		while((index = Host.find("nv_dynamic_host")) != string::npos)
			Host.replace(index, 15,CommandToExecute->m_sHost);
	}

	NvObjBinary WsdlBinary;

	if(!GetMain()->Webservices.SetWsdl(WebserviceName.c_str(), AppName, Host, &WsdlBinary))
	{
		CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_SET_WSDL, WebserviceName);
		return false;
	}

	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
	{
		// The WSDL output is directly sent to the browser
		// This command must block the standard xml output
		CommandToExecute->iOutputType = OUTPUT_BINARY;
		CommandToExecute->Data = WsdlBinary.m_cdata;
		CommandToExecute->DataSize = WsdlBinary.m_isize;
		WsdlBinary.m_cdata = NULL;
		WsdlBinary.m_isize = 0;

		CommandToExecute->HttpOutHeaders.SetValue("Cache-Control", "no-cache");
		CommandToExecute->HttpOutHeaders.SetValue("Pragma", "no-cache");
		CommandToExecute->HttpOutHeaders.SetValue("Expires", "Wed, 26 Feb 1997 08:21:57 GMT"); // old date

		CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "text/xml" );
	}
	else
	{
		// Get variable name that should be in command parameters
		string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
		CleanString(FileObjectName);
		MakeUpper(FileObjectName);

		if(!FileObjectName.length())
			FileObjectName = "WSDL";

		OutContainer->RemoveObject(FileObjectName.c_str());
		if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}

		NvObjFile *WebserviceWsdl = (NvObjFile *)(OutContainer->GetObject(FileObjectName.c_str()));
		WebserviceWsdl->SetAutoFileName(Application->GetWorkDir(), "wsdl");

		string FileName = WebserviceWsdl->GetFilePathName();
		WsdlBinary.SaveToFile(FileName.c_str());
	}

	return true;
}


//SYSTEM LICENSE GET
bool NvSession::LicenseGet(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	if(!ServiceName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LICENSE:GET:SERVICE");
		return false;
	}
	
	// Get variable name that should be in command parameters
	string ServiceId = CommandToExecute->Parameters.GetValue("SERVICE_ID");
	CleanString(ServiceId);
	MakeUpper(ServiceId);

	if(!ServiceId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LICENSE:GET:SERVICE_ID");
		return false;
	}
	
	// Get variable name that should be in command parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeUpper(Key);

	if(!Key.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LICENSE:GET:KEY");
		return false;
	}

	string Value = "";
	if(!GetMain()->Licenses.CheckKey(ServiceName, ServiceId, Key, Value))
	{
		string ErrorMessage = ServiceName;
		ErrorMessage += ":";
		ErrorMessage += Key;
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_BAD_KEY, ErrorMessage);
		return false;
	}

	GetStringValue(CommandToExecute, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//SYSTEM LICENSE SET
bool NvSession::LicenseSet(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "LICENSE_UPDATE"))
		return false;

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	if(!GetMain()->IsLcAvailable())
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_SET_KEY, "License manager is not available");
		return false;
	}

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "LICENSE_FILE";

	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		if(!InContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}
		FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
		FileObject->SetAutoFileName(Application->GetWorkDir());
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	string Key = CommandToExecute->Parameters.GetValue("KEY");
	if(Key.length() > 40)
		Key.erase(0, 40);

	NvLicense License;
	if(!License.Open())
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_SET_KEY, "Cannot open license library");
		return false;
	}

	License.SetLicenseFile(FileObject->GetFilePathName());
	string MachineId = CommandToExecute->Parameters.GetValue("MACHINE_ID");
	if(MachineId.length() > 40)
		MachineId.erase(0, 40);
	string ServiceId = CommandToExecute->Parameters.GetValue("SERVICE_ID");
	if(ServiceId.length() > 40)
		ServiceId.erase(0, 40);
	string Service = CommandToExecute->Parameters.GetValue("SERVICE");
	if(Service.length() > 40)
		Service.erase(0, 40);
	string Value = CommandToExecute->Parameters.GetValue("VALUE");
	if(Value.length() > 40)
		Value.erase(0, 40);
	string Date = CommandToExecute->Parameters.GetValue("DATE");
	if(Date.length() > 40)
		Date.erase(0, 40);

	if(!License.SetKey(MachineId, Service, ServiceId, Key, Value, Date))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_SET_KEY, Key);
		return false;
	}

	return true;
}


//SYSTEM LICENSE REMOVE
bool NvSession::LicenseRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "LICENSE_UPDATE"))
		return false;

	if(!GetMain()->IsLcAvailable())
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_REMOVE_KEY, "License manager is not available");
		return false;
	}

	// Get variable name that should be in command parameters
	string ServiceId = CommandToExecute->Parameters.GetValue("SERVICE_ID");
	CleanString(ServiceId);
	MakeUpper(ServiceId);

	if(!ServiceId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LICENSE:REMOVE:SERVICE_ID");
		return false;
	}
	
	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);
	
	string Key = CommandToExecute->Parameters.GetValue("KEY");

	NvLicense LocalLicense;
	if(!LocalLicense.Open())
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_REMOVE_KEY, "Cannot open license library");
		return false;
	}
	NvLicense *License =&(GetMain()->Licenses);


	if(FileObjectName.length())
	{
		// The function uses the input container
		if(!CheckInputContainer(CommandToExecute))
			return false;
	
		NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
		if(FileObject == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
			return false;
		}
		if(FileObject->GetType() != NV_OBJTYPE_FILE)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
			return false;
		}
		LocalLicense.SetLicenseFile(FileObject->GetFilePathName());
		LocalLicense.Load();
		License = &LocalLicense;
	}

	string Service = CommandToExecute->Parameters.GetValue("SERVICE");

	if(!License->RemoveKey(Service, ServiceId, Key))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_REMOVE_KEY, Key);
		return false;
	}

	return true;
}


//SYSTEM LICENSE INSTALL
bool NvSession::LicenseInstall(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "LICENSE_UPDATE"))
		return false;

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "LICENSE_FILE";

	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
		return false;
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	NvLicense *License =&(GetMain()->Licenses);
	if(!License->Import(FileObject->GetFilePathName()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_IMPORT, FileObject->GetFilePathName());
		return false;
	}

	return true;
}


//SYSTEM LICENSE EXPORT
bool NvSession::LicenseExport(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "LICENSE_FILE";

	NvObjFile *FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}
		FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
		FileObject->SetAutoFileName(Application->GetWorkDir());
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	string Service = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(Service);
	MakeUpper(Service);

	NvLicense *License =&(GetMain()->Licenses);
	if(!License->Export(FileObject->GetFilePathName(), Service))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_EXPORT, FileObject->GetFilePathName());
		return false;
	}

	return true;
}


//SYSTEM LICENSE INFO
bool NvSession::LicenseInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	if(!GetMain()->IsLcAvailable())
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_GET_INFO, "License manager is not available");
		return false;
	}

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	NvLicense LocalLicense;
	if(!LocalLicense.Open())
	{
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_GET_INFO, "Cannot open license library");
		return false;
	}
	NvLicense *License =&(GetMain()->Licenses);

	// Get variable name that should be in command parameters
	string Service = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(Service);
	MakeUpper(Service);


	if(FileObjectName.length())
	{
		// The function uses the input container
		if(!CheckInputContainer(CommandToExecute))
			return false;
	
		NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
		if(FileObject == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
			return false;
		}
		if(FileObject->GetType() != NV_OBJTYPE_FILE)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
			return false;
		}
		LocalLicense.SetLicenseFile(FileObject->GetFilePathName());
		LocalLicense.SetMachineKey("");
		LocalLicense.Load();
		License = &LocalLicense;
	}

	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}
	NvObjTable *LicenseInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));

	OutContainer->RemoveObject("MACHINE_ID");
	if(!OutContainer->CreateObject("MACHINE_ID", NV_OBJTYPE_STRING))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "MACHINE_ID");
		return false;
	}
	NvObjString *MachineId = (NvObjString *)(OutContainer->GetObject("MACHINE_ID"));

	if(!License->GetInfo(Service.c_str(), LicenseInfo, MachineId))
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "LICENSE", NV_LICENSE_CANNOT_GET_INFO, Service);
		return false;
	}

	return true;
}




//SYSTEM LICENSE ISDEMO (here only for compatibility reason, forced to no)
bool NvSession::LicenseIsDemo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	char Buf[20];
	strcpy(Buf, "NO");
	
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}




//SYSTEM LICENSE CHECK_EXPIRE_SOON (here only for compatibility reason, forced to no)
bool NvSession::LicenseCheckExpireSoon(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char Buf[20];
	strcpy(Buf, "NO");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}



//SYSTEM LICENSE GET_TYPE (stays for compatibility reason, forced to PROD)
bool NvSession::LicenseGetType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	int LicenseType = GetMain()->GetLicenseType();
	string Result = "UNKNOWN";
	if(LicenseType == LICENSE_DEMO)
		Result = "DEMO";
	else if(LicenseType == LICENSE_DEV)
		Result = "DEV";
	else if(LicenseType == LICENSE_TEST)
		Result = "TEST";
	else if(LicenseType == LICENSE_PROD)
		Result = "PROD";
	else if(LicenseType == LICENSE_FAIL)
		Result = "FAIL";

	GetStringValue(CommandToExecute, Result.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}


//SYSTEM PACKAGE PACKAGE
bool NvSession::PackagePackage(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_PACKAGE"))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}
		FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
		FileObject->SetAutoFileName(Application->GetWorkDir());
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	// Get variable name that should be in command parameters
	string DescriptionFile = CommandToExecute->Parameters.GetValue("DESC_FILE");
	CleanString(DescriptionFile);
	string DescriptionFileHere = GetMain()->GetNirvaDir();
	DescriptionFileHere += "Files";
	AddDirSuffix(DescriptionFileHere);
	if(DescriptionFile.length())
		DescriptionFileHere += DescriptionFile;
	else
		DescriptionFileHere += "package.lst";

	NvPackage Package;
	bool RetVal = Package.Check(DescriptionFileHere.c_str(), GetMain()->GetNirvaDir().c_str());
	if(RetVal)
		RetVal = Package.Create(DescriptionFileHere.c_str(), GetMain()->GetNirvaDir().c_str(), FileObject->GetFilePathName().c_str());
	if(!RetVal)
		CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_CANNOT_CREATE_PACKAGE, Package.Info);

	return RetVal;
}


//SYSTEM PACKAGE INSTALL
bool NvSession::PackageInstall(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
		return false;
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	NvPackage Package;

	// Get header lines from package
	vector<std::string> HeaderLines;
	if(!Package.GetHeaderLines(FileObject->GetFilePathName().c_str(), &HeaderLines))
	{
		CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_CANNOT_INSTALL, Package.Info);
		return false;
	}
	string AppNameInPackage = "";
	string SrvNameInPackage = "";
	string WebsNameInPackage = "";
	string PlatformInPackage = "";
	int i;
	string::size_type index;
	for(i=0; i<(int)(HeaderLines.size()); i++)
	{
		string Line = HeaderLines[i];
		index = Line.find('=');
		if(index == string::npos)
			continue;	// Bad header entry
		string EntryName = Line.substr(0, index);
		string EntryValue = Line.substr(index+1);
		CleanString(EntryName);
		CleanString(EntryValue);
		MakeUpper(EntryName);
		MakeUpper(EntryValue);
		if(!AppNameInPackage.length())
			if(EntryName == "APPLICATION")
				AppNameInPackage = EntryValue;
		if(!SrvNameInPackage.length())
			if(EntryName == "SERVICE")
				SrvNameInPackage = EntryValue;
		if(!WebsNameInPackage.length())
			if(EntryName == "WEBSERVICE")
				WebsNameInPackage = EntryValue;
		if(!PlatformInPackage.length())
			if(EntryName == "PLATFORM")
				PlatformInPackage = EntryValue;
	}

	bool bOkForInstall = true;
	if(AppNameInPackage.length() && SrvNameInPackage.length())
		bOkForInstall = false;
	if(AppNameInPackage.length() && WebsNameInPackage.length())
		bOkForInstall = false;
	if(SrvNameInPackage.length() && WebsNameInPackage.length())
		bOkForInstall = false;
	if(!bOkForInstall)
	{
		CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_CANNOT_INSTALL, "Cannot find package type (application service or web service)");
		return false;
	}

	string Platform = "WIN32";
#ifdef WIN32
#ifdef X64
	Platform = "WIN64";
#endif
#endif
#ifdef AIX
	Platform = "AIX";
#endif
#ifdef LINUX
	Platform = "LINUX";
#ifdef X64
	Platform = "LINUX64";
#endif
#endif
#ifdef HPUX
	Platform = "HPUX";
#endif
#ifdef HPUXI
	Platform = "HPUXI";
#endif
#ifdef SOLARIS
	Platform = "SOLARIS";
#endif

	// Now check the platform
	if(PlatformInPackage.length())
	{
		if(PlatformInPackage != Platform)
		{
			CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_BAD_PLATFORM, PlatformInPackage);
			return false;
		}
	}

	if(AppNameInPackage.length())
	{
		// Install an application package
		string Info = "";

		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_INSTALL"))
			return false;
	
		if(!CheckAsciiName(AppNameInPackage))
		{
			CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_BAD_NAME, AppNameInPackage);
			return false;
		}
		if(!GetMain()->Applications.Install(AppNameInPackage.c_str(), FileObject->GetFilePathName().c_str(), Info))
		{
			string ErrorMessage = AppNameInPackage;
			ErrorMessage += " ";
			ErrorMessage += Info;
			CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_INSTALL, ErrorMessage);
			return false;
		}
		return true;
	}

	if(WebsNameInPackage.length())
	{
		// Install a web service package
		string Info = "";

		if(!CheckPermission(CommandToExecute, "SYSTEM", "WEBSERVICE_INSTALL"))
			return false;
	
		if(!CheckAsciiName(WebsNameInPackage))
		{
			CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_BAD_NAME, WebsNameInPackage);
			return false;
		}
		if(!GetMain()->Webservices.Install(WebsNameInPackage.c_str(), FileObject->GetFilePathName().c_str(), Info))
		{
			string ErrorMessage = WebsNameInPackage;
			ErrorMessage += " ";
			ErrorMessage += Info;
			CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_INSTALL, ErrorMessage);
			return false;
		}
		return true;
	}

	if(SrvNameInPackage.length())
	{
		// Install a service package
		string Info = "";

		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_INSTALL"))
			return false;

		if(!CheckAsciiName(SrvNameInPackage))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_BAD_NAME, SrvNameInPackage);
			return false;
		}
		if(!GetMain()->Services.Install(SrvNameInPackage.c_str(), FileObject->GetFilePathName().c_str(), true, Info))
		{
			string ErrorMessage = SrvNameInPackage;
			ErrorMessage += " ";
			ErrorMessage += Info;
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_INSTALL, ErrorMessage);
			return false;
		}

		string ServiceProcedureName = SrvNameInPackage;
		ServiceProcedureName += "(install)";


		// Try to execute the procedure but do nothing if it doesn't exist
		if(!ExecuteProcedure(CommandToExecute, ServiceProcedureName, false, "", NULL, NULL, NULL, iEncoding))
			return false;

		return true;
	}

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_INSTALL"))
		return false;
	
	// In other cases, install a system package

	// Empty the xslt cache
	NvXml::EmptyCache();

	bool RetVal = Package.Install(FileObject->GetFilePathName().c_str(), GetMain()->GetNirvaDir().c_str(), GetMain()->GetDebug());
	if(!RetVal)
		CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_CANNOT_INSTALL, Package.Info);

	return RetVal;
}


//SYSTEM PACKAGE INFO
bool NvSession::PackageInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	// Get optionnal FILE_LIST object
	string FileListObjectName = CommandToExecute->Parameters.GetValue("FILE_LIST");
	CleanString(FileListObjectName);
	MakeUpper(FileListObjectName);


	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
		return false;
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	NvPackage Package;

	// Get header lines from package
	vector<std::string> HeaderLines;
	if(!Package.GetHeaderLines(FileObject->GetFilePathName().c_str(), &HeaderLines))
	{
		CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_CANNOT_INSTALL, Package.Info);
		return false;
	}

	// Prepare result object
	OutContainer->RemoveObject("PACKAGE_INFO");
	if(!OutContainer->CreateObject("PACKAGE_INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "PACKAGE_INFO");
		return false;
	}

	NvObjIndStringList *PackageInfo = (NvObjIndStringList *)(OutContainer->GetObject("PACKAGE_INFO"));

	int i;
	string::size_type index;
	for(i=0; i<(int)(HeaderLines.size()); i++)
	{
		string Line = HeaderLines[i];
		index = Line.find('=');
		if(index == string::npos)
			continue;	// Bad header entry
		string EntryName = Line.substr(0, index);
		string EntryValue = Line.substr(index+1);
		CleanString(EntryName);
		CleanString(EntryValue);
		MakeUpper(EntryName);
		if(!EntryName.length())
			continue;

		PackageInfo->SetValue(EntryName, EntryValue);
	}

	if(FileListObjectName.length())
	{
		OutContainer->RemoveObject(FileListObjectName.c_str());
		if(!OutContainer->CreateObject(FileListObjectName.c_str(), NV_OBJTYPE_STRINGLIST))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileListObjectName.c_str());
			return false;
		}

		NvObjStringList *FileList = (NvObjStringList *)(OutContainer->GetObject(FileListObjectName.c_str()));
		vector<std::string> Files;
		if(!Package.ListFiles(FileObject->GetFilePathName().c_str(), &Files))
		{
			CommandToExecute->Error.SetError("SYSTEM", "PACKAGE", NV_PACKAGE_CANNOT_INSTALL, Package.Info);
			return false;
		}
		FileList->m_vList = Files;
	}

	return true;
}



//SYSTEM LISTENER CREATE
bool NvSession::ListenerCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LISTENER:CREATE:NAME");
		return false;
	}

	// The real name is not cleaned
	Name = CommandToExecute->Parameters.GetValue("NAME");

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	string Proc = CommandToExecute->Parameters.GetValue("PROC");
	string Procf = CommandToExecute->Parameters.GetValue("PROCF");
	string User = CommandToExecute->Parameters.GetValue("USER");
	string OpenProc = CommandToExecute->Parameters.GetValue("OPEN");
	string CloseProc = CommandToExecute->Parameters.GetValue("CLOSE");
	string sNumThreads = CommandToExecute->Parameters.GetValue("THREADS");
	CleanString(sNumThreads);
	int iNumThreads = 1;
	if(sNumThreads.length())
		iNumThreads = atoi(sNumThreads.c_str());
	string sSleepTime = CommandToExecute->Parameters.GetValue("SLEEP");
	CleanString(sSleepTime);
	int iSleepTime = 100;
	if(sSleepTime.length())
		iSleepTime = atoi(sSleepTime.c_str());

	if(Application->Listeners.ListenerExist(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_EVER_EXIST, Name);
		return false;
	}

	if(!Application->Listeners.CreateListener(Name.c_str(), Description.c_str(), Proc.c_str(), Procf.c_str(), User.c_str(), OpenProc.c_str(), CloseProc.c_str(), iNumThreads, iSleepTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_CREATE, Name);
		return false;
	}
	return true;
}


//SYSTEM LISTENER REMOVE
bool NvSession::ListenerRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LISTENER:REMOVE:NAME");
		return false;
	}

	// The real name is not cleaned
	Name = CommandToExecute->Parameters.GetValue("NAME");

	if(Application->Listeners.IsListenerRunning(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_STOP_FIRST, Name);
		return false;
	}
	
	if(!Application->Listeners.RemoveListener(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_REMOVE, Name);
		return false;
	}

	return true;
}


//SYSTEM LISTENER START
bool NvSession::ListenerStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LISTENER:START:NAME");
		return false;
	}

	// The real name is not cleaned
	Name = CommandToExecute->Parameters.GetValue("NAME");

	if(!Application->Listeners.EnableListener(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_START, Name);
		return false;
	}
	if(!Application->Listeners.StartListener(Name.c_str()))
	{
		Application->Listeners.DisableListener(Name.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_START, Name);
		return false;
	}

	return true;
}


//SYSTEM LISTENER STOP
bool NvSession::ListenerStop(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LISTENER:STOP:NAME");
		return false;
	}

	// The real name is not cleaned
	Name = CommandToExecute->Parameters.GetValue("NAME");

	int iWaitTime = 10;
	string sWaitTime = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(sWaitTime);
	if(sWaitTime.length())
		iWaitTime = atoi(sWaitTime.c_str());
	if(iWaitTime < 0)
		iWaitTime = 0;

	bool bForce = false;
	string sForce = CommandToExecute->Parameters.GetValue("FORCE");
	CleanString(sForce);
	MakeUpper(sForce);
	if(sForce == "YES")
		bForce = true;

	if(!Application->Listeners.StopListener(Name.c_str(), bForce, iWaitTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_STOP, Name);
		return false;
	}
	Application->Listeners.DisableListener(Name.c_str());

	return true;
}


//SYSTEM LISTENER SET_PARAM
bool NvSession::ListenerSetParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LISTENER:SET_PARAM:NAME");
		return false;
	}


	// The real name is not cleaned
	Name = CommandToExecute->Parameters.GetValue("NAME");

	if(Application->Listeners.IsListenerRunning(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_STOP_FIRST, Name);
		return false;
	}

	const char *Description = NULL;
	string sDescription = "";
	if(CommandToExecute->Parameters.KeyExist("DESCRIPTION"))
	{
		sDescription = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		Description = sDescription.c_str();
	}

	const char *Proc = NULL;
	string sProc = "";
	if(CommandToExecute->Parameters.KeyExist("PROC"))
	{
		sProc = CommandToExecute->Parameters.GetValue("PROC");
		Proc = sProc.c_str();
	}

	const char *Procf = NULL;
	string sProcf = "";
	if(CommandToExecute->Parameters.KeyExist("PROCF"))
	{
		sProcf = CommandToExecute->Parameters.GetValue("PROCF");
		Procf = sProcf.c_str();
	}

	const char *User = NULL;
	string sUser = "";
	if(CommandToExecute->Parameters.KeyExist("USER"))
	{
		sUser = CommandToExecute->Parameters.GetValue("USER");
		User = sUser.c_str();
	}

	const char *OpenProc = NULL;
	string sOpenProc = "";
	if(CommandToExecute->Parameters.KeyExist("OPEN"))
	{
		sOpenProc = CommandToExecute->Parameters.GetValue("OPEN");
		OpenProc = sOpenProc.c_str();
	}

	const char *CloseProc = NULL;
	string sCloseProc = "";
	if(CommandToExecute->Parameters.KeyExist("CLOSE"))
	{
		sCloseProc = CommandToExecute->Parameters.GetValue("CLOSE");
		CloseProc = sCloseProc.c_str();
	}

	int iNumThreads = -1;
	string sNumThreads = "";
	if(CommandToExecute->Parameters.KeyExist("THREADS"))
	{
		sNumThreads = CommandToExecute->Parameters.GetValue("THREADS");
		CleanString(sNumThreads);
		iNumThreads = 1;
		if(sNumThreads.length())
			iNumThreads = atoi(sNumThreads.c_str());
	}

	int iSleepTime = -1;
	string sSleepTime = "";
	if(CommandToExecute->Parameters.KeyExist("SLEEP"))
	{
		sSleepTime = CommandToExecute->Parameters.GetValue("SLEEP");
		CleanString(sSleepTime);
		iSleepTime = 0;
		if(sSleepTime.length())
			iSleepTime = atoi(sSleepTime.c_str());
	}

	if(!Application->Listeners.SetListenerParam(Name.c_str(), Description, Proc, Procf, User, OpenProc, CloseProc, iNumThreads, iSleepTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_SET_PARAM, Name);
		return false;
	}
	return true;
}


//SYSTEM LISTENER LIST
bool NvSession::ListenerList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_ADMIN"))
		if(!CheckPermission(CommandToExecute, "SYSTEM", "LISTENER_LIST"))
			return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("LISTENER_LIST");
	if(!OutContainer->CreateObject("LISTENER_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "LISTENER_LIST");
		return false;
	}

	NvObjTable *ListenerInfo = (NvObjTable *)(OutContainer->GetObject("LISTENER_LIST"));

	string Name = CommandToExecute->Parameters.GetValue("NAME");

	if(!Application->Listeners.ListenerList(Name.c_str(), ListenerInfo))
	{
		OutContainer->RemoveObject("LISTENER_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "LISTENER", NV_LISTENER_CANNOT_GET_LIST, Name);
		return false;
	}
	return true;
}




//SYSTEM THREAD CREATE
bool NvSession::ThreadCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:THREAD:CREATE");
		return false;
	}

	string Proc = CommandToExecute->Parameters.GetValue("PROC");
	string Procf = CommandToExecute->Parameters.GetValue("PROCF");
	string User = CommandToExecute->Parameters.GetValue("USER");
	if(!User.length())
		User = GetUserName();
	string OpenProc = CommandToExecute->Parameters.GetValue("OPEN");
	string CloseProc = CommandToExecute->Parameters.GetValue("CLOSE");
	string Language = CommandToExecute->Parameters.GetValue("LANGUAGE");
	if(!Language.length())
		Language = CommandToExecute->m_sLanguage;

	string AppNameHere = GetAppName();

	NvSessionThread *SessionThread = new NvSessionThread;
	if(SessionThread == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "THREAD", NV_THREAD_CANNOT_CREATE);
		return false;
	}

	string ThreadSessionId = SessionThread->Start(m_sIdent.c_str(), AppNameHere.c_str(), Proc.c_str(), Procf.c_str(), User.c_str(), OpenProc.c_str(), CloseProc.c_str(), Language.c_str());
	if(!ThreadSessionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "THREAD", NV_THREAD_CANNOT_CREATE);
		delete SessionThread;
		return false;
	}
	
	GetStringValue(CommandToExecute, ThreadSessionId.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}
